      ******************************************************************
      * Analysis Tool for redocumenting Englich Text Docus
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. ENGREDOC.
      *AUTHOR.     H. SNEED.
      *DATE.       SEPT. 2009.
      *VERSION.    1.0 of Sept.2009
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *                  Text Audit Specifiaction                      *
      * Accept XML Parameter File from the WebClient                   *
      * Convert XML Parameters to Internal Parameters                  *
      * Init UseCase Report                                            *
      * Init Object Reference Report                                   *
      * Parse Text Document                                            *
      * Print Object Reference Report                                  *
      * Print UseCase Report                                           *
      * Close Reports                                                  *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *
           SELECT TEXT-FILE ASSIGN TO DYNAMIC INPUT-FILE
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS INPUT-STAT.
      *
           SELECT LOG-FILE ASSIGN TO DYNAMIC LOG-NAME
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS LOG-STAT.
XML   *
XML        SELECT PARAM-FILE
XML               ASSIGN TO DYNAMIC XML-FILE
XML               ORGANIZATION IS LINE SEQUENTIAL
XML               ACCESS IS SEQUENTIAL
XML               FILE STATUS IS XML-STAT.
XML   *
XML        SELECT RESPONSE-FILE
XML               ASSIGN TO DYNAMIC OUTPUT-FILE
XML               ORGANIZATION IS LINE SEQUENTIAL
XML               ACCESS IS SEQUENTIAL
XML               FILE STATUS IS OUTPUT-STAT.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
XML   *
XML    FD  PARAM-FILE
XML        RECORD CONTAINS 80 CHARACTERS
XML        DATA RECORD IS PARAM-REC.
XML    01  PARAM-REC.
XML        05  PARAM-LINE                    PIC X(80).
XML        05  PARAM-CHARS                   REDEFINES PARAM-LINE.
XML            10  PARAM-CHAR                PIC X  OCCURS 80 TIMES.
XML   *
XML    FD  RESPONSE-FILE
XML        RECORD CONTAINS 80 CHARACTERS
XML        DATA RECORD IS RESPONSE-REC.
XML    01  RESPONSE-REC.
XML        05  RESPONSE-LINE                 PIC X(80).
XML        05  RESPONSE-CHARS                REDEFINES RESPONSE-LINE.
XML            10  RESPONSE-CHAR             PIC X  OCCURS 80 TIMES.
      *
       FD  TEXT-FILE
           RECORD CONTAINS 160 CHARACTERS
           DATA RECORD IS TEXT-REC.
       01  TEXT-REC                         PIC X(160).
      *
       FD  LOG-FILE
           RECORD CONTAINS 72 CHARACTERS
           DATA RECORD IS LOG-REC.
       01  LOG-REC.
           05  FILLER                       PIC X(4).
           05  LOG-TEXT                     PIC X(64).
           05  FILLER                       PIC X(4).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
XML    77  XML-FILE                          PIC X(80).
       77  INPUT-FILE                        PIC X(80).
       77  OUTPUT-FILE                       PIC X(80).
       77  LOG-NAME                          PIC X(80).
       77  INPUT-FILE-NAME                   PIC X(80).
       77  FUNCTION-FILE-NAME                PIC X(80).
       77  MODEL-FILE-NAME                   PIC X(80).
XML    77  OUTPUT-STAT                       PIC XX.
       77  LOG-STAT                          PIC XX.
XML    77  XML-STAT                          PIC XX.
XML        88  XML-OK                        VALUE '00'.
XML        88  XML-EOF                       VALUE '09'.
       77  INPUT-STAT                        PIC XX.
           88  INPUT-OK                      VALUE '00'.
           88  INPUT-EOF                     VALUE '09'.
XML    77  ERROR-CODE                        PIC 99   VALUE ZERO.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
      *                          Parameter Keywords
           COPY XMLPARS.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 9999.
           05  J                             PIC 9999.
           05  K                             PIC 9999.
      ****************************************************************
      *
      *                          Pointer Fields
       01  POINTERS.
           05  TEXT-PTR                      PIC 9999.
           05  END-PTR                       PIC 9999.
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  LINE-LNG                      PIC 9999 VALUE 400.
           05  STRING-LNG                    PIC 9999 VALUE 120.
           05  LINE-MAX                      PIC 9999 VALUE 20.
           05  RULE-MAX                      PIC 9999 VALUE 80.
           05  FILE-MAX                      PIC 9999 VALUE 16.
           05  NAME-MAX                      PIC 9999 VALUE 8.
           05  LINE-LNG-MAX                  PIC 9999 VALUE 160.
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  FOUND-SW                      PIC 9    VALUE 0.
           05  END-SW                        PIC 9    VALUE 0.
           05  LIT-SW                        PIC 9    VALUE 0.
      *
      ****************************************************************
       01  VARIABLES.
           05  CHAR-COUNT                        PIC 9999.
           05  CODE-LINE                         PIC X(400).
           05  CODE-STRING           REDEFINES   CODE-LINE.
               10  CODE-CHAR                     PIC X OCCURS 400 TIMES.
           05  NAME-STRING.
               10  NAME-CHAR                     PIC X OCCURS 120 TIMES.
           05  SOURCE-STMT                       PIC X(80).
           05  SOURCE-TYPE                       PIC XXX.
           05  TEXT-TYPE                         PIC XXX.
           05  LAST-MODULE                       PIC X(8).
           05  LAST-COMPONENT                    PIC X(40).
           05  NR-TEXT-FILES                     PIC 9999 VALUE ZERO.
           05  NR-OBJECTS-READ                   PIC 9999 VALUE ZERO.
           05  MIN-TESTCASES                     PIC 9999 VALUE ZERO.
      *
      ****************************************************************
       01  MODEL-PARAMS.
           05  BASE-TYPE                         PIC X(4).
           05  BASE-ENTITY                       PIC X(40).
           05  TARGET-TYPE                       PIC X(4).
           05  TARGET-ENTITY                     PIC X(120).
      ****************************************************************
           COPY SFILPAR.
      *
      ****************************************************************
REPO       COPY CALCPARS.
      *
      ****************************************************************
REPO       COPY REPOPARM.
      *
      ****************************************************************
XML        COPY XMLMSGS.
      *
      ****************************************************************
      *  Data for processing XML Files                               *
      ****************************************************************
      *
XML        COPY XMLDATA.cpy.
      *
      ****************************************************************
      *LINKAGE SECTION.
      *                          Parameters
           COPY SYSPARS.
       77  TEXT-NAME                         PIC X(8).
SN0300 77  CHECK-OPTION                      PIC X.
SN0300 77  TEST-OPTION                       PIC X.
SN0300 77  SPEC-OPTION                       PIC X.
      *
      * Interface to Text Processing
      *
           COPY TEXTPARS.
      *
      * Interface to Function Table
      *
           COPY FUNCPARS.
      *
      ****************************************************************
      *
      *  Parameters to Operating System
      *
       01  CMD-LINE-STRING                   PIC X(80).
       01  RESULT                            PIC X  COMP-X.
       01  FUNCT                             PIC X  COMP-X VALUE 35.
       01  CMD-LINED.
           05  CMD-LINE-LNG                  PIC X  COMP-X VALUE ZERO.
      ******************************************************************
      *
      /
       PROCEDURE DIVISION.
      ******************************************************************
      *     Start routine     ******************************************
      ******************************************************************
       CSSANAL-START SECTION.
      ******************************************************************
      *
XML    SETUP-PARAMETERS.
      *
WRAP       ACCEPT CMD-LINE-STRING FROM COMMAND-LINE
WRAP       MOVE CMD-LINE-STRING TO XML-FILE.
TEST***    DISPLAY 'Enter ParamFile Name !'
TEST***    ACCEPT XML-FILE.
      *
           MOVE ZERO   TO ERROR-CODE.
           MOVE ZEROES TO XML-STAT.
           OPEN INPUT PARAM-FILE
           IF XML-STAT NOT = ZEROS
              DISPLAY '| Parameter File not found'
              DISPLAY '| Parameter File Name:' XML-FILE
              DISPLAY '| TestComp  could not execute !!!'
              DISPLAY '| Press enter'
              ACCEPT CONTINUE-KEY
              MOVE 1 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           END-IF.
      *
XML        PERFORM READ-PARAM-FILE.
REPO       MOVE 'Y' TO REPO-CODE.
      *
      *
           IF ERROR-CODE = ZERO
              DISPLAY '+----------------------------------------------+'
              DISPLAY '| ENGRedoc Text Analyzer Version 3.1 Oct. 2012 |'
              DISPLAY '| Copyright: Harry M. Sneed Budapest, Hungary  |'
              DISPLAY '+----------------------------------------------+'
              DISPLAY '| Product Name     = ' PRODUCT-NAME
              DISPLAY '| System Name      = ' SYSTEM-NAME
              DISPLAY '| Language Version = ' AUDIT-VERSION
              DISPLAY '| Repository Code  = ' REPO-CODE
              DISPLAY '| Input  Directory = ' INPUT-LIB-NAME
              DISPLAY '| Output Directory = ' OUTPUT-LIB-NAME
              DISPLAY '| Work   Directory = ' WORK-LIB-NAME
              DISPLAY '| Function Table   = ' FUNCTION-FILE-NAME
              DISPLAY '+-------------------------------------------+'
TEST***       DISPLAY '| Press Enter to start Analysis !!'
TEST***       ACCEPT CONTINUE-KEY
           ELSE
              MOVE 2 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           END-IF.
      *
      *****************************************************
       SET-SUBDIRECTORIES.
      ***********************
      * for Requirement specifications
DOS        MOVE SPACES TO SPEC-LIB-NAME
DOS        STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
DOS               '\REQUS'         DELIMITED BY SIZE
DOS           INTO REQU-LIB-NAME
DOS        END-STRING.
      *
      ***********************
      * for Object Cross References
DOS        MOVE SPACES TO SPEC-LIB-NAME
DOS        STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
DOS               '\SPECS'         DELIMITED BY SIZE
DOS           INTO SPEC-LIB-NAME
DOS        END-STRING.
      *
      ***********************
      * for Repository Files
DOS        MOVE SPACES TO EXPO-LIB-NAME
DOS        STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
DOS               '\EXPORTS'       DELIMITED BY SIZE
DOS           INTO EXPO-LIB-NAME
DOS        END-STRING.
      *
      *****************************************************
       OPEN-LOG-FILE.
           MOVE SPACES TO LOG-NAME
           STRING OUTPUT-LIB-NAME    DELIMITED BY SPACE
                 '\'                 DELIMITED BY SIZE
                 SYSTEM-NAME         DELIMITED BY SPACE
                 '.log'              DELIMITED BY SIZE
             INTO LOG-NAME
           END-STRING.
           MOVE ZEROES TO LOG-STAT.
           OPEN OUTPUT LOG-FILE.
           MOVE SPACES TO LOG-REC.
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT(1:1)
           WRITE LOG-REC.
           MOVE '| ENGRedoc Version 3.1 of Oct. 2012' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '| Copyright Harry M. Sneed ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '| of Soring Kft., Budapest, Hungary ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '| =======> Execution Protocol       ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT(1:1)
           WRITE LOG-REC.
           MOVE SPACES TO LOG-TEXT
           STRING '| Product Name = '         DELIMITED BY SIZE
                     PRODUCT-NAME             DELIMITED BY SIZE
              INTO LOG-TEXT
           WRITE LOG-REC.
           MOVE SPACES TO LOG-TEXT
           STRING '| System Name  = '         DELIMITED BY SIZE
                     SYSTEM-NAME              DELIMITED BY SIZE
              INTO LOG-TEXT
           WRITE LOG-REC.
           MOVE SPACES TO LOG-TEXT
           STRING '| Language     = '         DELIMITED BY SIZE
SN0107              'English'                 DELIMITED BY SIZE
              INTO LOG-TEXT
           WRITE LOG-REC.
      ***************************************************************
       INIT-COUNTERS.
           MOVE ZEROES TO INDEXES.
      ***************************************************************
       OPEN-FUNCTION-TABLE.
      * set up function file name
           MOVE ZERO TO FUNCTION-RETC.
           MOVE 'OPEN' TO FUNCTION-IO-CODE.
           MOVE SPACES TO FUNCTION-IO-STRING.
           CALL 'TXTFUNC' USING FUNCTION-IO-PARAMS, FUNCTION-FILE-NAME
           IF FUNCTION-RETC NOT = ZERO
              DISPLAY '| Function Table not found '
              DISPLAY '| Function File Name = ' FUNCTION-FILE-NAME
              MOVE    '| Function Table not found ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Function File Name = ' DELIMITED BY SIZE
                        FUNCTION-FILE-NAME     DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| Execution was terminated !' TO LOG-TEXT
              WRITE LOG-REC
              CLOSE LOG-FILE
TEST***       DISPLAY '| Press Enter to Terminate '
TEST***       ACCEPT  CONTINUE-KEY
              MOVE 3 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           ELSE
              DISPLAY '| Function Table for Txt was opened '
              DISPLAY '| Function Table = ' FUNCTION-FILE-NAME
              DISPLAY '| Nr-Objects     = ' FUNCTION-IO-LNG
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
AUG06      MOVE FUNCTION-IO-LNG TO NR-OBJECTS-READ.
      ***************************************************************
       OPEN-FUN-MODEL.
      * set up function file name
           MOVE PRODUCT-NAME TO EXPORT-PRODUCT-NAME
           MOVE VERSION-NAME TO EXPORT-VERSION-NAME
           MOVE SYSTEM-NAME  TO EXPORT-SYSTEM-NAME
           MOVE 'GERRedoc'   TO EXPORTEUR
           MOVE 'German  '   TO ENTITY-LANG
           MOVE 'Model   '   TO ENTITY-TYPE
           MOVE 'OPEN' TO FUNCTION-IO-CODE.
           MOVE ZERO TO FUNCTION-RETC.
           MOVE SPACES TO MODEL-FILE-NAME
           STRING SPEC-LIB-NAME         DELIMITED BY SPACE
                  '\FunModel.xml'       DELIMITED BY SIZE
              INTO MODEL-FILE-NAME
           END-STRING.
           MOVE SPACES TO TARGET-TYPE
           MOVE SPACES TO TARGET-ENTITY
           CALL 'FUNMODEL' USING FUNCTION-IO-CODE, FUNCTION-RETC,
                           MODEL-FILE-NAME, EXPORT-DATA,
                           TARGET-TYPE, TARGET-ENTITY.
           IF FUNCTION-RETC NOT = ZERO
              DISPLAY '| XML Model File could not be opened! '
              DISPLAY '| Model File Name = ' MODEL-FILE-NAME
              MOVE    '| Model File could not be found ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| Function Model will be skipped!' TO LOG-TEXT
              WRITE LOG-REC
           ELSE
              DISPLAY '| Function Model File was opened '
              DISPLAY '| Function Model is in: ' MODEL-FILE-NAME
              MOVE    '| Model File was opened! ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Function Model is in: ' DELIMITED BY SIZE
                      MODEL-FILE-NAME            DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
      ***************************************************************
       OPEN-DAT-MODEL.
      * set up function file name
           MOVE PRODUCT-NAME TO EXPORT-PRODUCT-NAME
           MOVE VERSION-NAME TO EXPORT-VERSION-NAME
           MOVE SYSTEM-NAME  TO EXPORT-SYSTEM-NAME
           MOVE 'GERRedoc'   TO EXPORTEUR
           MOVE 'German  '   TO ENTITY-LANG
           MOVE 'Model   '   TO ENTITY-TYPE
           MOVE 'OPEN' TO FUNCTION-IO-CODE.
           MOVE ZERO TO FUNCTION-RETC.
           MOVE SPACES TO MODEL-FILE-NAME
           STRING SPEC-LIB-NAME         DELIMITED BY SPACE
                  '\DatModel.xml'       DELIMITED BY SIZE
              INTO MODEL-FILE-NAME
           END-STRING.
           MOVE SPACES TO TARGET-TYPE
           MOVE SPACES TO TARGET-ENTITY
           CALL 'DATMODEL' USING FUNCTION-IO-CODE, FUNCTION-RETC,
                           MODEL-FILE-NAME, EXPORT-DATA,
                           BASE-TYPE, BASE-ENTITY,
                           TARGET-TYPE, TARGET-ENTITY.
           IF FUNCTION-RETC NOT = ZERO
              DISPLAY '| XML Model File could not be opened! '
              DISPLAY '| Model File Name = ' MODEL-FILE-NAME
              MOVE    '| Model File could not be found ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| Data Model will be skipped!' TO LOG-TEXT
              WRITE LOG-REC
           ELSE
              DISPLAY '| Data Model File was opened '
              DISPLAY '| Data Model is in: ' MODEL-FILE-NAME
              MOVE    '| Model File was opened! ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Data Model is in: ' DELIMITED BY SIZE
                      MODEL-FILE-NAME               DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
      ***************************************************************
      ***************************************************************
       PROCESS-ALL-FILES.
XML   *
XML   ** Read all Source File Names from XML File
XML   *
           MOVE SPACES TO LAST-MODULE.
XML        PERFORM UNTIL XML-LABEL = FILES-END
XML                   OR XML-EOF
XML           READ PARAM-FILE
XML           AT END
XML              MOVE '09' TO XML-STAT
XML           NOT AT END
XML              MOVE PARAM-REC TO XML-LINE
XML              PERFORM PARSE-XML
XML              IF ERROR-CODE > ZERO
XML                 MOVE 2 TO ERROR-CODE
XML                 PERFORM RETURN-RESPONSE
XML              END-IF
XML           END-READ
XML           IF XML-STAT = ZEROES AND
XML              XML-LABEL (1:9) = ORIG-FILE
XML              MOVE XML-VALUE TO PATH-NAME
XML              DISPLAY 'Orig Text File = ' PATH-NAME
TEST***          ACCEPT CONTINUE-KEY
XML              PERFORM GET-NAMES
LOG              MOVE SPACES TO LOG-TEXT
LOG              STRING '| Next Text File = '   DELIMITED BY SIZE
LOG                       PATH-NAME             DELIMITED BY SPACE
LOG                 INTO LOG-TEXT
LOG              WRITE LOG-REC
XML           END-IF
XML           IF XML-STAT = ZEROES AND
XML              XML-LABEL (1:5) = FILENR
XML              MOVE SPACES TO INPUT-FILE
XML              STRING
XML                     INPUT-LIB-NAME   DELIMITED BY SPACE
XML                    '\'               DELIMITED BY SIZE
XML                     XML-VALUE        DELIMITED BY SPACE
XML                 INTO INPUT-FILE
XML              END-STRING
XML              DISPLAY '| Next Text File = ' INPUT-FILE
TEST***          ACCEPT CONTINUE-KEY
XML   *************************************************************
                 PERFORM GET-MODULE-NAME
                 MOVE SOURCE-NAME TO TEXT-NAME
                 MOVE SOURCE-TYPE TO TEXT-TYPE
      *************************************************************
      **      Group Change for Module
      *************************************************************
                 IF MODULE-NAME NOT = LAST-MODULE
REPO                PERFORM OPEN-REPO-FILE
                 END-IF
                 MOVE MODULE-NAME TO LAST-MODULE
                 MOVE COMPONENT-NAME TO LAST-COMPONENT
LOG              MOVE SPACES TO LOG-TEXT
LOG              STRING '| Current Subsystem = '    DELIMITED BY SIZE
LOG                       LAST-COMPONENT            DELIMITED BY SPACE
LOG                 INTO LOG-TEXT
LOG              WRITE LOG-REC
      *************************************************************
      **      Read and Process next Text Member
      *************************************************************
REPO             PERFORM INSERT-REPO-MEMBER
                 PERFORM CALL-TXTPARSE
                 ADD 1 TO NR-TEXT-FILES
              END-IF
           END-PERFORM.
           CLOSE PARAM-FILE.
      **********************************************************
       END-TXTREDOC.
           MOVE 'CLOS' TO FUNCTION-IO-CODE.
           MOVE ZERO TO FUNCTION-RETC.
           MOVE SPACES TO TARGET-TYPE
           MOVE SPACES TO TARGET-ENTITY
           CALL 'FUNMODEL' USING FUNCTION-IO-CODE, FUNCTION-RETC,
                           MODEL-FILE-NAME, EXPORT-DATA,
                           TARGET-TYPE, TARGET-ENTITY.
REPO       PERFORM CLOSE-REPO-FILE.
           DISPLAY '+-------------------------------------------+'
           DISPLAY '| ENGREDOC English Text Analysis completed  |'
           DISPLAY '+-------------------------------------------+'
           DISPLAY '| Texts processed = ' NR-TEXT-FILES
LOG        MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
LOG        WRITE LOG-REC.
           IF RETCODE > ZERO
              MOVE '|ENGREDOC unsuccessfully terminated!' TO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '|Error Code was: '          DELIMITED BY SIZE
                       RETCODE                     DELIMITED BY SIZE
                INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE '|see Help function to diagnose cause of problem'
                TO LOG-TEXT
LOG           WRITE LOG-REC
           ELSE
              MOVE '| ENGREDOC successfully completed!' TO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text Files processed = '   DELIMITED BY SIZE
                      NR-TEXT-FILES                DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text Lines processed = '   DELIMITED BY SIZE
                        TEXT-LOCS                  DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text Stmts processed = '   DELIMITED BY SIZE
                        TEXT-STMTS                 DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text Titles processed = '   DELIMITED BY SIZE
                        TEXT-TITLES                 DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text Reques processed = '   DELIMITED BY SIZE
                        TEXT-REQUS                  DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text UseCases processed = ' DELIMITED BY SIZE
                        TEXT-CASES                  DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text Objects processed = '   DELIMITED BY SIZE
                        TEXT-OBJECTS                 DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text Rules processed = '   DELIMITED BY SIZE
                        TEXT-RULES                 DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text States processed = '  DELIMITED BY SIZE
                        TEXT-STATES                DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Text Actions processed = '  DELIMITED BY SIZE
                        TEXT-ACTIONS                DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING '| Requirements processed = '   DELIMITED BY SIZE
                        TEXT-REQUS                   DELIMITED BY SIZE
                 INTO LOG-TEXT
LOG           WRITE LOG-REC
           END-IF.
           MOVE
           '| Text Specification is to be found in Subdirectory SPECS'
             TO LOG-TEXT
LOG        WRITE LOG-REC
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
LOG        WRITE LOG-REC.
           CLOSE LOG-FILE.
TEST***    DISPLAY '| Press Enter to Terminate!'
TEST***    ACCEPT  CONTINUE-KEY
     **       return Reponse & terminate analysis
           PERFORM RETURN-RESPONSE.
           GOBACK.
      ******************************************************************
      ******************************************************************
      *      Main Loop for each Text Member           ******************
      ******************************************************************
       PROCESS-TEXT SECTION.
      ***************************************************************
       OPEN-INPUT-FILE.
           MOVE ZEROES TO INPUT-STAT.
           OPEN INPUT TEXT-FILE.
           IF INPUT-STAT NOT = ZEROES
              DISPLAY '| Text File cannot be opened' INPUT-STAT
              DISPLAY '| Text File Path = ' INPUT-FILE
              DISPLAY '| Analysis cannot be continued !!!'
              MOVE    '| Text File not opened' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Text File Name = ' DELIMITED BY SIZE
                        INPUT-FILE          DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| File can not be processed!' TO LOG-TEXT
              WRITE LOG-REC
TEST***       DISPLAY '| Press Enter to Continue!'
TEST***       ACCEPT  CONTINUE-KEY
TEST          MOVE 5 TO ERROR-CODE
TEST          PERFORM RETURN-RESPONSE
TEST          GOBACK
           ELSE
              DISPLAY '+--------------------------------------+'
              DISPLAY '| Text File to be analyzed = ' TEXT-NAME
              DISPLAY '+--------------------------------------+'
              MOVE SPACES TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Text File analyzed = ' DELIMITED BY SIZE
                        TEXT-NAME               DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
REPO          PERFORM INSERT-REPO-MEMBER
              CLOSE TEXT-FILE
           END-IF.
      **************************************************************
       CALL-TXTPARSE.
      * Call TXTPARSE to analyze Text
           MOVE INPUT-FILE TO INPUT-FILE-NAME,
SN1008     MOVE SRC-FILE-NAME TO TEXT-VORGANG
           MOVE SYSTEM-NAME TO TEXT-MOD-NAME
           MOVE PRODUCT-NAME TO TEXT-PRODUCT
           MOVE TEXT-NAME TO TEXT-FILE-NAME
           MOVE TEXT-TYPE TO TEXT-FILE-EXT
           MOVE ZEROES    TO TEXT-STATISTICS.
TEST***    DISPLAY 'Call to TXTPARSE'
TEST***    ACCEPT CONTINUE-KEY
           CALL 'ENGPARSE' USING TEXT-FILE-PARAMS,
                                 FILE-TAB,
                                 INPUT-FILE-NAME,
                                 REQU-LIB-NAME,
                                 WORK-LIB-NAME,
                                 REPO-CODE,
                                 RETCODE.
TEST***    DISPLAY 'Return from TXTPARSE Retcode = ' RETCODE
TEST***    ACCEPT CONTINUE-KEY.
           IF RETCODE = ZERO
              CALL 'TXTSPEC' USING PRODUCT-NAME,
                                   SYSTEM-NAME,
                                   TEXT-MOD-NAME,
                                   COMPONENT-NAME,
                                   SPEC-LIB-NAME,
                                   WORK-LIB-NAME,
                                   RETCODE
LOG           MOVE '| Object Usage has been specified ' TO LOG-TEXT
LOG           WRITE LOG-REC
           END-IF.
           EXIT.
      *
      ******************************************************************
      ******************************************************************
      *    Extract Names from Paths
           COPY NAMEXTR.
           EXIT.
      ******************************************************************
      *    REPO Initialisation
           COPY REPOINI.
           EXIT.
      ******************************************************************
      *    XML Initialisation
           COPY XMLINIT.
           EXIT.
      ********************** end of source *****************************
