       IDENTIFICATION DIVISION.
       PROGRAM-ID. REPOREF.
       ENVIRONMENT DIVISION.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      ***************************************************************
      *  Reference file with a record for each repository entry
      *  Entries are:
      *    Prodsys = 1:n relation to a product        (SYST)
      *    SysAct  = 1:n relation to actors           (REQU)
      *    SysRequ = 1:n relation to requirements     (REQU)
      *    SysSect = 1:n relations to sections        (SECT)
      *    SysCase = 1:n relations to use cases       (CASE)
      *    CasCase = 1:n relations to trigger         (TRIG)
      *    CasStep = 1:n relations to steps           (STEP)
      *    SysData = 1:n relations to data objects    (DATA)
      *    SysPred = 1:n relations to data predicates (PRED)
      *    SysRepo = 1:n relations to reports         (REPO)
      *    SysGUI  = 1:n relations to GUIs            (GUI)
      *    SysMess = 1:n relations to messages        (MESS)
      *    SysCase = 1:n relations to test cases      (TEST)
      ***************************************************************
           SELECT REPO-REF-FILE
                  ASSIGN TO DYNAMIC REF-FILENAME
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS REF-FSTS.
      *
           SELECT CALC-EXP-FILE
                  ASSIGN TO DYNAMIC EXP-FILENAME
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS EXP-FSTS.
      *
      ******************************************************************
      *
       DATA DIVISION.
      *
       FILE SECTION.
      *
      ******************************************************************
      *
       FD  REPO-REF-FILE
           RECORD CONTAINS 112 CHARACTERS
           DATA RECORD IS REPO-REF-REC.
       01  REPO-REF-REC.
           02 REPO-REF-TYPE         PIC X(4).
           02 REPO-REF-SEP-1        PIC X.
           02 REPO-REF-BASE         PIC X(36).
           02 REPO-REF-SEP-2        PIC X.
           02 REPO-REF-REL-TYPE     PIC X(4).
           02 REPO-REF-SEP-3        PIC X.
           02 REPO-REF-TARGET-TYPE  PIC X(4).
           02 REPO-REF-SEP-4        PIC X.
           02 REPO-REF-TARGET       PIC X(60).
      *
       FD  CALC-EXP-FILE
           RECORD CONTAINS 80 CHARACTERS
           DATA RECORD IS CALC-REC.
       01  CALC-REC                 PIC X(80).
      *
      ******************************************************************
      *
       WORKING-STORAGE SECTION.
      *                       File indicators for the dynamic assign
       77  REF-FILENAME             PIC X(80).
       77  EXP-FILENAME             PIC X(80).
       77  REF-FSTS                 PIC XX.
       77  EXP-FSTS                 PIC XX.
       77  CONTINUE-KEY             PIC X.
      *
       01  SWITCHES.
           05 OPEN-SW               PIC 9 VALUE ZERO.
           05 FOUND-SW              PIC 9 VALUE ZERO.
      *
       01  ERROR-RETCODE-VALUES.
      *           Type error
           02 TYPE-ERROR            PIC 99 VALUE 80.
      *           Open error
           02 OPEN-ERROR            PIC 99 VALUE 81.
      *          Key error on direct access file
           02 KEY-ERROR             PIC 99 VALUE 82.
      *
       01  CALC-EXPORT-VALUES.
      *
           05  DISPLAY-VALUE            PIC Z9     VALUE ZERO.
           05  CURRENT-VALUE            PIC 99     VALUE ZEROES.
           05  CURRENT-TAG              PIC X(20)  VALUE SPACES.
           05  CURRENT-ATTR             PIC X(40)  VALUE SPACES.
           05  XML-LINE             PIC X(72) VALUE
              '<?xml version="1.0" encoding="UTF-8" ?>'.
           05  DOC-LINE             PIC X(72) VALUE
              '<!--DOCTYPE TESTCALC SYSTEM "testcalc.xsd"-->'.
           05  YYMMDD.
               10  YY                   PIC 99.
               10  MM                   PIC 99.
               10  DD                   PIC 99.
           05  DATUM.
               10  TAG                  PIC 99.
               10  FILLER               PIC X   VALUE '.'.
               10  MON                  PIC 99.
               10  FILLER               PIC X   VALUE '.'.
               10  JAHR                 PIC 99.
      *
       01  METRIK-EXPORT-TAGS.
           05  FILE-TAGS.
               10  METRIC-TAG       PIC X(20) VALUE 'EXPORT_FILE'.
               10  PRODUCT-ATTR     PIC X(20) VALUE 'Product'.
               10  SYSTEM-ATTR      PIC X(20) VALUE 'System '.
               10  SENDER-ATTR      PIC X(20) VALUE 'Source '.
               10  DATE-ATTR        PIC X(20) VALUE 'Date   '.
           05  ENTITY-TAGS.
               10  ENTITY-TAG       PIC X(20) VALUE 'TEST_ENTITY'.
               10  ENTITY-ATTR      PIC X(20) VALUE 'Entity'.
               10  NAME-ATTR        PIC X(20) VALUE 'EName'.
               10  TYPE-ATTR        PIC X(20) VALUE 'EType'.
           05  STRUCTURE-TAGS.
               10  USECASE-TAG   PIC X(20) VALUE 'UseCase_Attributes'.
               10  OBJECT-TAG    PIC X(20) VALUE 'Object_Attributes'.
               10  INTERFAC-TAG  PIC X(20) VALUE 'Interface_Attributes'.
               10  TESTCASE-TAG  PIC X(20) VALUE 'TestCase_Attributes'.
               10  CHANGE-TAG       PIC X(20) VALUE 'Change_Rate'.
           05  USECASE-TAGS.
               10  PATH-TAG         PIC X(20) VALUE 'Nr_Paths'.
               10  STEP-TAG         PIC X(20) VALUE 'Nr_Steps'.
               10  INPUT-TAG        PIC X(20) VALUE 'Nr_Inputs'.
               10  OUTPUT-TAG       PIC X(20) VALUE 'Nr_Outputs'.
           05  OBJECT-TAGS.
               10  RELATION-TAG     PIC X(20) VALUE 'Nr_Relations'.
               10  METHOD-TAG       PIC X(20) VALUE 'Nr_Methods'.
               10  KEY-TAG          PIC X(20) VALUE 'Nr_Keys'.
               10  ATTR-TAG         PIC X(20) VALUE 'Nr_Attributes'.
           05  INTERFACE-TAGS.
               10  TARGET-TAG       PIC X(20) VALUE 'Nr_Targets'.
               10  OPERATION-TAG    PIC X(20) VALUE 'Nr_Operations'.
               10  ARGUMENT-TAG     PIC X(20) VALUE 'Nr_Arguments'.
               10  RESULT-TAG       PIC X(20) VALUE 'Nr_Results'.
      *
      ******************************************************************
      *
       LINKAGE SECTION.
      *
      ******************************************************************
      *
           COPY REPOPARM.
      *
      ******************************************************************
      *
       PROCEDURE DIVISION USING REPO-NAME,
                                EXTERNAL-REF,
                                OUTPUT-DIR,
                                RETCODE.
      *
      ******************************************************************
      *
       REPO-OPENING SECTION.
      *
       OPEN-REPOREF-FILE.
      *
      *    DISPLAY 'REF-BASE-TYPE = ' REF-BASE-TYPE
      *    DISPLAY 'REF-BASE-NAME = ' REF-BASE-NAME
      *    DISPLAY 'REF-TARGET-TYPE = ' REF-TARGET-TYPE
      *    DISPLAY 'REF-TARGET-NAME = ' REF-TARGET-NAME
      *    ACCEPT CONTINUE-KEY
           MOVE ZERO TO RETCODE.
           IF REF-BASE-TYPE = 'PROD'
              IF OPEN-SW = 1
*****************CLOSE REPO-REF-FILE
                 CLOSE CALC-EXP-FILE
                 MOVE ZERO TO OPEN-SW
              END-IF
              MOVE SPACES TO REF-FILENAME
              STRING OUTPUT-DIR      DELIMITED BY SPACE
SN1213               '\Exports\'     DELIMITED BY SIZE
                     REPO-NAME       DELIMITED BY SPACE
                     '.csv'          DELIMITED BY SIZE
                 INTO REF-FILENAME
              END-STRING
              MOVE ZEROS TO REF-FSTS
              OPEN OUTPUT REPO-REF-FILE
              IF REF-FSTS  = ZEROES
                 MOVE SPACES TO REPO-REF-REC
                 MOVE 'Type' TO REPO-REF-TYPE
                 MOVE 'Base Entity' TO REPO-REF-BASE
                 MOVE 'Rel ' TO REPO-REF-REL-TYPE
                 MOVE 'Type' TO REPO-REF-TARGET-TYPE
                 MOVE 'Target Entity' TO REPO-REF-TARGET
                 MOVE ';' TO REPO-REF-SEP-1
                 MOVE ';' TO REPO-REF-SEP-2
                 MOVE ';' TO REPO-REF-SEP-3
                 MOVE ';' TO REPO-REF-SEP-4
                 WRITE REPO-REF-REC
                 MOVE REF-BASE-TYPE   TO  REPO-REF-TYPE
                 MOVE REF-BASE-NAME   TO  REPO-REF-BASE
                 MOVE REF-RELATION    TO  REPO-REF-REL-TYPE
                 MOVE REF-TARGET-TYPE TO  REPO-REF-TARGET-TYPE
                 MOVE REF-TARGET-NAME TO  REPO-REF-TARGET
                 WRITE REPO-REF-REC
                 MOVE ZERO TO RETCODE
                 MOVE 1    TO OPEN-SW
TEST             DISPLAY 'Repo Ref File was opened:' REF-FILENAME
TEST***          ACCEPT CONTINUE-KEY
              ELSE
                 MOVE OPEN-ERROR TO RETCODE
TEST             DISPLAY 'Open Error = ' REF-FSTS
TEST             DISPLAY 'Repo Ref File:' REF-FILENAME
TEST             DISPLAY 'Press Enter to Continue !!'
TEST***          ACCEPT CONTINUE-KEY
              END-IF
      ******************************************************************
      *    Open and init Calc Export File                              *
              MOVE SPACES TO EXP-FILENAME
              STRING OUTPUT-DIR      DELIMITED BY SPACE
SN1213               '\SoftCalc\'    DELIMITED BY SIZE
                     REPO-NAME       DELIMITED BY SPACE
                     '.xml'          DELIMITED BY SIZE
                 INTO EXP-FILENAME
              END-STRING
              MOVE ZEROS TO EXP-FSTS
              OPEN OUTPUT CALC-EXP-FILE
              IF EXP-FSTS  = ZEROES
TEST             DISPLAY 'Calc Import File was opened:' EXP-FILENAME
TEST***          ACCEPT CONTINUE-KEY
                 PERFORM WRITE-CALC-FILE-HEADER
              ELSE
                 MOVE OPEN-ERROR TO RETCODE
TEST             DISPLAY 'Open Error = ' EXP-FSTS
TEST             DISPLAY 'Calc Import File:' EXP-FILENAME
TEST             DISPLAY 'Press Enter to Continue !!'
TEST             ACCEPT CONTINUE-KEY
              END-IF
              GOBACK
           END-IF.
      *
       CLOSE-REPOREF-FILE.
      *
           IF REF-BASE-TYPE = 'CLOS'
              CLOSE REPO-REF-FILE
TEST          DISPLAY 'Repo Ref File was closed:' REF-FILENAME
TEST***       ACCEPT CONTINUE-KEY
              MOVE 'TESTCALC' TO CURRENT-TAG
              PERFORM WRITE-FIRST-LEVEL-END
              CLOSE CALC-EXP-FILE
TEST          DISPLAY 'Calc Import File was closed:' EXP-FILENAME
TEST***       ACCEPT CONTINUE-KEY
              MOVE ZERO TO RETCODE
              GOBACK
           END-IF.
      *
      ******************************************************************
      *
       REPO-PROCESSING SECTION.
      *
       CHECK-REF-TYPE.
      *
           EVALUATE REF-BASE-TYPE
           WHEN 'PROD'
           WHEN 'SYST'
           WHEN 'DOCU'
           WHEN 'MOD '
           WHEN 'SECT'
           WHEN 'CASE'
           WHEN 'REQU'
           WHEN 'INTR'
           WHEN 'FUNC'
           WHEN 'STRU'
           WHEN 'DATA'
           WHEN 'PARM'
           WHEN 'SPEC'
           WHEN 'PROC'
           WHEN 'TEST'
              CONTINUE
           WHEN OTHER
              MOVE TYPE-ERROR TO RETCODE
TEST          DISPLAY 'Base Type Error = ' REF-BASE-TYPE
TEST          DISPLAY 'Press Enter to Continue !!'
TEST          ACCEPT CONTINUE-KEY
              GOBACK
           END-EVALUATE.
      *
       CHECK-REL-TYPE.
      *
           EVALUATE REF-TARGET-TYPE
           WHEN 'LIB '
           WHEN 'SYST'
           WHEN 'DOCU'
           WHEN 'MOD '
           WHEN 'SECT'
           WHEN 'TEXT'
           WHEN 'CASE'
           WHEN 'REQU'
           WHEN 'DATA'
           WHEN 'REPO'
           WHEN 'MASK'
           WHEN 'INPT'
           WHEN 'OUTP'
           WHEN 'RULE'
           WHEN 'ACT '
           WHEN 'TRIG'
           WHEN 'STEP'
           WHEN 'PROC'
           WHEN 'SPEC'
           WHEN 'FUNC'
           WHEN 'TEST'
              CONTINUE
           WHEN OTHER
              MOVE TYPE-ERROR TO RETCODE
TEST          DISPLAY 'Target Type Error = ' REF-TARGET-TYPE
TEST          DISPLAY 'Press Enter to Continue !!'
TEST          ACCEPT CONTINUE-KEY
              GOBACK
           END-EVALUATE.
      *
       INSERT-REPO-REFERENCE.
      *
           IF  REF-BASE-TYPE = REPO-REF-TYPE
           AND REF-BASE-NAME = REPO-REF-BASE
           AND REF-RELATION  = REPO-REF-REL-TYPE
           AND REF-TARGET-TYPE = REPO-REF-TARGET-TYPE
           AND REF-TARGET-NAME = REPO-REF-TARGET
              CONTINUE
           ELSE
              MOVE SPACES TO REPO-REF-REC
              MOVE ';' TO REPO-REF-SEP-1
              MOVE ';' TO REPO-REF-SEP-2
              MOVE ';' TO REPO-REF-SEP-3
              MOVE ';' TO REPO-REF-SEP-4
              MOVE REF-BASE-TYPE   TO  REPO-REF-TYPE
              MOVE REF-BASE-NAME   TO  REPO-REF-BASE
              MOVE REF-RELATION    TO  REPO-REF-REL-TYPE
              MOVE REF-TARGET-TYPE TO  REPO-REF-TARGET-TYPE
              MOVE REF-TARGET-NAME TO  REPO-REF-TARGET
              WRITE REPO-REF-REC
           END-IF.
           IF REF-RELATION = 'OWNS'
           AND REF-TARGET-TYPE = 'TEST'
              PERFORM WRITE-CALC-TESTCASE
           END-IF.
           MOVE ZERO TO RETCODE.
TEST***    DISPLAY 'RepoRef-Rec  written'
TEST***    DISPLAY 'RepoRef-Type ' REF-TARGET-TYPE
TEST***    DISPLAY 'RepoRef-Name ' REF-TARGET-NAME
TEST***    ACCEPT CONTINUE-KEY.
       RETURN-TO-TEXTANAL.
           GOBACK.
      *
      ******************************************************************
      *
       CALC-PROCESSING SECTION.
      ************************
      *
       WRITE-CALC-FILE-HEADER.
      *  Create Metric Header
      *
           MOVE XML-LINE TO CALC-REC
           WRITE CALC-REC
           MOVE DOC-LINE TO CALC-REC
           WRITE CALC-REC
           MOVE 'TESTCALC' TO CURRENT-TAG.
           PERFORM WRITE-FIRST-LEVEL-START.
           MOVE METRIC-TAG TO CURRENT-TAG.
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE PRODUCT-ATTR TO CURRENT-TAG
           MOVE REPO-REF-BASE TO CURRENT-ATTR
           PERFORM WRITE-THIRD-LEVEL-ATTR.
           MOVE SYSTEM-ATTR TO CURRENT-TAG
           MOVE REPO-REF-TARGET TO CURRENT-ATTR
           PERFORM WRITE-THIRD-LEVEL-ATTR.
           ACCEPT YYMMDD FROM DATE
           MOVE YY TO JAHR
           MOVE MM TO MON
           MOVE DD TO TAG
           MOVE DATE-ATTR TO CURRENT-TAG
           MOVE DATUM TO CURRENT-ATTR
           PERFORM WRITE-THIRD-LEVEL-ATTR.
           MOVE SENDER-ATTR TO CURRENT-TAG
           MOVE 'TESTANAL' TO CURRENT-ATTR
           PERFORM WRITE-THIRD-LEVEL-ATTR.
           MOVE METRIC-TAG TO CURRENT-TAG.
           PERFORM WRITE-FILE-HEADER-END.
      *
      ******************************************************************
       WRITE-CALC-TESTCASE.
      *                               write interface attributes
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE ENTITY-ATTR TO CURRENT-TAG
           MOVE 'TESTCASE' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE NAME-ATTR   TO CURRENT-TAG
           MOVE REF-TARGET-NAME TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE TYPE-ATTR   TO CURRENT-TAG
           MOVE REF-BASE-TYPE TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE TESTCASE-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-START.
           MOVE ARGUMENT-TAG TO CURRENT-TAG
           MOVE 04 TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE RESULT-TAG TO CURRENT-TAG
           MOVE 02 TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE PATH-TAG TO CURRENT-TAG
           MOVE 01 TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE STEP-TAG TO CURRENT-TAG
           MOVE 03 TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE CHANGE-TAG TO CURRENT-TAG
           MOVE 00 TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE TESTCASE-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-END
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-END.
           EXIT.
      *
      ****************************************************************
       WRITE-XML SECTION.
      ************************************************ 1st Level
       WRITE-FIRST-LEVEL-START.
           MOVE SPACES TO CALC-REC
           STRING ' <'              DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC
           WRITE CALC-REC.
      ************************************************ 1st Level
       WRITE-FIRST-LEVEL-END.
           MOVE SPACES TO CALC-REC.
           STRING ' </'             DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-SECOND-LEVEL-START.
           MOVE SPACES TO CALC-REC.
           STRING '   <'            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-SECOND-LEVEL-ATTR.
           MOVE SPACES TO CALC-REC.
           STRING '    '            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  ' = "'            DELIMITED BY SIZE
                   CURRENT-ATTR     DELIMITED BY SPACE
                  '"'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE SPACES TO CALC-REC.
           STRING '    '            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  ' = "'            DELIMITED BY SIZE
                   CURRENT-ATTR     DELIMITED BY SPACE
                  '" >'             DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-SECOND-LEVEL-END.
           MOVE SPACES TO CALC-REC.
           STRING '   </'           DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-FILE-HEADER-END.
           MOVE SPACES TO CALC-REC.
           MOVE   '   />'  TO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 3rd Level
       WRITE-THIRD-LEVEL-START.
           MOVE SPACES TO CALC-REC.
           STRING '     <'          DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 3rd Level
       WRITE-THIRD-LEVEL-ATTR.
           MOVE SPACES TO CALC-REC.
           STRING '    '            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  ' = "'            DELIMITED BY SIZE
                   CURRENT-ATTR     DELIMITED BY SPACE
                  '"'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 3rd Level
       WRITE-THIRD-LEVEL-LAST-ATTR.
           MOVE SPACES TO CALC-REC.
           STRING '    '            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  ' = "'            DELIMITED BY SIZE
                   CURRENT-ATTR     DELIMITED BY SPACE
                  '" >'             DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 3rd Level
       WRITE-THIRD-LEVEL-END.
           MOVE SPACES TO CALC-REC.
           STRING '     </'         DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 5th Level
       WRITE-FIFTH-LEVEL-DATA.
           MOVE SPACES TO CALC-REC.
           STRING '         <'      DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
                   CURRENT-VALUE    DELIMITED BY SIZE
                  '</'              DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
           EXIT.
      *
      ******************************************************************
      *                      end of REPOREF
      ******************************************************************
