      ******************************************************************
      *   PROGRAM  FOR  PROCESSING REQUIREMENTS                        *
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. REQSPEC.
      *AUTHOR.     H. SNEED.
      *DATE.       MARCH 2006.
      *VERS.       1.0 of Mar. 2006
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *                  REQU DOCUMENTATION SPECIFICATION              *
      * Open DB-Testcases                                              *
      * Initialize Requ Report                                         *
      * Open Header File                                               *
      * Do for each Header                                             *
      *    Read Header                                                 *
      *    If Header = Require                                         *
      *       Call TCSIOMOD                                            *
      *       if Requirement has TestCase                              *
      *          Print Requirement to TestCase Link                    *
      *       else                                                     *
      *          Print Requirement has no TestCase                     *
      *       end                                                      *
      *    end                                                         *
      * endDo                                                          *
      * Close Header File                                              *
      * Close Files                                                    *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
      ******************************************************************
       FILE-CONTROL.
      ******************************************************************
      *
      * Testcase Input File
      *
           SELECT  REPO-FILE
                   ASSIGN TO DYNAMIC REPO-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS REPO-STATUS.
      *
           SELECT  REQU-REPORT
                   ASSIGN TO DYNAMIC REQU-REPO-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS REQU-STATUS.
      *
      * Requirement Table
      *
           SELECT  REQU-TABLE
                   ASSIGN TO DYNAMIC REQU-TAB-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS REQU-STATUS.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
      *
       FD  REPO-FILE
           RECORD CONTAINS 112 CHARACTERS
           DATA RECORD IS REPO-REC.
       01  REPO-REC.
           05  REPO-BASE-TYPE               PIC X(4).
           05  FILLER                       PIC X.
           05  REPO-BASE-ENTITY             PIC X(36).
           05  FILLER                       PIC X.
           05  REPO-RELATION                PIC X(4).
           05  FILLER                       PIC X.
           05  REPO-TARGET-TYPE             PIC X(4).
           05  FILLER                       PIC X.
           05  REPO-TARGET-ENTITY           PIC X(60).
      *
       FD  REQU-REPORT
           RECORD CONTAINS 96 CHARACTERS
           DATA RECORD IS REQU-REPORT-REC.
       01  REQU-REPORT-REC                  PIC X(96).
      *
       FD  REQU-TABLE
           RECORD CONTAINS 160 CHARACTERS
           DATA RECORD IS REQU-TABLE-REC.
      *
       01  REQU-TABLE-REC                   PIC X(160).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
       77  REPO-FILE-NAME                    PIC X(80).
       77  REQU-REPO-NAME                    PIC X(80).
       77  REQU-TAB-NAME                     PIC X(80).
       77  REQU-STATUS                       PIC XX.
       77  REPO-STATUS                       PIC XX.
           88  REPO-OK                       VALUE '00'.
           88  REPO-EOF                      VALUE '09'.
       01  REPO-TEST  REDEFINES REPO-STATUS.
            05  REPO-TEST-1                  PIC 9.
            05  REPO-TEST-2                  PIC X COMP-X.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 99.
           05  J                             PIC 99.
           05  K                             PIC 99.
      *
      ****************************************************************
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  LINE-LNG                      PIC 99   VALUE 80.
           05  NAME-LNG                      PIC 99   VALUE 40.
           05  PAGE-MAX                      PIC 99   VALUE 60.
           05  MAX-TC-REFS                   PIC 99   VALUE 20.
      *                          Array Lengths
      ****************************************************
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  FOUND-SW                      PIC 9    VALUE 0.
      *
      *
       01  COUNTS.
           05  LINE-COUNT               PIC 99     VALUE ZERO.
           05  PAGE-COUNT               PIC 9999   VALUE ZERO.
           05  NR-REQUIRES              PIC 9(7)   VALUE ZERO.
      *
       01  LOCAL-VARIABLES.
           05  LAST-FILE                PIC X(8).
           05  PERCENT                  PIC 999V99.
           05  NAME-STRING              PIC X(40).
           05  NAME-CHARS               REDEFINES NAME-STRING.
               10  NAME-CHAR            PIC X OCCURS 40 TIMES.
      *
KATI****   COPY DBPARMS.
      *
      ****************************************************************
      *   Data for printing out Requirements
      *
       01  YYMMDD.
           05  YY                       PIC 99.
           05  MM                       PIC 99.
           05  DD                       PIC 99.

      *
       01  NEWPAGE-LINE.
           05  FILLER                   PIC X(96)  VALUE '/'.
      *
       01  SEPARATOR-LINE.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(94)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
      *
       01  SEPARATOR-LINE-1.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(34)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(34)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(24)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
      *
       01  BLANK-LINE.
           05  FILLER                   PIC X(01)  VALUE '|'.
           05  FILLER                   PIC X(94)  VALUE SPACES.
           05  FILLER                   PIC X(01)  VALUE '|'.
      *
       01  REQ-REC-HEADER-1.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  FILLER                   PIC X(07)  VALUE SPACES.
           05  REQ-HEADER-1             PIC X(50)  VALUE
ENG           ' T E S T A N A L  T E S T C A S E   A U D I T O R '.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  FILLER                   PIC X(07)  VALUE SPACES.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  REQ-REC-HEADER-2.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  FILLER                   PIC X(14)  VALUE SPACES.
           05  REQ-HEADER-2             PIC X(40)  VALUE
ENG           'R E Q U I R E M E N T    R E P O R T   '.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  FILLER                   PIC X(10)  VALUE SPACES.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  REC-HEADER-3.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  REC-HEADER-3-1           PIC X(10)  VALUE
ENG           'TESTPROC: '.
           05  FILE-NAME-OUT            PIC X(40)  VALUE SPACES.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  REC-HEADER-3-2           PIC X(6)   VALUE
ENG           'DATE: '.
           05  DATUM-OUT.
               10  DAY-OUT              PIC 99.
               10  FILLER               PIC X VALUE '.'.
               10  MONTH-OUT            PIC 99.
               10  FILLER               PIC X VALUE '.'.
               10  YEAR-OUT             PIC 99.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  REC-HEADER-4.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  REC-HEADER-4-1           PIC X(10)  VALUE
ENG           'SYSTEM:   '.
           05  SYST-NAME-OUT            PIC X(40)  VALUE SPACES.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  REC-HEADER-4-2           PIC X(6)   VALUE
ENG           'PAGE: '.
           05  SEITE-OUT                PIC ZZZZZZZ9.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  REQ-TITLE-LINE.
           05  FILLER                   PIC X(2)   VALUE '| '.
           05  REQ-TITLE                PIC X(32)  VALUE
              'Requirement Name '.
           05  FILLER                   PIC X(3)   VALUE ' | '.
           05  REQ-REL-TITLE            PIC X(32)  VALUE
              'in Document Section '.
           05  FILLER                   PIC X(3)   VALUE ' | '.
           05  REQ-TEST-TITLE           PIC X(22)  VALUE
              ' TestCase Name '.
           05  FILLER                   PIC X(2)   VALUE ' |'.
      *
       01  REQ-DATA-LINE.
           05  FILLER                   PIC X(2)   VALUE '| '.
           05  REQ-NAME-OUT             PIC X(32).
           05  FILLER                   PIC X(3)   VALUE ' | '.
           05  REQ-BASE-OUT             PIC X(32).
           05  FILLER                   PIC X(3)   VALUE ' | '.
           05  REQ-TEST-OUT             PIC X(22).
           05  FILLER                   PIC X(2)   VALUE ' |'.
      *
       01  REQ-FOOT-LINE-1.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  REQ-FOOT-1       PIC X(48)  VALUE
ENG           'Number of Requirements to be tested         =   '.
           05  REQUS-TOTAL      PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(36)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  REQ-FOOT-LINE-2.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  REQ-FOOT-2       PIC X(48)  VALUE
ENG           'Number of Requirements with a Test Case     =   '.
           05  REQUS-WITH       PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(36)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  REQ-FOOT-LINE-3.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  REQ-FOOT-3       PIC X(48)  VALUE
ENG           'Number of Requirements without a Test Case  =   '.
           05  REQUS-WITHOUT    PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(36)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  REQ-FOOT-LINE-4.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  REQ-FOOT-4       PIC X(48)  VALUE
ENG           'Degree of Requirements Coverage             =   '.
           05  REQU-COVERAGE    PIC Z(2)9.99 VALUE ZERO.
           05  FILLER           PIC X(36)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  REQ-CSV-TITLE-LINE.
           05  REQ-TITLE-1      PIC X(12)  VALUE 'Requirement;'.
           05  REQ-TITLE-2      PIC X(09)  VALUE 'Document;'.
           05  REQ-TITLE-3      PIC X(08)  VALUE 'Testcase'.
      *
       01  REQU-TABLE-LINE      PIC X(160) VALUE SPACES.
      *
      ****************************************************************
       LINKAGE SECTION.
      ****************************************************************
       77  SYSTEM-NAME                       PIC X(8).
       77  FILE-NAME                         PIC X(8).
       77  TEST-PROC-NAME                    PIC X(40).
       77  INPUT-FILE-NAME                   PIC X(80).
       77  SPECS-LIB-NAME                    PIC X(80).
       77  TABLE-LIB-NAME                    PIC X(80).
       77  WORK-LIB-NAME                     PIC X(80).
       77  RETCODE                           PIC 99.
      *
      * Interface to Requirements checking
      *
       01  REQU-METRICS.
           05  NR-REQUS                      PIC 9999.
           05  NR-REQUS-WITH                 PIC 9999.
           05  NR-REQUS-WITHOUT              PIC 9999.
      *
KATI       COPY DBPARMS.
      *
      ****************************************************************
      /
       PROCEDURE DIVISION USING SYSTEM-NAME,
                                FILE-NAME,
                                TEST-PROC-NAME,
                                INPUT-FILE-NAME,
                                SPECS-LIB-NAME,
                                TABLE-LIB-NAME,
                                WORK-LIB-NAME,
                                RETCODE,
                                REQU-METRICS,
KATI                            DB-PARAMS.
      ******************************************************************
      *
       REQUIREMENT-PROCESSING SECTION.
      *
      ******************************************************************
      *
       START-REQUIREMENTS.
      *
           DISPLAY '+----------------------------------------------+'
           DISPLAY '| REQSPEC Report Program Vers 1.2 of Oct. 2008 |'
           DISPLAY '+----------------------------------------------+'
           DISPLAY '| System Name           = ' SYSTEM-NAME
           DISPLAY '| File   Name           = ' FILE-NAME
           DISPLAY '| Test Procedure Name   = ' TEST-PROC-NAME
           DISPLAY '| Import File Name      = ' INPUT-FILE-NAME
           DISPLAY '| Require Report Lib    = ' SPECS-LIB-NAME
           DISPLAY '| Require Tables Lib    = ' TABLE-LIB-NAME
           DISPLAY '+----------------------------------------------+'
TEST       DISPLAY '| Press Enter to continue !!'
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
       OPEN-TESTCASE-DB.
      *
           MOVE 'REQU' TO DB-TYPE-CODE
           MOVE 'OPEN' TO DB-FUNC-CODE
           CALL 'TCSIOMOD'  USING DB-PARAMS,
                                  REPO-FILE-NAME,
                                  WORK-LIB-NAME
           MOVE DB-RETCODE TO RETCODE
           IF RETCODE > ZERO
              DISPLAY '| TestCase Database could not be opened'
              DISPLAY '| TestCase FileName = ' FILE-NAME
              DISPLAY '| Press Enter to continue !'
              ACCEPT CONTINUE-KEY
              GOBACK
           END-IF
TEST***    DISPLAY '| TestCase database was opened '
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      ******************************************************************
      *
       OPEN-REQUIREMENT-REPORT.
      *
           MOVE ZEROES TO COUNTS
           MOVE ZEROES TO SWITCHES
           MOVE ZEROES TO REQU-STATUS
           MOVE SPACES TO REQU-REPO-NAME
           STRING SPECS-LIB-NAME            DELIMITED BY SPACE
                 '\'                        DELIMITED BY SIZE
                  FILE-NAME                 DELIMITED BY SPACE
                 '.req'                     DELIMITED BY SIZE
              INTO REQU-REPO-NAME
           OPEN OUTPUT REQU-REPORT.
TAB        MOVE SPACES TO REQU-TAB-NAME
TAB        STRING SPECS-LIB-NAME            DELIMITED BY SPACE
TAB              '\'                        DELIMITED BY SIZE
TAB              FILE-NAME                  DELIMITED BY SPACE
TEST             '.csr'                     DELIMITED BY SIZE
TAB           INTO REQU-TAB-NAME
TEST       DISPLAY 'REQSPEC Output Table in:' REQU-TAB-NAME
TEST***    ACCEPT CONTINUE-KEY
TAB        OPEN OUTPUT REQU-TABLE.
TAB        MOVE REQ-CSV-TITLE-LINE TO REQU-TABLE-REC
TAB        WRITE REQU-TABLE-REC.
      *****************************************************************
           MOVE ZERO  TO LINE-COUNT
           MOVE 1     TO PAGE-COUNT
           ACCEPT YYMMDD FROM DATE
           MOVE YY TO YEAR-OUT
           MOVE MM TO MONTH-OUT
           MOVE DD TO DAY-OUT.
           MOVE PAGE-COUNT   TO SEITE-OUT
           MOVE SYSTEM-NAME  TO SYST-NAME-OUT
           MOVE TEST-PROC-NAME TO FILE-NAME-OUT
           MOVE NEWPAGE-LINE TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE SEPARATOR-LINE TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE BLANK-LINE TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REQ-REC-HEADER-1 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REQ-REC-HEADER-2 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE BLANK-LINE TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REC-HEADER-3 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REC-HEADER-4 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE SEPARATOR-LINE-1 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REQ-TITLE-LINE TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE SEPARATOR-LINE-1 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC.
      *
      ******************************************************************
      *
       OPEN-REQUIREMENT-FILE.
           MOVE INPUT-FILE-NAME TO REPO-FILE-NAME
           MOVE ZEROES TO REPO-STATUS
           OPEN INPUT REPO-FILE
           IF REPO-STATUS NOT = ZEROES
              DISPLAY '| Requirements could not be opened !!'
              DISPLAY '| Status = ' REPO-STATUS
              DISPLAY '| Status1= ' REPO-TEST-1
              DISPLAY '| Status2= ' REPO-TEST-2
              DISPLAY '| Require File Name = ' REPO-FILE-NAME
              DISPLAY '| Press Enter to continue !'
              ACCEPT CONTINUE-KEY
              MOVE 23 TO RETCODE
              GOBACK
           END-IF.
      *
      ******************************************************************
      *
       PROCESS-REQUIREMENTS.
      *
TEST***    DISPLAY 'Read in Requirements  !!'
TEST***    ACCEPT CONTINUE-KEY
           MOVE ZEROS TO REQU-METRICS.
           PERFORM UNTIL REPO-EOF
              READ REPO-FILE
              AT END
                 MOVE '09' TO REPO-STATUS
              NOT AT END
                 IF REPO-TARGET-TYPE = 'REQU' AND
                    REPO-RELATION = 'OWNS'
                    ADD 1 TO NR-REQUS
                    MOVE REPO-TARGET-ENTITY TO REQ-NAME-OUT
                    MOVE 'REQU' TO DB-TYPE-CODE
                    MOVE 'READ' TO DB-FUNC-CODE
                    PERFORM ADJUST-TARGET-ENTITY
                    MOVE REPO-TARGET-ENTITY TO DB-REQ-NAME
                    MOVE ZEROES TO DB-RETCODE
TEST***    DISPLAY 'CALL TCSIOMOD USING ' DB-TYPE-CODE ' ' DB-FUNC-CODE
TEST***            ' Key= '  DB-REQ-NAME
TEST***    ACCEPT CONTINUE-KEY
                    CALL 'TCSIOMOD'  USING DB-PARAMS,
                                           REPO-FILE-NAME,
                                           WORK-LIB-NAME
                    MOVE DB-RETCODE TO RETCODE
                    IF RETCODE > ZERO
TEST                   DISPLAY '| Req: ' DB-REQ-NAME
TEST                   DISPLAY '| is missing in the Test Cases!'
TEST***                DISPLAY '| Press Enter to continue !'
TEST***                ACCEPT CONTINUE-KEY
                    END-IF
                    IF NR-TC-REFS > ZERO
                       ADD 1 TO NR-REQUS-WITH
                       MOVE 1 TO I
                       PERFORM UNTIL I > NR-TC-REFS
                                  OR I > MAX-TC-REFS
                          MOVE REPO-BASE-ENTITY TO REQ-BASE-OUT
                          MOVE TC-ID-REF (I) TO REQ-TEST-OUT
                          MOVE REQ-DATA-LINE TO REQU-REPORT-REC
                          WRITE REQU-REPORT-REC
TAB                       PERFORM WRITE-REQU-TABLE
                          ADD 1 TO LINE-COUNT
                          ADD 1 TO I
                       END-PERFORM
                    ELSE
                       ADD 1 TO NR-REQUS-WITHOUT
                       MOVE REPO-BASE-ENTITY TO REQ-BASE-OUT
                       MOVE 'Testcase is missing!' TO REQ-TEST-OUT
                       MOVE REQ-DATA-LINE TO REQU-REPORT-REC
                       WRITE REQU-REPORT-REC
                       ADD 1 TO LINE-COUNT
                    END-IF
                    MOVE SEPARATOR-LINE-1 TO REQU-REPORT-REC
                    WRITE REQU-REPORT-REC
                    ADD 1 TO LINE-COUNT
                 END-IF
              END-READ
           END-PERFORM.
      *
      ******************************************************************
      *
       CLOSE-REQUIREMENT-REPORT.
           MOVE NR-REQUS TO REQUS-TOTAL
           MOVE NR-REQUS-WITH TO REQUS-WITH
           MOVE NR-REQUS-WITHOUT TO REQUS-WITHOUT
           COMPUTE PERCENT = NR-REQUS-WITH / NR-REQUS
           MOVE PERCENT TO REQU-COVERAGE
           MOVE SEPARATOR-LINE-1 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE BLANK-LINE TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REQ-FOOT-LINE-1 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REQ-FOOT-LINE-2 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REQ-FOOT-LINE-3 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE REQ-FOOT-LINE-4 TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE BLANK-LINE TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           MOVE SEPARATOR-LINE TO REQU-REPORT-REC
           WRITE REQU-REPORT-REC
           CLOSE REQU-REPORT.
TAB        CLOSE REQU-TABLE.
      *
      ******************************************************************
      *
       CLOSE-REQUIREMENT-FILES.
           CLOSE REPO-FILE
           MOVE 'REQU' TO DB-TYPE-CODE
           DISPLAY '+-----------------------------------------+'
           DISPLAY '| Requirements were checked !!'
           DISPLAY '| Number of Requirements          = ' NR-REQUS
           DISPLAY '| Number of Requirements tested   = ' NR-REQUS-WITH
           DISPLAY '| Number of Requirements untested = '
                                                       NR-REQUS-WITHOUT
           DISPLAY '+----------------------------------------------+'
           DISPLAY '| Press Enter to terminate requireent analysis ! '
TEST       ACCEPT CONTINUE-KEY.
TEST       EXIT.
      *
      ******************************************************************
      *
       SUBROUTINES  SECTION.
      *
       ADJUST-TARGET-ENTITY.
           MOVE REPO-TARGET-ENTITY TO NAME-STRING
           MOVE NAME-LNG TO J
           PERFORM UNTIL NAME-CHAR (J) NOT = SPACE
                      OR J = 1
              SUBTRACT 1 FROM J
           END-PERFORM
           INSPECT NAME-STRING (1:J) REPLACING ALL ' ' BY '_'
           MOVE NAME-STRING TO REPO-TARGET-ENTITY
           EXIT.
      *
TAB    WRITE-REQU-TABLE.
TAB        MOVE SPACES TO REQU-TABLE-REC
TAB        MOVE REQ-NAME-OUT TO REPO-TARGET-ENTITY
TAB        PERFORM ADJUST-TARGET-ENTITY
TAB        STRING REPO-TARGET-ENTITY   DELIMITED BY SPACE
TAB              ';'                   DELIMITED BY SIZE
TAB          INTO REQU-TABLE-REC
TAB        END-STRING
TAB        MOVE REQU-TABLE-REC TO REQU-TABLE-LINE
TAB        MOVE SPACES TO REQU-TABLE-REC
TAB        MOVE REQ-BASE-OUT TO REPO-TARGET-ENTITY
TAB        PERFORM ADJUST-TARGET-ENTITY
TAB        STRING REQU-TABLE-LINE      DELIMITED BY SPACE
TAB               REPO-TARGET-ENTITY   DELIMITED BY SPACE
TAB              ';'                   DELIMITED BY SIZE
TAB          INTO REQU-TABLE-REC
TAB        END-STRING
TAB        MOVE REQU-TABLE-REC TO REQU-TABLE-LINE
TAB        MOVE SPACES TO REQU-TABLE-REC
TAB        STRING REQU-TABLE-LINE      DELIMITED BY SPACE
TAB               REQ-TEST-OUT         DELIMITED BY SPACE
TAB              ';'                   DELIMITED BY SIZE
TAB          INTO REQU-TABLE-REC
TAB        END-STRING
TAB        WRITE REQU-TABLE-REC.
TAB        EXIT.
      *
      ********************** end of source *****************************
