       IDENTIFICATION DIVISION.
       PROGRAM-ID. SOFCALC.
      ******************************************************************
      *  PROGRAM NAME: SOFCALC  (SOFAUDIT)                             *
      *  PURPOSE:      subprogram for producing the Centity export file*
      *                for SOFTCALC from COBOL Source                  *
      *                                                                *
      *  VERSION:      2.1 (01.2000)                                   *
      *                                                                *
      *  LANGUAGE:     MICROSOFT (R) COBOL OPTIMIZIG COMPILER          *
      *                VERSION 3.00A ("MICROFOCUS COBOL")              *
      *                                                                *
      *  PROGRAM FUNCTIONS:                                            *
      *    - open Calc Export File                                     *
      *    - write one introducing header record                       *
      *    - write 1 fixed format detail records with metric data      *
      *    - close Calc Export File                                    *
      ******************************************************************
      *
      *AUTHOR. SNEED.
      *INSTALLATION. BUDAPEST.
      *DATE-WRITTEN. JULY 2007.
      *
      ******************************************************************
      *
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SOURCE-COMPUTER. PC.
       OBJECT-COMPUTER. PC.
      *
      ******************************************************************
      *
       INPUT-OUTPUT SECTION.
      ******************************************************************
       FILE-CONTROL.
      ******************************************************************
      *
      * CALC export file
      *
           SELECT CALC-FILE
           ASSIGN TO DYNAMIC CALC-FILE-NAME
           ORGANISATION LINE SEQUENTIAL
           ACCESS SEQUENTIAL.
      *
       I-O-CONTROL.
      ******************************************************************
      *
       DATA DIVISION.
      *
      ******************************************************************
      *
       FILE SECTION.
      *
      ******************************************************************
      *
       FD  CALC-FILE
           RECORD CONTAINS 80 CHARACTERS
           DATA RECORD IS CALC-REC.
      *
       01  CALC-REC                           PIC X(80).
      ******************************************************************
      *
       WORKING-STORAGE SECTION.
      *
      ******************************************************************
       77  CALC-FILE-NAME               PIC X(80).
       01  CALC-FSTS-X.
           05  CALC-FSTS                PIC X(02).
           05  FILLER                   PIC X(01)  VALUE '#'.
       77  CONTINUE-KEY                 PIC X.
      *
       01  CALC-COUNTERS.
           05  NR-CALC-INPUTS           PIC 9999   VALUE ZERO.
           05  NR-CALC-OUTPUTS          PIC 9999   VALUE ZERO.
           05  NR-CALC-DATA             PIC 9999   VALUE ZERO.
           05  NR-CALC-VIEWS            PIC 9999   VALUE ZERO.
           05  NR-PROCS                 PIC 9999   VALUE ZERO.
      *
      ****************************************************************
      *    SOFTCALC EXPORT FILE LAYOUT
      ****************************************************************
      *
       01  CALC-EXPORT-VALUES.
      *
           05  OVERSIZE-VALUE           PIC 999999 VALUE ZEROES.
           05  CURRENT-VALUE            PIC 99999  VALUE ZEROES.
           05  CURRENT-RATIO            PIC 9.999  VALUE ZEROES.
           05  CURRENT-TAG              PIC X(20)  VALUE SPACES.
           05  CURRENT-ATTR             PIC X(40)  VALUE SPACES.
           05  XML-LINE             PIC X(72) VALUE
              '<?xml version="1.0" encoding="UTF-8" ?>'.
           05  DOC-LINE             PIC X(72) VALUE
              '<!--DOCTYPE IENTITY SYSTEM "Ientity.xsd"-->'.
           05  YYMMDD.
               10  YY                   PIC 99.
               10  MM                   PIC 99.
               10  DD                   PIC 99.
           05  DATUM.
               10  TAG                  PIC 99.
               10  FILLER               PIC X   VALUE '.'.
               10  MON                  PIC 99.
               10  FILLER               PIC X   VALUE '.'.
               10  JAHR                 PIC 99.
      *
       01  METRIK-EXPORT-TAGS.
           05  FILE-TAGS.
               10  METRIC-TAG       PIC X(20) VALUE 'IENTITY_FILE'.
               10  PRODUCT-ATTR     PIC X(20) VALUE 'Product'.
               10  VERSION-ATTR     PIC X(20) VALUE 'Version'.
               10  SYSTEM-ATTR      PIC X(20) VALUE 'System '.
               10  SENDER-ATTR      PIC X(20) VALUE 'Source '.
               10  DATE-ATTR        PIC X(20) VALUE 'Date   '.
           05  ENTITY-TAGS.
               10  ENTITY-TAG       PIC X(20) VALUE 'INTR_ENTITY'.
               10  ENTITY-ATTR      PIC X(20) VALUE 'Entity'.
               10  NAME-ATTR        PIC X(20) VALUE 'EName'.
               10  TYPE-ATTR        PIC X(20) VALUE 'EType'.
           05  STRUCTURE-TAGS.
               10  COMP-TAG      PIC X(20) VALUE 'Component_Attributes'.
               10  INTERFACE-TAG PIC X(20) VALUE 'Interface_Attributes'.
               10  OBJECT-TAG    PIC X(20) VALUE 'Object_Attributes'.
               10  PRODUCT-TAG   PIC X(20) VALUE 'Product_Attributes'.
               10  CHANGE-TAG       PIC X(20) VALUE 'Change_Rate'.
           05  COMPONENT-TAGS.
               10  LOCS-TAG         PIC X(20) VALUE 'Nr_Locs'.
               10  STMT-TAG         PIC X(20) VALUE 'Nr_Stmts'.
               10  FUNCT-TAG        PIC X(20) VALUE 'Nr_Functs'.
               10  PROC-TAG         PIC X(20) VALUE 'Nr_Procs'.
               10  VARIABLE-TAG     PIC X(20) VALUE 'Nr_Variables'.
           05  INTERFACE-TAGS.
               10  SOURCE-TAG       PIC X(20) VALUE 'Nr_Sources'.
               10  OPERATION-TAG    PIC X(20) VALUE 'Nr_Operations'.
               10  ARGUMENT-TAG     PIC X(20) VALUE 'Nr_Arguments'.
               10  RESULT-TAG       PIC X(20) VALUE 'Nr_Results'.
           05  OBJECT-TAGS.
               10  RELATION-TAG     PIC X(20) VALUE 'Nr_Relations'.
               10  METHOD-TAG       PIC X(20) VALUE 'Nr_Methods'.
               10  KEY-TAG          PIC X(20) VALUE 'Nr_Keys'.
               10  ATTR-TAG         PIC X(20) VALUE 'Nr_Attributes'.
           05  PRODUCT-TAGS.
               10  COMPNR-TAG       PIC X(20) VALUE 'Nr_Components'.
               10  FUNCPT-TAG       PIC X(20) VALUE 'Nr_Funct_Points'.
               10  DATAPT-TAG       PIC X(20) VALUE 'Nr_Data_Points'.
               10  OBJTPT-TAG       PIC X(20) VALUE 'Nr_Objt_Points'.
               10  COMPLEX-TAG      PIC X(20) VALUE 'Complexity_Rate'.
               10  QUALITY-TAG      PIC X(20) VALUE 'Quality_Rate'.
      *
      ******************************************************************
      *
       LINKAGE SECTION.
      *
      ******************************************************************
      * "Dynamic linkage file names" (and others)
      ******************************************************************
      *
       77  FUNCT-CODE                        PIC XX.
       77  RET-CODE                          PIC XX.
       77  FILE-NAME                         PIC X(80).
           COPY EXPPARMS.
      ******************************************************************
      *    Basic Measurements Required
      ******************************************************************
      *
           COPY OMETTAB.
      *
      ******************************************************************
      *
       PROCEDURE DIVISION USING FUNCT-CODE,
                                RET-CODE,
                                FILE-NAME,
                                EXPORT-DATA,
                                METRIC-TAB.
      *
      ******************************************************************
      *
       SOFCALC-MAIN SECTION.
      *
      ******************************************************************
      *
      * Produce the metric export file for PC-CALC
      *
      ******************************************************************
      *
           MOVE ZEROS TO RET-CODE.
      *
           EVALUATE FUNCT-CODE
              WHEN 'OP'
                PERFORM OPEN-CALC-EXPORT
              WHEN 'WR'
                PERFORM WRITE-CALC-METRICS
              WHEN 'CL'
                PERFORM CLOSE-CALC-EXPORT
              WHEN OTHER
                MOVE 11 TO RET-CODE
           END-EVALUATE.
      *
           GOBACK.
      *
      ******************************************************************
      *
       SOFCALC-IO-ROUTINES SECTION.
      *
      ******************************************************************
      *
      * Internal routines used in the section SOFCALC-MAIN
      *
      ******************************************************************
      *
       OPEN-CALC-EXPORT.
      *
      ******************************************************************
      *
      *  Open the Export File
      *
      ******************************************************************
      *
           MOVE FILE-NAME TO CALC-FILE-NAME
           MOVE ZEROES TO CALC-FSTS
           OPEN EXTEND CALC-FILE
           IF CALC-FSTS NOT = ZEROES
              DISPLAY '| CALC Export File cannot be opened'
              DISPLAY '| CALC Export File = ' CALC-FILE-NAME
TEST***       ACCEPT CONTINUE-KEY
              MOVE '99' TO RET-CODE
              GOBACK
           ELSE
              DISPLAY '| SOFCALC Version 2.2 of July 2007 '
              DISPLAY '| SoftCalc Export File = ' CALC-FILE-NAME
TEST***       ACCEPT CONTINUE-KEY
           END-IF.
           PERFORM CREATE-CALC-HEADER.
           EXIT.
      *
      ******************************************************************
      *
       CLOSE-CALC-EXPORT.
      *
      ******************************************************************
      *
      *  Close the Export File
      *
      ******************************************************************
      *
           PERFORM WRITE-CALC-PRODUCT
           MOVE 'IENTITY' TO CURRENT-TAG
           PERFORM WRITE-FIRST-LEVEL-END
           CLOSE CALC-FILE.
           EXIT.
      *
      ************************
      *
       CREATE-CALC-HEADER.
      *
      ******************************************************************
      *
      *  Create Metric Header
      *
      ******************************************************************
      *
      *                               Set up Header Info
      *
      *  Create Metric Header
      *
           MOVE XML-LINE TO CALC-REC
           WRITE CALC-REC
           MOVE DOC-LINE TO CALC-REC
           WRITE CALC-REC
           MOVE 'IENTITY' TO CURRENT-TAG.
           PERFORM WRITE-FIRST-LEVEL-START.
           MOVE METRIC-TAG TO CURRENT-TAG.
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE PRODUCT-ATTR TO CURRENT-TAG
           MOVE EXPORT-PRODUCT-NAME TO CURRENT-ATTR
           PERFORM WRITE-THIRD-LEVEL-ATTR.
SN0611     MOVE VERSION-ATTR TO CURRENT-TAG
SN0611     MOVE EXPORT-VERSION-NAME TO CURRENT-ATTR
SN0611     PERFORM WRITE-THIRD-LEVEL-ATTR.
           MOVE SYSTEM-ATTR TO CURRENT-TAG
           MOVE EXPORT-SYSTEM-NAME TO CURRENT-ATTR
           PERFORM WRITE-THIRD-LEVEL-ATTR.
           ACCEPT YYMMDD FROM DATE
           MOVE YY TO JAHR
           MOVE MM TO MON
           MOVE DD TO TAG
           MOVE DATE-ATTR TO CURRENT-TAG
           MOVE DATUM TO CURRENT-ATTR
           PERFORM WRITE-THIRD-LEVEL-ATTR.
           MOVE SENDER-ATTR TO CURRENT-TAG
           MOVE 'SOFAUDIT' TO CURRENT-ATTR
           PERFORM WRITE-THIRD-LEVEL-ATTR.
           MOVE METRIC-TAG TO CURRENT-TAG.
           PERFORM WRITE-FILE-HEADER-END.
           EXIT.
      *
      ************************
      *
       WRITE-CALC-METRICS.
      *
      ******************************************************************
      *
      *  Assign Values to Calc Metrics & Write Out
      *
      ******************************************************************
      *
           EVALUATE ENTITY-TYPE
              WHEN 'PROD'  PERFORM WRITE-CALC-PRODUCT
              WHEN 'PROG'  PERFORM WRITE-CALC-MODULE
              WHEN 'MOD '  PERFORM WRITE-CALC-MODULE
              WHEN 'CLAS'  PERFORM WRITE-CALC-CLASS
              WHEN 'TAB '  PERFORM WRITE-CALC-OBJECT
              WHEN 'VIEW'  PERFORM WRITE-CALC-OBJECT
              WHEN 'FILE'  PERFORM WRITE-CALC-OBJECT
              WHEN 'REC '  PERFORM WRITE-CALC-OBJECT
              WHEN 'LIST'  PERFORM WRITE-CALC-INTERFACE
              WHEN 'MASK'  PERFORM WRITE-CALC-INTERFACE
              WHEN 'GUI '  PERFORM WRITE-CALC-INTERFACE
              WHEN 'PAGE'  PERFORM WRITE-CALC-INTERFACE
              WHEN 'CALL'  PERFORM WRITE-CALC-INTERFACE
              WHEN 'MESS'  PERFORM WRITE-CALC-INTERFACE
              WHEN OTHER   CONTINUE
           END-EVALUATE.
           EXIT.
      *
      ******************************************************************
       WRITE-CALC-MODULE.
      *                               write module attributes
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE ENTITY-ATTR TO CURRENT-TAG
           MOVE 'Component' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE NAME-ATTR   TO CURRENT-TAG
           MOVE ENTITY-NAME TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE TYPE-ATTR   TO CURRENT-TAG
           MOVE 'MODU' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE COMP-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-START.
           MOVE LOCS-TAG TO CURRENT-TAG
           MOVE NR-LINES TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE STMT-TAG TO CURRENT-TAG
           MOVE NR-STMTS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE FUNCT-TAG TO CURRENT-TAG
           MOVE NR-PROCEDURES TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE VARIABLE-TAG TO CURRENT-TAG
           MOVE NR-VARIABLES TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE CHANGE-TAG TO CURRENT-TAG
           MOVE 0 TO CURRENT-RATIO
           PERFORM WRITE-FIFTH-LEVEL-RATIO
           MOVE COMP-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-END
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-END.
           EXIT.
      *
      ******************************************************************
       WRITE-CALC-CLASS.
      *                               write class attributes
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE ENTITY-ATTR TO CURRENT-TAG
           MOVE 'Component' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE NAME-ATTR   TO CURRENT-TAG
           MOVE ENTITY-NAME TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE TYPE-ATTR   TO CURRENT-TAG
           MOVE 'CLAS' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE COMP-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-START.
           MOVE LOCS-TAG TO CURRENT-TAG
           MOVE NR-LINES TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE STMT-TAG TO CURRENT-TAG
           MOVE NR-STMTS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE FUNCT-TAG TO CURRENT-TAG
           MOVE NR-METHODS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE VARIABLE-TAG TO CURRENT-TAG
           MOVE NR-VARIABLES TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE CHANGE-TAG TO CURRENT-TAG
           MOVE 0 TO CURRENT-RATIO
           PERFORM WRITE-FIFTH-LEVEL-RATIO
           MOVE COMP-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-END
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-END.
           EXIT.
      *
      ******************************************************************
       WRITE-CALC-DATA.
      *                               write object attributes
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE ENTITY-ATTR TO CURRENT-TAG
           MOVE 'Object ' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE NAME-ATTR   TO CURRENT-TAG
           MOVE ENTITY-NAME TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE TYPE-ATTR   TO CURRENT-TAG
           MOVE 'DATA'      TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE OBJECT-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-START.
           MOVE RELATION-TAG TO CURRENT-TAG
           MOVE NR-FOREIGN-FUNCTS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE METHOD-TAG TO CURRENT-TAG
           MOVE NR-METHODS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE KEY-TAG TO CURRENT-TAG
           MOVE NR-STRUCTURES TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE ATTR-TAG TO CURRENT-TAG
           MOVE NR-VARIABLES TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE CHANGE-TAG TO CURRENT-TAG
           MOVE 0 TO CURRENT-RATIO
           PERFORM WRITE-FIFTH-LEVEL-RATIO
           MOVE OBJECT-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-END
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-END.
           EXIT.
      *
      ******************************************************************
       WRITE-CALC-OBJECT.
      *                               write object attributes
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE ENTITY-ATTR TO CURRENT-TAG
           MOVE 'Object ' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE NAME-ATTR   TO CURRENT-TAG
           MOVE ENTITY-NAME TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE TYPE-ATTR   TO CURRENT-TAG
           MOVE ENTITY-TYPE TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE OBJECT-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-START.
           MOVE RELATION-TAG TO CURRENT-TAG
           MOVE NR-FOREIGN-FUNCTS TO CURRENT-VALUE
PATCH      IF NR-FOREIGN-FUNCTS = ZERO
PATCH         MOVE 2 TO CURRENT-VALUE
PATCH      END-IF
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE METHOD-TAG TO CURRENT-TAG
           MOVE NR-METHODS TO CURRENT-VALUE
PATCH      IF NR-METHODS = ZERO
PATCH         MOVE 4 TO CURRENT-VALUE
PATCH      END-IF
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE KEY-TAG TO CURRENT-TAG
           MOVE NR-STRUCTURES TO CURRENT-VALUE
PATCH      IF NR-STRUCTURES = ZERO
PATCH         MOVE 1 TO CURRENT-VALUE
PATCH      END-IF
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE ATTR-TAG TO CURRENT-TAG
           MOVE NR-VARIABLES TO CURRENT-VALUE
PATCH      IF NR-VARIABLES = ZERO
PATCH         MOVE 8 TO CURRENT-VALUE
PATCH      END-IF
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE CHANGE-TAG TO CURRENT-TAG
           MOVE 0 TO CURRENT-RATIO
           PERFORM WRITE-FIFTH-LEVEL-RATIO
           MOVE OBJECT-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-END
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-END.
           EXIT.
      *
      ******************************************************************
       WRITE-CALC-INTERFACE.
      *                               write interface attributes
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE ENTITY-ATTR TO CURRENT-TAG
           MOVE 'Interface' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE NAME-ATTR   TO CURRENT-TAG
           MOVE ENTITY-NAME TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE TYPE-ATTR   TO CURRENT-TAG
           MOVE ENTITY-TYPE TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE INTERFACE-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-START.
           MOVE SOURCE-TAG TO CURRENT-TAG
           MOVE 01 TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE OPERATION-TAG TO CURRENT-TAG
           IF NR-PROCEDURES > ZERO
              MOVE NR-PROCEDURES TO CURRENT-VALUE
           ELSE
              IF NR-METHODS > ZERO
                 MOVE NR-METHODS TO CURRENT-VALUE
PATCH         ELSE
PATCH            MOVE 2 TO CURRENT-VALUE
PATCH         END-IF
           END-IF
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE ARGUMENT-TAG TO CURRENT-TAG
           MOVE NR-ARGUMENTS  TO CURRENT-VALUE
PATCH      IF NR-ARGUMENTS = ZERO
PATCH         MOVE 4 TO CURRENT-VALUE
PATCH      END-IF
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE RESULT-TAG TO CURRENT-TAG
           MOVE NR-RESULTS TO CURRENT-VALUE
PATCH      IF NR-RESULTS = ZERO
PATCH         MOVE 1 TO CURRENT-VALUE
PATCH      END-IF
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE CHANGE-TAG TO CURRENT-TAG
           MOVE 0 TO CURRENT-RATIO
           PERFORM WRITE-FIFTH-LEVEL-RATIO
           MOVE INTERFACE-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-END
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-END.
           EXIT.
      *
      ******************************************************************
       WRITE-CALC-PRODUCT.
      *                               write product attributes
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-START.
           MOVE ENTITY-ATTR TO CURRENT-TAG
           MOVE 'Product  ' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE NAME-ATTR   TO CURRENT-TAG
           MOVE ENTITY-NAME TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR.
           MOVE TYPE-ATTR   TO CURRENT-TAG
           MOVE 'DIST' TO CURRENT-ATTR
           PERFORM WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE PRODUCT-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-START.
           MOVE COMPNR-TAG TO CURRENT-TAG
           MOVE NR-MODULES TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE LOCS-TAG TO CURRENT-TAG
           MOVE NR-LINES TO OVERSIZE-VALUE
           PERFORM WRITE-FIFTH-LEVEL-OVERSIZE
           MOVE STMT-TAG TO CURRENT-TAG
           MOVE NR-STMTS TO OVERSIZE-VALUE
           PERFORM WRITE-FIFTH-LEVEL-OVERSIZE
           MOVE PROC-TAG TO CURRENT-TAG
           COMPUTE NR-PROCS = NR-PROCEDURES + NR-METHODS
           MOVE NR-PROCS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE FUNCPT-TAG TO CURRENT-TAG
           MOVE FUNC-POINTS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE DATAPT-TAG TO CURRENT-TAG
           MOVE DATA-POINTS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE OBJTPT-TAG TO CURRENT-TAG
           MOVE OBJT-POINTS TO CURRENT-VALUE
           PERFORM WRITE-FIFTH-LEVEL-DATA
           MOVE COMPLEX-TAG TO CURRENT-TAG
           MOVE PROG-COMPLEXITY TO CURRENT-RATIO
           PERFORM WRITE-FIFTH-LEVEL-RATIO
           MOVE QUALITY-TAG TO CURRENT-TAG
           MOVE PROG-QUALITY TO CURRENT-RATIO
           PERFORM WRITE-FIFTH-LEVEL-RATIO
           MOVE CHANGE-TAG TO CURRENT-TAG
           MOVE 0 TO CURRENT-RATIO
           PERFORM WRITE-FIFTH-LEVEL-RATIO
           MOVE PRODUCT-TAG TO CURRENT-TAG
           PERFORM WRITE-THIRD-LEVEL-END
           MOVE ENTITY-TAG TO CURRENT-TAG
           PERFORM WRITE-SECOND-LEVEL-END.
           EXIT.
      ******************************************************************
      *
       WRITE-XML SECTION.
      ************************************************ 1st Level
       WRITE-FIRST-LEVEL-START.
           MOVE SPACES TO CALC-REC
           STRING ' <'              DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC
           WRITE CALC-REC.
      ************************************************ 1st Level
       WRITE-FIRST-LEVEL-END.
           MOVE SPACES TO CALC-REC.
           STRING ' </'             DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-SECOND-LEVEL-START.
           MOVE SPACES TO CALC-REC.
           STRING '   <'            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-SECOND-LEVEL-ATTR.
           MOVE SPACES TO CALC-REC.
           STRING '    '            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  ' = "'            DELIMITED BY SIZE
                   CURRENT-ATTR     DELIMITED BY SPACE
                  '"'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-SECOND-LEVEL-ATTR-LAST.
           MOVE SPACES TO CALC-REC.
           STRING '    '            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  ' = "'            DELIMITED BY SIZE
                   CURRENT-ATTR     DELIMITED BY SPACE
                  '" >'             DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-SECOND-LEVEL-END.
           MOVE SPACES TO CALC-REC.
           STRING '   </'           DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 2nd Level
       WRITE-FILE-HEADER-END.
           MOVE SPACES TO CALC-REC.
           MOVE   '   />'  TO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 3rd Level
       WRITE-THIRD-LEVEL-START.
           MOVE SPACES TO CALC-REC.
           STRING '     <'          DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 3rd Level
       WRITE-THIRD-LEVEL-ATTR.
           MOVE SPACES TO CALC-REC.
           STRING '    '            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  ' = "'            DELIMITED BY SIZE
                   CURRENT-ATTR     DELIMITED BY SPACE
                  '"'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 3rd Level
       WRITE-THIRD-LEVEL-LAST-ATTR.
           MOVE SPACES TO CALC-REC.
           STRING '    '            DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  ' = "'            DELIMITED BY SIZE
                   CURRENT-ATTR     DELIMITED BY SPACE
                  '" >'             DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 3rd Level
       WRITE-THIRD-LEVEL-END.
           MOVE SPACES TO CALC-REC.
           STRING '     </'         DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
      ************************************************ 5th Level
       WRITE-FIFTH-LEVEL-DATA.
           MOVE SPACES TO CALC-REC.
           STRING '         <'      DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
                   CURRENT-VALUE    DELIMITED BY SIZE
                  '</'              DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
           EXIT.
      ************************************************ 5th Level
       WRITE-FIFTH-LEVEL-OVERSIZE.
           MOVE SPACES TO CALC-REC.
           STRING '         <'      DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
                   OVERSIZE-VALUE   DELIMITED BY SIZE
                  '</'              DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
           EXIT.
      *
      ************************************************ 5th Level
       WRITE-FIFTH-LEVEL-RATIO.
           MOVE SPACES TO CALC-REC.
           STRING '         <'      DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
                   CURRENT-RATIO    DELIMITED BY SIZE
                  '</'              DELIMITED BY SIZE
                   CURRENT-TAG      DELIMITED BY SPACE
                  '>'               DELIMITED BY SIZE
              INTO CALC-REC.
           WRITE CALC-REC.
           EXIT.
      *
      ******************************************************************
      *                      end of SOFTCALC
      ******************************************************************
