       IDENTIFICATION DIVISION.
VER-1. PROGRAM-ID. TCSIOMOD.
      *************************************************************
      *
      *              Create the TCS-DB file.
      *          Retreive data from the TCS-DB file.
      *             Called from TCSANAL for creation,
      *        from REQSPEC, OBJSPEC, xxxxxx, and yyyyyy
      *                   to retreive data.
      *
      *AUTHOR : KATALIN ERDS
      *DATE   : MARTZ, 2006
      *************************************************************
       ENVIRONMENT DIVISION.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *                             input file : testcase tcs file
           SELECT TCS-INPUT ASSIGN TO DYNAMIC INPUT-FILE-IND
                  ORGANIZATION IS LINE SEQUENTIAL
                  FILE STATUS IS FSTS.
      *                             db-file for creation
           SELECT TCS-DB-OUT ASSIGN TO DYNAMIC TCS-DB-FILE-IND
                  ORGANIZATION IS INDEXED ACCESS IS SEQUENTIAL
                  RECORD KEY IS DB-REC-KEY-O
                  ALTERNATE KEY IS DB-REC-ALT-KEY-O WITH DUPLICATES
                  FILE STATUS IS FSTS.
      *                              db-file for input
           SELECT TCS-DB-IN ASSIGN TO DYNAMIC TCS-DB-FILE-IND
                  ORGANIZATION IS INDEXED ACCESS IS DYNAMIC
                  RECORD KEY IS DB-REC-KEY
                  ALTERNATE KEY IS DB-REC-ALT-KEY WITH DUPLICATES
                  FILE STATUS IS FSTS.
      ******************************************************************
      *
       DATA DIVISION.
      *
       FILE SECTION.
      *
      ******************************************************************
      *
       FD  TCS-INPUT.
       01  TCS-INPUT-REC.
           05 REC-NR                 PIC 9(4).
           05 REC-FIELDS.
              10 REC-NAME            PIC X(40).
              10 REC-VALUE           PIC X(80).
      *
       FD  TCS-DB-OUT.
       01  DB-RECORD-O.
           05  DB-REC-KEY-O.
               10  DB-REC-NR-O               PIC 9999.
               10  DB-SEQ-NR-O               PIC 99.
           05  DB-REC-ALT-KEY-O.
      *                                      TEST / REQU / CASE / OBJT
               10  DB-REC-TYPE-O             PIC X(4).
               10  DB-REC-VALUE-O            PIC X(40).
      *
       FD  TCS-DB-IN.
       01  DB-RECORD.
           05  DB-REC-KEY.
               10  DB-REC-NR                 PIC 9999.
               10  DB-SEQ-NR                 PIC 99.
           05  DB-REC-ALT-KEY.
      *                                      TEST / REQU / CASE / OBJT
               10  DB-REC-TYPE               PIC X(4).
               10  DB-REC-VALUE              PIC X(40).
      *
      ******************************************************************
      *
       WORKING-STORAGE SECTION.
      *
       01  FILE-STATUS-VARS.
           02 FSTS                PIC XX.
              88 FSTS-OK          VALUE '00'.
           02 FILLER              REDEFINES FSTS.
              03 KEY-1            PIC X.
              03 KEY-2            PIC X COMP-X.
           02 EOF-IN              PIC 9.
              88 EOF              VALUE 1.
           02 OPEN-FLAG           PIC 9.
           02 DB-CREATED          PIC 9.
           02 FOUND-SW            PIC 9.
       01  KEY-3                  PIC 999.
       01  FILLER.
           02 INPUT-FILE-IND      PIC X(80).
           02 TCS-DB-FILE-IND     PIC X(80).
           02 WRK-FILE-IND        PIC X(80).
           02 SAVE-KEY            PIC X(44).
       01  ERROR-LINES.
           02 ERROR-LINE-1        PIC X(79).
           02 ERROR-LINE-2        PIC X(79).
      *
       01  FILLER.
           05 INIT-X              PIC X(4) VALUE 'INIT'.
           05 OPEN-X              PIC X(4) VALUE 'OPEN'.
           05 READ-X              PIC X(4) VALUE 'READ'.
           05 CLOS-X              PIC X(4) VALUE 'CLOS'.
           05 DB-TYPES.
              10 TEST-X           PIC X(4) VALUE 'TEST'.
              10 REQU-X           PIC X(4) VALUE 'REQU'.
              10 CASE-X           PIC X(4) VALUE 'CASE'.
              10 OBJT-X           PIC X(4) VALUE 'OBJT'.
           05 FILLER              REDEFINES DB-TYPES.
              10 DB-TYPE          OCCURS 4 TIMES PIC X(4).
      *
      *                               convert lower case to upper case
       01  CONVERSION-TAB.
           05 CONV-LINE-LNG       PIC X COMP-X VALUE 80.
           05 C-I                 PIC X COMP-X.
           05 CHAR-LINE.
              10 CHAR             OCCURS 80 TIMES PIC X.
           05 CODE-NUMBERS        REDEFINES CHAR-LINE.
              10 CHAR-CODE        OCCURS 80 TIMES PIC X COMP-X.
      *
      *
       01  FILLER.
           05 NR-DB-RECS          PIC 9(4) BINARY.
           05 NR-DB-RECS-Z        PIC ZZZ9.
           05 WRK-REC-NR          PIC 9(4).
           05 I                   PIC X COMP-X.
           05 CONTINUE-KEY        PIC X.
      *
      ******************************************************************
      *
       LINKAGE SECTION.
      *
      ******************************************************************
           COPY DBPARMS.
       01  CASE-FILE-NAME         PIC X(80).
       01  WORK-LIB-NAME          PIC X(80).
      *
      ******************************************************************
      *
       PROCEDURE DIVISION USING DB-PARAMS
                                CASE-FILE-NAME
                                WORK-LIB-NAME.
      *
      ******************************************************************
       TCS-IO-MOD SECTION.
      ******************************************************************
           MOVE ZERO TO DB-RETCODE
TEST***    IF DB-TYPE-CODE = OBJT-X
TEST***     DISPLAY DB-FUNC-CODE ' ' DB-TYPE-CODE  SPACE DB-OBJECT-NAME
TEST***      ACCEPT CONTINUE-KEY
TEST***    END-IF
           EVALUATE DB-FUNC-CODE
             WHEN INIT-X
               PERFORM CREATE-TCS-DB-FILE
               MOVE ZERO TO OPEN-FLAG
             WHEN OPEN-X
               IF DB-CREATED = 1
                 IF OPEN-FLAG = ZERO
                   OPEN INPUT TCS-DB-IN
                   PERFORM CHECK-TCS-DB-FSTS
                   MOVE 1 TO OPEN-FLAG
                 ELSE
                   DISPLAY 'TCS-DB file already opened'
                 END-IF
               ELSE
                 DISPLAY 'TCS-DB file does not exist'
                 MOVE 71 TO DB-RETCODE
               END-IF
             WHEN READ-X
               IF OPEN-FLAG = 1
                 PERFORM READ-TCS-DB-FILE
               ELSE
                 DISPLAY 'TCS-DB file is not opened'
                 MOVE 72 TO DB-RETCODE
               END-IF
             WHEN CLOS-X
               CLOSE TCS-DB-IN
               MOVE ZERO TO OPEN-FLAG
TEST**         DELETE FILE TCS-DB-IN
               MOVE ZERO TO DB-CREATED
           END-EVALUATE.
       EXIT-PROGRAM.
           EXIT PROGRAM.
      *****************************************************************
       CREATE-TCS-DB-FILE SECTION.
      *****************************************************************
           MOVE CASE-FILE-NAME TO INPUT-FILE-IND
           OPEN INPUT TCS-INPUT
           PERFORM CHECK-INPUT-FILE-FSTS
           MOVE SPACES TO TCS-DB-FILE-IND
           STRING WORK-LIB-NAME  DELIMITED SPACE
                  '\TCSDATA.IDS' DELIMITED SIZE
             INTO TCS-DB-FILE-IND
           OPEN OUTPUT TCS-DB-OUT
           PERFORM CHECK-TCS-DB-FSTS
           MOVE ZERO TO DB-CREATED
           MOVE ZERO TO WRK-REC-NR
           MOVE ZERO TO NR-DB-RECS
           MOVE ZERO TO EOF-IN
           READ TCS-INPUT
             AT END
               MOVE 1 TO EOF-IN
           END-READ
           PERFORM UNTIL EOF
             IF REC-NR NOT = WRK-REC-NR
               MOVE ZERO TO DB-SEQ-NR-O
               MOVE REC-NR TO WRK-REC-NR
             END-IF
             PERFORM VARYING I FROM 1 BY 1 UNTIL I > 4
      *                            convert lower case to upper case
               MOVE REC-FIELDS TO CHAR-LINE
               PERFORM CONVERT-LOWER-TO-UPPER-CASE
               MOVE CHAR-LINE TO REC-FIELDS
               IF REC-NAME = DB-FIELD (I)
                 MOVE REC-NR TO DB-REC-NR-O
                 MOVE REC-VALUE TO DB-REC-VALUE-O
                 MOVE DB-TYPE (I) TO DB-REC-TYPE-O
                 ADD 1 TO DB-SEQ-NR-O
                 WRITE DB-RECORD-O
                   INVALID KEY
                     SUBTRACT 1 FROM NR-DB-RECS
                     SUBTRACT 1 FROM DB-SEQ-NR-O
                     DISPLAY 'Key error on TCS-DB ' DB-RECORD-O
TEST**               ACCEPT CONTINUE-KEY
                   NOT INVALID KEY
                     ADD 1 TO NR-DB-RECS
                 END-WRITE
               END-IF
             END-PERFORM
             READ TCS-INPUT
               AT END
                 MOVE 1 TO EOF-IN
             END-READ
           END-PERFORM
           IF NR-DB-RECS > ZERO
             MOVE 1 TO DB-CREATED
           END-IF
           MOVE NR-DB-RECS TO NR-DB-RECS-Z
           MOVE SPACES TO ERROR-LINES
           STRING 'TCS-DB-FILE created with ' DELIMITED SIZE
                  NR-DB-RECS-Z                DELIMITED SIZE
                  ' records'                  DELIMITED SIZE
             INTO ERROR-LINE-1
           DISPLAY ERROR-LINE-1.
TEST**     ACCEPT CONTINUE-KEY.
           CLOSE TCS-DB-OUT.
      *****************************************************************
       READ-TCS-DB-FILE SECTION.
      *****************************************************************
           MOVE ZERO TO NR-TC-REFS
           PERFORM VARYING I FROM 2 BY 1 UNTIL I > 4
                   OR DB-TYPE-CODE = DB-TYPE (I)
             CONTINUE
           END-PERFORM
           IF I NOT > 4
             MOVE DB-TYPE-CODE TO DB-REC-TYPE
             MOVE DB-FIELD (I) TO DB-REC-VALUE
           ELSE
             DISPLAY 'Unknown DB.record type : ' DB-TYPE-CODE
             MOVE 74 TO DB-RETCODE
             PERFORM EXIT-PROGRAM
           END-IF
      *                           convert lower case to upper case char.
           IF DB-TYPE-CODE = TEST-X
      *                           this comes already in upper case char.
             CONTINUE
           ELSE
             MOVE DB-REC-VALUE TO CHAR-LINE
             PERFORM CONVERT-LOWER-TO-UPPER-CASE
             MOVE CHAR-LINE TO DB-REC-VALUE
           END-IF
           MOVE DB-REC-ALT-KEY TO SAVE-KEY
      *              read REQU/CASE/OBJT type records with alternate key
           START TCS-DB-IN KEY = DB-REC-ALT-KEY
             INVALID KEY
      *        DISPLAY 'TCSIOMOD No entry found ' DB-REC-ALT-KEY
               MOVE 73 TO DB-RETCODE
               PERFORM EXIT-PROGRAM
           END-START
           MOVE ZERO TO EOF-IN
           READ TCS-DB-IN NEXT RECORD
             AT END MOVE 1 TO EOF-IN
           END-READ
           MOVE ZERO TO I
           PERFORM UNTIL DB-REC-ALT-KEY NOT = SAVE-KEY OR EOF
             ADD 1 TO I
             IF I NOT > MAX-REFS
               MOVE DB-REC-NR TO TC-NR-REF (I)
               MOVE SPACES TO TC-ID-REF (I)
               MOVE I TO NR-TC-REFS
             END-IF
             READ TCS-DB-IN NEXT RECORD
               AT END MOVE 1 TO EOF-IN
             END-READ
           END-PERFORM.
      *                               read in the TestCase-ID-s with
      *                               alternate key beginning with TEST
           MOVE TEST-X TO DB-REC-ALT-KEY
           START TCS-DB-IN KEY >= DB-REC-ALT-KEY
             INVALID KEY
               MOVE ZERO TO FOUND-SW
               MOVE 73 TO DB-RETCODE
             NOT INVALID KEY
               MOVE 1 TO FOUND-SW
           END-START
           MOVE ZERO TO EOF-IN
           IF FOUND-SW = 1
             READ TCS-DB-IN NEXT RECORD
               AT END MOVE 1 TO EOF-IN
             END-READ
           END-IF
           PERFORM UNTIL FOUND-SW = ZERO OR EOF OR
                         DB-REC-TYPE NOT = TEST-X
             PERFORM VARYING I FROM 1 BY 1 UNTIL I > NR-TC-REFS
               IF TC-NR-REF (I) = DB-REC-NR
                 MOVE DB-REC-VALUE TO TC-ID-REF (I)
TEST***          DISPLAY DB-REC-NR ' TC-ID = ' DB-REC-VALUE
               END-IF
             END-PERFORM
             READ TCS-DB-IN NEXT RECORD
               AT END MOVE 1 TO EOF-IN
             END-READ
           END-PERFORM.
TEST***    ACCEPT CONTINUE-KEY.
      *****************************************************************
      *         convert lower case characters to upper case characters
      *COPY LOWUPCOD.
      *****************************************************************
       SUBROUTINES SECTION.
      *****************************************************************
      *
       CONVERT-LOWER-TO-UPPER-CASE.
      *        Convert lower case characters to upper case characters
      *        This algorithm is valid only for ASCII code
           PERFORM VARYING C-I FROM 1 BY 1 UNTIL C-I > CONV-LINE-LNG
      *                                the code of 'a' is 97
      *                                the code of 'A' is 65
      *                                the code of 'z' is 122
      *                                the code of 'Z' is 90
             IF CHAR-CODE (C-I) > 96 AND CHAR-CODE (C-I) < 123
               SUBTRACT 32 FROM CHAR-CODE (C-I)
             END-IF
           END-PERFORM.
      *****************************************************************
       CHECK-FILE-STATUS SECTION.
      *****************************************************************
       CHECK-INPUT-FILE-FSTS.
           MOVE INPUT-FILE-IND TO WRK-FILE-IND
           PERFORM CHECK-FSTS.
      *****************************************************************
       CHECK-TCS-DB-FSTS.
           MOVE TCS-DB-FILE-IND TO WRK-FILE-IND
           PERFORM CHECK-FSTS.
      *****************************************************************
       CHECK-FSTS.
           IF NOT FSTS-OK
             MOVE SPACES TO ERROR-LINE-1 ERROR-LINE-2
             STRING  'Error on open ' DELIMITED SIZE
                     WRK-FILE-IND     DELIMITED SPACE
               INTO ERROR-LINE-1
             IF KEY-1 = '9'
               MOVE KEY-2 TO KEY-3
               STRING 'File Status: ' DELIMITED SIZE
                      KEY-1           DELIMITED SIZE
                      SPACE           DELIMITED SIZE
                      KEY-3           DELIMITED SIZE
                  INTO ERROR-LINE-2
             ELSE
               STRING 'File Status: ' DELIMITED SIZE
                      FSTS            DELIMITED SIZE
                  INTO ERROR-LINE-2
             END-IF
             MOVE 71 TO DB-RETCODE
TEST**       DISPLAY ERROR-LINE-1
TEST**       DISPLAY ERROR-LINE-2
TEST**       ACCEPT CONTINUE-KEY
             PERFORM EXIT-PROGRAM
           END-IF.
      ******************** end-of-source-code *************************
