      ******************************************************************
      * Analysis Tool for scanning Header of CSV Files                 *
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. TCSSCAN.
      *AUTHOR.     H. SNEED.
      *DATE.       NOV. 2013.
      *VERSION.    1.1 of Nov. 2013
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *                  TCS SCANNING SPECIFICATION                    *
      * copy  CSV Header Line Titles                                   *
      * create Word List                                               *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *
           SELECT SOURCE-FILE  ASSIGN TO DYNAMIC INPUT-FILE
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS INPUT-STAT.
      *
           SELECT LOG-FILE ASSIGN TO DYNAMIC LOG-NAME
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS LOG-STAT.
XML   *
XML        SELECT PARAM-FILE
XML               ASSIGN TO DYNAMIC XML-FILE
XML               ORGANIZATION IS LINE SEQUENTIAL
XML               ACCESS IS SEQUENTIAL
XML               FILE STATUS IS XML-STAT.
XML   *
XML        SELECT RESPONSE-FILE
XML               ASSIGN TO DYNAMIC OUTPUT-FILE
XML               ORGANIZATION IS LINE SEQUENTIAL
XML               ACCESS IS SEQUENTIAL
XML               FILE STATUS IS OUTPUT-STAT.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
XML   *
XML    FD  PARAM-FILE
XML        RECORD CONTAINS 80 CHARACTERS
XML        DATA RECORD IS PARAM-REC.
XML    01  PARAM-REC.
XML        05  PARAM-LINE                    PIC X(80).
XML        05  PARAM-CHARS                   REDEFINES PARAM-LINE.
XML            10  PARAM-CHAR                PIC X  OCCURS 80 TIMES.
XML   *
XML    FD  RESPONSE-FILE
XML        RECORD CONTAINS 80 CHARACTERS
XML        DATA RECORD IS RESPONSE-REC.
XML    01  RESPONSE-REC.
XML        05  RESPONSE-LINE                 PIC X(80).
XML        05  RESPONSE-CHARS                REDEFINES RESPONSE-LINE.
XML            10  RESPONSE-CHAR             PIC X  OCCURS 80 TIMES.
      *
       FD  SOURCE-FILE
           RECORD CONTAINS 400 CHARACTERS
           DATA RECORD IS SOURCE-REC.
       01  SOURCE-REC                       PIC X(400).
      *
       FD  LOG-FILE
           RECORD CONTAINS 96 CHARACTERS
           DATA RECORD IS LOG-REC.
       01  LOG-REC.
           05  FILLER                       PIC X(4).
           05  LOG-TEXT                     PIC X(88).
           05  FILLER                       PIC X(4).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
XML    77  XML-FILE                          PIC X(80).
       77  INPUT-FILE                        PIC X(80).
XML    77  OUTPUT-FILE                       PIC X(80).
       77  FUNCTION-FILE-NAME                PIC X(80).
       77  REPO-FILE-NAME                    PIC X(80).
       77  LOG-NAME                          PIC X(80).
XML    77  OUTPUT-STAT                       PIC XX.
       77  LOG-STAT                          PIC XX.
XML    77  XML-STAT                          PIC XX.
XML        88  XML-OK                        VALUE '00'.
XML        88  XML-EOF                       VALUE '09'.
       77  INPUT-STAT                        PIC XX.
           88  INPUT-OK                      VALUE '00'.
           88  INPUT-EOF                     VALUE '09'.
XML    77  ERROR-CODE                        PIC 99   VALUE ZERO.
       77  RET-CODE                          PIC XX.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
      *                          Parameter Keywords
           COPY XMLPARS.
      *                          Maximum Limits
           COPY TMAXPARS.
      *                          Metric Tab Parameters
           COPY TMETPARS.
      *                          File Tab Parameters
           COPY TFILPAR.
      *                          Rule Tab Parameters
           COPY TRULPAR.
      *                          Test Case Parameters
           COPY CASEPARS.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 9999.
           05  J                             PIC 9999.
           05  K                             PIC 9999.
           05  LINE-INDEX                    PIC 9999.
      ****************************************************************
      *
      *                          Pointer Fields
       01  POINTERS.
           05  CODE-PTR                      PIC 9999.
           05  LAST-PTR                      PIC 9999.
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  LINE-LNG                      PIC 9999 VALUE 400.
           05  STRING-LNG                    PIC 9999 VALUE 120.
           05  NAME-MAX                      PIC 9999 VALUE 80.
           05  WORD-LNG                      PIC 9999 VALUE 40.
           05  COLUMN-MAX                    PIC 99   VALUE 20.
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  FOUND-SW                      PIC 9    VALUE 0.
      *
       01  COUNTS.
      *
           05 COLUMN-COUNT                   PIC 999  VALUE 0.
           05 SOURCE-NR                      PIC 9999 VALUE 0.
           05 SOURCE-LINE-NR                 PIC 9999 VALUE 0.
      *
      ****************************************************************
       01  VARIABLES.
      *
           05 COLUMN-TITLE                   PIC X(40).
           05 CODE-LINE                      PIC X(400).
           05 CODE-LINE-TAB    REDEFINES     CODE-LINE.
              10  CODE-CHAR                  PIC X OCCURS 400 TIMES.
           05 NAME-STRING.
              10  NAME-CHAR                 PIC X OCCURS 120 TIMES.
           05 SOURCE-STMT                   PIC X(80).
      *
      ****************************************************************
XML        COPY XMLMSGS.
      *
      ****************************************************************
      *  Data for processing XML Files                               *
      ****************************************************************
      *
XML        COPY XMLDATA.cpy.
      *
      ****************************************************************
      *LINKAGE SECTION.
      *                          Parameters
           COPY SYSPARS.
      *
           COPY FUNCPARS.
      *
      ****************************************************************
      *
      *  Parameters to Operating System
      *
       01  CMD-LINE                          PIC X(80).
       01  RESULT                            PIC X  COMP-X.
       01  FUNCT                             PIC X  COMP-X VALUE 35.
       01  CMD-LINED.
           05  CMD-LINE-LNG                  PIC X  COMP-X VALUE ZERO.
      ******************************************************************
      *
      /
       PROCEDURE DIVISION.
      ******************************************************************
      *     Start routine     ******************************************
      ******************************************************************
       TCSSCAN-START SECTION.
      ******************************************************************
      ******************************************************************
      *
XML    SETUP-PARAMETERS.
      *
WEB        ACCEPT CMD-LINE FROM COMMAND-LINE
WEB        MOVE CMD-LINE TO XML-FILE.
TEST  *    DISPLAY 'Enter ParamFile Name !'
TEST  *    ACCEPT XML-FILE.
      *
           MOVE ZERO   TO ERROR-CODE.
           MOVE ZEROES TO XML-STAT.
           OPEN INPUT PARAM-FILE
           IF XML-STAT NOT = ZEROS
              DISPLAY 'Parameter File not found'
              DISPLAY 'Parameter File Name:' XML-FILE
              DISPLAY 'TestComp  could not execute !!!'
              DISPLAY 'Press enter'
              ACCEPT CONTINUE-KEY
              MOVE 1 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           END-IF.
      *
XML        PERFORM READ-PARAM-FILE.
      *
      *
           IF ERROR-CODE = ZERO
              DISPLAY '+----------------------------------------------+'
              DISPLAY '| TCSScan TCS Scanner Version 1.1 of Nov. 2013 |'
              DISPLAY '| Copyright: Harry M. Sneed Budapest, Hungary  |'
              DISPLAY '+----------------------------------------------+'
              DISPLAY '| Product Name     = ' PRODUCT-NAME
              DISPLAY '| System Name      = ' SYSTEM-NAME
              DISPLAY '| Language Version = ' AUDIT-VERSION
              DISPLAY '| Auditing Code    = ' AUDIT-CODE
              DISPLAY '| Export Code      = ' REPO-CODE
              DISPLAY '| Input  Directory = ' INPUT-LIB-NAME
              DISPLAY '| Output Directory = ' OUTPUT-LIB-NAME
              DISPLAY '| Work   Directory = ' WORK-LIB-NAME
              DISPLAY '| Function Table   = ' FUNCTION-FILE-NAME
              DISPLAY '| Press Enter to start Analysis !!'
              DISPLAY '+-------------------------------------------+'
TEST****      ACCEPT CONTINUE-KEY
           ELSE
              MOVE 2 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           END-IF.
      *
      *****************************************************
LOG    OPEN-LOG-FILE.
           MOVE SPACES TO LOG-NAME
           STRING OUTPUT-LIB-NAME    DELIMITED BY SPACE
                 '\'                 DELIMITED BY SIZE
                 SYSTEM-NAME         DELIMITED BY SPACE
                 '.log'              DELIMITED BY SIZE
             INTO LOG-NAME
           END-STRING.
           MOVE ZEROES TO LOG-STAT.
           OPEN OUTPUT LOG-FILE.
           MOVE SPACES TO LOG-REC.
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
           WRITE LOG-REC.
           MOVE '| TCSSCAN Version 1.1 of November 2013' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '| Copyright of Harry M Sneed ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '| Soring Kft, Budapest, Hungary ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '|        Execution Protocol       ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '|                                 ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
LOG        WRITE LOG-REC.
      ***************************************************************
       INIT-COUNTERS.
           MOVE ZEROES TO INDEXES.
           MOVE ZEROES TO COLUMN-COUNT.
           MOVE SPACES TO COLUMN-TITLE.
           PERFORM OPEN-FUNCTION-TABLE.
      ***************************************************************
       PROCESS-ALL-FILES.
XML   *
XML   ** Read all Source File Names from XML File
XML   *
XML        PERFORM UNTIL XML-LABEL = FILES-END
XML                   OR XML-EOF
XML           READ PARAM-FILE
XML           AT END
XML              MOVE '09' TO XML-STAT
XML           NOT AT END
XML              MOVE PARAM-REC TO XML-LINE
XML              PERFORM PARSE-XML
XML              IF ERROR-CODE > ZERO
XML                 MOVE 2 TO ERROR-CODE
XML                 PERFORM RETURN-RESPONSE
XML              END-IF
XML           END-READ
XML           IF XML-STAT = ZEROES AND
XML              XML-LABEL (1:9) = 'Orig_File'
XML              MOVE XML-VALUE TO PATH-NAME
XML              DISPLAY '| Orig Source File = ' PATH-NAME
                 PERFORM GET-ORIGINAL-NAMES
XML           END-IF
XML           IF XML-STAT = ZEROES AND
XML              XML-LABEL (1:5) = FILENR
XML              MOVE SPACES TO INPUT-FILE
XML              STRING INPUT-LIB-NAME   DELIMITED BY SPACE
XML                    '\'               DELIMITED BY SIZE
XML                     XML-VALUE        DELIMITED BY SPACE
XML                 INTO INPUT-FILE
XML              END-STRING
XML              DISPLAY '| Next Source File = ' INPUT-FILE
TEST***          ACCEPT CONTINUE-KEY
XML   *************************************************************
                 MOVE ZERO   TO SOURCE-LINE-NR
                 PERFORM GET-SHORT-NAMES
      *************************************************************
      **      Read and Process next Source Member
      *************************************************************
TEST             DISPLAY 'FileType = ' FILE-TYPE
TEST             ACCEPT CONTINUE-KEY
                 PERFORM OPEN-INPUT-FILE
                 EVALUATE FILE-TYPE
                 WHEN  'CSV'
                 WHEN  'csv'
                    ADD 1 TO SOURCE-NR
                    PERFORM PARSE-CSV-LINE
                 WHEN  'XML'
                 WHEN  'xml'
                    ADD 1 TO SOURCE-NR
                    PERFORM PARSE-XML-LINES
                 WHEN  'ASR'
                 WHEN  'asr'
                 WHEN  'TXT'
                 WHEN  'txt'
                    ADD 1 TO SOURCE-NR
                    PERFORM PARSE-TXT-LINES
                 END-EVALUATE
                 PERFORM CLOSE-INPUT-FILE
XML           END-IF
           END-PERFORM.
           CLOSE PARAM-FILE.
           MOVE 'CL' TO FUNCTION-CODE.
           PERFORM CLOS-FUNCTION-TABLE.
      ***************************************************************
       PARSE-HEADER-LINE.
      ***************************************************************
      ***************************************************************
       END-TCSANAL.
           DISPLAY '| TCSSCAN  is completed!'
           DISPLAY '| Number Sources processed = ' SOURCE-NR
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
           WRITE LOG-REC.
           MOVE    '| TCSSCAN is completed! ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE SPACES TO LOG-TEXT.
           STRING  '| Number Sources processed = ' DELIMITED BY SIZE
                    SOURCE-NR                      DELIMITED BY SIZE
              INTO LOG-TEXT
           WRITE LOG-REC.
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
           WRITE LOG-REC.
           CLOSE LOG-FILE.
TEST       DISPLAY '| Press Enter to Terminate!'
TEST       ACCEPT  CONTINUE-KEY
      *       return Reponse & terminate analysis
           PERFORM RETURN-RESPONSE.
           GOBACK.
      ******************************************************************
      ******************************************************************
      *      Main Loop for each Source Member         ******************
      ******************************************************************
       PROCESS-TESTCASE SECTION.
      ***************************************************************
       OPEN-INPUT-FILE.
           MOVE ZEROES TO INPUT-STAT.
           OPEN INPUT SOURCE-FILE.
           IF INPUT-STAT NOT = ZEROES
              DISPLAY '| Source File cannot be opened' INPUT-STAT
              DISPLAY '| Source File Path = ' INPUT-FILE
              DISPLAY '| Analysis cannot be continued !!!'
              MOVE    '| Source File not found ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Source File Name = ' DELIMITED BY SIZE
                         INPUT-FILE           DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| Path to Source File is too long' TO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| Processing is terminated !!' TO LOG-TEXT
              WRITE LOG-REC
TEST***       DISPLAY ' Press Enter to Continue!'
TEST***       ACCEPT  CONTINUE-KEY
TEST          MOVE 5 TO ERROR-CODE
TEST          PERFORM RETURN-RESPONSE
TEST          GOBACK
           ELSE
              DISPLAY '+--------------------------------------------+'
              DISPLAY '| Source File to be analyzed = ' FILE-NAME
              DISPLAY '+--------------------------------------------+'
              MOVE SPACES TO LOG-TEXT
              STRING  '| Source File analyzed = ' DELIMITED BY SIZE
                        PATH-NAME                 DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
           END-IF.
      ***************************************************************
       PARSE-CSV-LINE.
           MOVE ZEROES TO INPUT-STAT
           MOVE ZEROES TO SWITCHES
           READ SOURCE-FILE
           AT END
              DISPLAY 'TestCase File is empty!!'
              ACCEPT CONTINUE-KEY
              MOVE '09' TO INPUT-STAT
           NOT AT END
              ADD 1 TO SOURCE-LINE-NR
              MOVE SOURCE-REC TO CODE-LINE
TEST***       DISPLAY 'CSV Header Line:'  CODE-LINE
TEST***       ACCEPT CONTINUE-KEY
              MOVE 1 TO CODE-PTR
              PERFORM UNTIL CODE-LINE (CODE-PTR:8) = SPACES
                         OR CODE-PTR > LINE-LNG
                         OR COLUMN-COUNT > COLUMN-MAX
                 UNSTRING CODE-LINE (CODE-PTR:40) DELIMITED BY ';'
                     INTO COLUMN-TITLE
TEST***          DISPLAY 'ColumnTitle: ' COLUMN-TITLE
TEST***          ACCEPT CONTINUE-KEY
                 ADD 1 TO COLUMN-COUNT
                 ADD 1 TO CODE-PTR
                 PERFORM UNTIL CODE-CHAR (CODE-PTR) = ';'
                            OR CODE-LINE (CODE-PTR:8) = SPACES
                            OR CODE-PTR > LINE-LNG
                    ADD 1 TO CODE-PTR
                 END-PERFORM
                 ADD 1 TO CODE-PTR
                 PERFORM CHECK-FUNC-TYPE
              END-PERFORM
           END-READ.
      ***************************************************************
       PARSE-XML-LINES.
           MOVE ZEROES TO INPUT-STAT
           MOVE ZEROES TO SWITCHES
           PERFORM UNTIL INPUT-EOF
              READ SOURCE-FILE
              AT END
                 MOVE '09' TO INPUT-STAT
              NOT AT END
                 ADD 1 TO SOURCE-LINE-NR
                 MOVE SOURCE-REC TO CODE-LINE
              END-READ
           END-PERFORM.
           MOVE ZERO TO LINE-INDEX.
      ***************************************************************
       PARSE-TXT-LINES.
           MOVE ZEROES TO INPUT-STAT
           MOVE ZEROES TO SWITCHES
           PERFORM UNTIL INPUT-EOF
              READ SOURCE-FILE
              AT END
                 MOVE '09' TO INPUT-STAT
              NOT AT END
                 ADD 1 TO SOURCE-LINE-NR
                 MOVE SOURCE-REC TO CODE-LINE
              END-READ
           END-PERFORM.
           MOVE ZERO TO LINE-INDEX.
      ***************************************************************
       CLOSE-INPUT-FILE.
      *    Close Test Case File
           CLOSE SOURCE-FILE.
SN0707     EXIT.
      *
      ******************************************************************
       PROCESS-FUNCTION-TABLE SECTION.
      ***************************************************************
      ***************************************************************
       OPEN-FUNCTION-TABLE.
      * set up function file name
           MOVE ZERO TO FUNCTION-RETC.
           MOVE 'OPEN' TO FUNCTION-IO-CODE.
           MOVE SPACES TO FUNCTION-IO-STRING.
           CALL 'TCSFUNC' USING FUNCTION-IO-PARAMS,
                                FUNCTION-FILE-NAME,
                                LOG-NAME
           IF FUNCTION-RETC NOT = ZERO
              DISPLAY '| Function Table not found '
              DISPLAY '| Function File Name = ' FUNCTION-FILE-NAME
              MOVE    '| Function Table not found ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Function File Name = ' DELIMITED BY SIZE
                         FUNCTION-FILE-NAME     DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| Execution was terminated !' TO LOG-TEXT
              WRITE LOG-REC
TEST***       DISPLAY '| Press Enter to Terminate '
TEST***       ACCEPT  CONTINUE-KEY
              MOVE 3 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
LOG        ELSE
              MOVE    '| Function Table was opened ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Function Table Name = ' DELIMITED BY SIZE
                         FUNCTION-FILE-NAME      DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
           END-IF.
      *
      ***************************************************************
      *                   find Function and get Type
       CHECK-FUNC-TYPE.
TEST***    DISPLAY  ' Check Function Type '
TEST***    DISPLAY  ' Function-Name = ' COLUMN-TITLE
TEST***    ACCEPT CONTINUE-KEY.
           MOVE 'ADD ' TO FUNCTION-IO-CODE
           MOVE COLUMN-TITLE TO FUNCTION-IO-STRING
           MOVE SPACES TO FUNCTION-IO-TYPE
           CALL 'TCSFUNC' USING  FUNCTION-IO-PARAMS
SN0111     IF FUNCTION-RETC > ZERO
SN0111        MOVE SPACES TO FUNCTION-IO-TYPE
SN0111     ELSE
SN0111        CONTINUE
SN0111     END-IF.
TEST***    DISPLAY 'Return-Code=' FUNCTION-RETC
TEST***    DISPLAY 'Func-Code  =' FUNCTION-IO-TYPE
TEST***    ACCEPT CONTINUE-KEY.
      *
      ***************************************************************
       CLOS-FUNCTION-TABLE.
      * set up function file name
           MOVE ZERO TO FUNCTION-RETC.
           MOVE 'CLOS' TO FUNCTION-IO-CODE.
           MOVE SPACES TO FUNCTION-IO-STRING.
           CALL 'TCSFUNC' USING FUNCTION-IO-PARAMS,
                                FUNCTION-FILE-NAME,
                                LOG-NAME.
           EXIT.
      ******************************************************************
      *    XML Initialisation
           COPY XMLINIT.
           EXIT.
      ******************************************************************
      *    XML Name Extraction
           COPY NAMEXTR.
           EXIT.
      ********************** end of source *****************************
