      ******************************************************************
      *   PROGRAM  FOR  PRODUCING A LIST OF TEST CASES                 *
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. TCSSPEC.
      *AUTHOR.     H. SNEED.
      *DATE.       MARCH 2006.
      *VERS.       1.0 of Mar. 2006
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *                  TCS    CHECKING    SPECIFICATION              *
      * Read in Attributes                                             *
      * Initialize TestCase Report                                     *
      * Open TestCase File                                             *
      * Read TestCases                                                 *
      *    Read a TestCase                                             *
      *       print TestCase-ID                                        *
      *       print requirement                                        *
      *       print use cases                                          *
      *       print objects                                            *
      * Eof Testcases                                                  *
      * Close File                                                     *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
      ******************************************************************
       FILE-CONTROL.
      ******************************************************************
      *
      * Testcase Input File
      *
           SELECT  ATTR-FILE
                   ASSIGN TO DYNAMIC ATTR-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS ATTR-STATUS.
      *
           SELECT  CASE-FILE
                   ASSIGN TO DYNAMIC CASE-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS CASE-STATUS.
      *
      * Test Case List
      *
           SELECT  TEST-REPORT
                   ASSIGN TO DYNAMIC TEST-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS TEST-STATUS.
      *
      * Test Case Table
      *
           SELECT  TEST-TABLE
                   ASSIGN TO DYNAMIC TEST-TAB-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS TEST-STATUS.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
      *
       FD  ATTR-FILE
           RECORD CONTAINS 80 CHARACTERS
           DATA RECORD IS ATTR-REC.
       01  ATTR-REC.
           05 ATTR-NUM               PIC 9(4).
           05 FILLER                 PIC X.
           05 ATTR-LNG               PIC 99.
           05 FILLER                 PIC X.
           05 ATTR-TYP               PIC X(4).
           05 FILLER                 PIC X.
           05 ATTR-NAME              PIC X(36).
           05 FILLER                 PIC X(31).
      *
       FD  CASE-FILE
           RECORD CONTAINS 244 CHARACTERS
           DATA RECORD IS CASE-REC.
       01  CASE-REC.
           05  CASE-REC-NR                   PIC 9999.
           05  CASE-REC-LABEL                PIC X(40).
           05  CASE-REC-VALUE                PIC X(200).
      *
       FD  TEST-REPORT
           RECORD CONTAINS 104 CHARACTERS
           DATA RECORD IS TEST-REPORT-REC.
      *
       01  TEST-REPORT-REC                   PIC X(104).
      *
       FD  TEST-TABLE
           RECORD CONTAINS 160 CHARACTERS
           DATA RECORD IS TEST-TABLE-REC.
      *
       01  TEST-TABLE-REC                    PIC X(160).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
       77  TEST-FILE-NAME                    PIC X(80).
       77  TEST-TAB-NAME                     PIC X(80).
       77  ATTR-FILE-NAME                    PIC X(80).
       77  THIS-FILE-NAME                    PIC X(80).
       77  TEST-STATUS                       PIC XX.
       77  CASE-STATUS                       PIC XX.
           88  CASE-OK                       VALUE '00'.
           88  CASE-EOF                      VALUE '09'.
       77  ATTR-STATUS                       PIC XX.
           88  ATTR-OK                       VALUE '00'.
           88  ATTR-EOF                      VALUE '09'.
       01  ATTR-STAT                         REDEFINES ATTR-STATUS.
           05  FILLER                        PIC X.
           05  ATTR-STAT-2                   PIC X COMP-X.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 9999.
           05  J                             PIC 9999.
           05  K                             PIC 9999.
      *
      ****************************************************************
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  LINE-LNG                      PIC 9999 VALUE 125.
           05  NAME-LNG                      PIC 9999 VALUE 40.
           05  PAGE-MAX                      PIC 99   VALUE 60.
      *                          Array Lengths
      ****************************************************
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  FOUND-SW                      PIC 9    VALUE 0.
      *
      *
       01  COUNTS.
           05  LINE-COUNT               PIC 99     VALUE ZERO.
           05  DEF-COUNT                PIC 99     VALUE ZERO.
           05  PAGE-COUNT               PIC 9999   VALUE ZERO.
      *
       01  LOCAL-VARIABLES.
           05  TCS-ID                   PIC X(20).
           05  TCS-ID-LAST              PIC X(20).
           05  TCS-GOAL                 PIC X(80).
           05  CURRENT-REQ-TYPE         PIC X(8)  VALUE 'REQUIRE'.
           05  CURRENT-RECNR            PIC 9999  VALUE ZEROES.
           05  LAST-REC-NR              PIC 9999  VALUE ZEROES.
      *
      ****************************************************************
      *
       77  ATTR-NR                      PIC 99 VALUE ZERO.
       77  ATTR-MAX                     PIC 99 VALUE 40.
       01  ATTR-TAB.
           05  ATTR-ENTRY               OCCURS 40 TIMES.
               10  ATTR-TYPE            PIC X(4).
               10  ATTR-LABEL           PIC X(40).
      *
       77  CASE-NR                      PIC 99 VALUE ZERO.
       77  CASE-MAX                     PIC 99 VALUE 40.
       01  CASE-TAB.
           05  CASE-ENTRY               OCCURS 40 TIMES.
               10  CASE-LABEL           PIC X(40).
               10  CASE-VALUE           PIC X(60).
      *
      ***************************************************************
      *
           COPY REPOPARM.
      *
      ***************************************************************
      * Data for converting lower case characters to upper case
      ***************************************************************
      *
       77  STRING-LNG                    PIC 9(4) BINARY VALUE 256.
       77  COL-I                         PIC 9(4) BINARY.
       01  CHAR-STRING.
           02 PRG-CH                     PIC X OCCURS 256 TIMES.
       01  UPPER-CASE-CHARS              PIC X(26) VALUE
                                        'ABCDEFGHIJKLMNOPQRSTUVWXYZ'.
       01  UPPER-CASE-TAB                REDEFINES UPPER-CASE-CHARS.
           05 UPPER-CHARS               OCCURS 26 TIMES INDEXED BY UP-I.
              10 UPPER-C                 PIC X.
       01  LOWER-CASE-CHARS              PIC X(26) VALUE
                                        'abcdefghijklmnopqrstuvwxyz'.
       01  LOWER-CASE-TAB                REDEFINES LOWER-CASE-CHARS.
           05 LOWER-CHARS               OCCURS 26 TIMES INDEXED BY LO-I.
              10 LOWER-C                 PIC X.
      *
      ****************************************************************
      *   Data for printing out Test Cases
      *
       01  YYMMDD.
           05  YY                       PIC 99.
           05  MM                       PIC 99.
           05  DD                       PIC 99.

      *
       01  NEWPAGE-LINE.
           05  FILLER                   PIC X(104) VALUE '/'.
      *
       01  SEPARATOR-LINE.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(102) VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
      *
       01  SEPARATOR-LINE-1.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(20)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(40)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(40)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
      *
       01  BLANK-LINE.
           05  FILLER                   PIC X(01)  VALUE '|'.
           05  FILLER                   PIC X(102) VALUE SPACES.
           05  FILLER                   PIC X(01)  VALUE '|'.
      *
       01  TCS-REC-HEADER-1.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  FILLER                   PIC X(16)  VALUE SPACES.
           05  TCS-HEADER-1             PIC X(50)  VALUE
ENG           ' T E S T A N A L  T E S T C A S E   A U D I T O R '.
           05  FILLER                   PIC X(30)  VALUE SPACES.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  TCS-REC-HEADER-2.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  TCS-HEADER-2             PIC X(50)  VALUE
ENG           '  T E S T     C A S E    R E P O R T   '.
           05  FILLER                   PIC X(22)  VALUE SPACES.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  REC-HEADER-3.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  REC-HEADER-3-1           PIC X(10)  VALUE
ENG           'TESTPROC: '.
           05  FILE-NAME-OUT            PIC X(72)  VALUE SPACES.
           05  REC-HEADER-3-2           PIC X(6)   VALUE
ENG           'DATE: '.
           05  DATUM-OUT.
               10  DAY-OUT              PIC 99.
               10  FILLER               PIC X VALUE '.'.
               10  MONTH-OUT            PIC 99.
               10  FILLER               PIC X VALUE '.'.
               10  YEAR-OUT             PIC 99.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  REC-HEADER-4.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  REC-HEADER-4-1           PIC X(10)  VALUE
ENG           'SYSTEM:   '.
           05  SYST-NAME-OUT            PIC X(72)  VALUE SPACES.
           05  REC-HEADER-4-2           PIC X(6)   VALUE
ENG           'PAGE: '.
           05  SEITE-OUT                PIC ZZZZZZZ9.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  TCS-TITLE-LINE.
           05  FILLER                   PIC X      VALUE '|'.
           05  TCS-TITLE-1              PIC X(20)  VALUE
              'Test Case   tests '.
           05  FILLER                   PIC X      VALUE '|'.
           05  TCS-TITLE-2              PIC X(40)  VALUE
              '  Requirement/UseCase '.
           05  FILLER                   PIC X      VALUE '|'.
           05  TCS-TITLE-4              PIC X(40)  VALUE
              '  Logical Object  '.
           05  FILLER                   PIC X      VALUE '|'.
      *
       01  TCS-DATA-LINE.
           05  FILLER                   PIC X      VALUE '|'.
           05  TCS-TEST-OUT             PIC X(20).
           05  FILLER                   PIC X      VALUE '|'.
           05  TCS-REQU-OUT             PIC X(40).
           05  FILLER                   PIC X      VALUE '|'.
           05  TCS-OBJT-OUT             PIC X(40).
           05  FILLER                   PIC X      VALUE '|'.
      *
       01  TCS-FOOT-LINE-1.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  TCS-FOOT-1       PIC X(48)  VALUE
ENG           'Number of Test Cases   to be tested         =   '.
           05  TESTS-TOTAL      PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  TCS-FOOT-LINE-2.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  TCS-FOOT-2       PIC X(48)  VALUE
ENG           'Number of TestCase/TextSection Relations    =   '.
           05  TEXTS-TOTAL      PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  TCS-FOOT-LINE-3.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  TCS-FOOT-3       PIC X(48)  VALUE
ENG           'Number of TestCase/Requirement Relations    =   '.
           05  REQUS-TOTAL      PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  TCS-FOOT-LINE-4.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  TCS-FOOT-4       PIC X(48)  VALUE
ENG           'Number of TestCase/UseCase  Relations       =   '.
           05  CASES-TOTAL      PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  TCS-FOOT-LINE-5.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  REQ-FOOT-5       PIC X(48)  VALUE
ENG           'Number of TestCase/Data Object Relations    =   '.
           05  OBJTS-TOTAL      PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  TCS-CSV-TITLE-LINE.
           05  CSV-TITLE-1      PIC X(09)  VALUE 'TestCase;'.
           05  CSV-TITLE-2      PIC X(20)  VALUE 'Requirement/UseCase;'.
           05  CSV-TITLE-4      PIC X(14)  VALUE 'LogicalObject;'.
           05  CSV-TITLE-5      PIC X(11)  VALUE 'TestObject;'.
           05  CSV-TITLE-6      PIC X(10)  VALUE 'ObjectType'.
      *
       01  TEST-TABLE-LINE      PIC X(160) VALUE SPACES.
      *
      ****************************************************************
       LINKAGE SECTION.
      ****************************************************************
       77  SYSTEM-NAME                       PIC X(8).
       77  FILE-NAME                         PIC X(8).
       77  TEST-PROC-NAME                    PIC X(40).
       77  CASE-TAB-NAME                     PIC X(80).
       77  SPECS-LIB-NAME                    PIC X(80).
       77  TABLE-LIB-NAME                    PIC X(80).
       77  EXPO-LIB-NAME                     PIC X(80).
       77  FUNC-TAB-NAME                     PIC X(80).
       77  ORIG-TAB-NAME                     PIC X(80).
       77  REPO-CODE                         PIC XX.
       77  RET-CODE                          PIC 99.
      *
       01  TCS-METRICS.
           05  TCS-NR                        PIC 9999 VALUE ZERO.
           05  TCS-TXT-NR                    PIC 9999 VALUE ZERO.
           05  TCS-REQ-NR                    PIC 9999 VALUE ZERO.
           05  TCS-UCS-NR                    PIC 9999 VALUE ZERO.
           05  TCS-OBJ-NR                    PIC 9999 VALUE ZERO.
      *
      ****************************************************************
      /
       PROCEDURE DIVISION USING SYSTEM-NAME,
                                FILE-NAME,
                                TEST-PROC-NAME,
                                CASE-TAB-NAME,
                                SPECS-LIB-NAME,
                                TABLE-LIB-NAME,
                                EXPO-LIB-NAME,
                                FUNC-TAB-NAME,
                                ORIG-TAB-NAME,
                                REPO-CODE,
                                RET-CODE,
                                TCS-METRICS.
      ******************************************************************
      *           M A I N   P R O C E D U R E
      ******************************************************************
      *
       TCSSPEC-MAIN-PROCESSING SECTION.
      *
      ******************************************************************
      *
       START-TCSSPEC.
      *
           DISPLAY '+---------------------------------------------+'
           DISPLAY '| TCSSPEC List  Program Vers 1.2 of Oct. 2008 |'
           DISPLAY '+---------------------------------------------+'
           DISPLAY '| System Name           = ' SYSTEM-NAME
           DISPLAY '| File   Name           = ' FILE-NAME
           DISPLAY '| Test Procedure Name   = ' TEST-PROC-NAME
           DISPLAY '| Repository Code       = ' REPO-CODE
           DISPLAY '| TestCase Report Lib   = ' SPECS-LIB-NAME
           DISPLAY '| TestCase Tables Lib   = ' TABLE-LIB-NAME
           DISPLAY '+---------------------------------------------+'
TEST       DISPLAY '| Press Enter to continue !!'
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
       READ-ATTRIBUTES.
      *
           MOVE FUNC-TAB-NAME TO ATTR-FILE-NAME
           MOVE ZEROES TO ATTR-STATUS
           OPEN INPUT ATTR-FILE
           IF ATTR-STATUS NOT = ZEROES
              DISPLAY '| Attribute File could not be opened !!'
              DISPLAY '| Attribute File Name = ' ATTR-FILE-NAME
              DISPLAY '| Press Enter to continue !'
              ACCEPT CONTINUE-KEY
              MOVE 21 TO RET-CODE
              GOBACK
           END-IF
TEST***    DISPLAY '| Read in Attribute File !!'
TEST***    ACCEPT CONTINUE-KEY
           MOVE SPACES TO ATTR-TAB
           MOVE ZERO TO ATTR-NR
           PERFORM UNTIL ATTR-STATUS NOT = ZERO
              READ ATTR-FILE
              AT END
                 MOVE '09' TO ATTR-STATUS
              NOT AT END
                 MOVE ATTR-REC TO CHAR-STRING
                 PERFORM TRANSLATE-CHARS
                 MOVE CHAR-STRING TO ATTR-REC
                 ADD 1 TO ATTR-NR
                 MOVE ATTR-TYP TO ATTR-TYPE (ATTR-NR)
                 MOVE ATTR-NAME TO ATTR-LABEL (ATTR-NR)
TEST***          DISPLAY 'CASE-LABEL:' ATTR-TYPE (ATTR-NR)
TEST***          DISPLAY 'CASE-VALUE:' ATTR-LABEL (ATTR-NR)
TEST***          ACCEPT CONTINUE-KEY
              END-READ
           END-PERFORM.
           CLOSE ATTR-FILE.
           DISPLAY '| ' ATTR-NR ' Attributes were read !'
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      ******************************************************************
      *
       OPEN-TESTCASE-REPORT.
      *
           MOVE ZEROES TO COUNTS
           MOVE ZEROES TO SWITCHES
           MOVE ZEROES TO TEST-STATUS
           MOVE SPACES TO TEST-FILE-NAME
           STRING SPECS-LIB-NAME            DELIMITED BY SPACE
                 '\'                        DELIMITED BY SIZE
                  FILE-NAME                 DELIMITED BY SPACE
                 '.tcs'                     DELIMITED BY SIZE
              INTO TEST-FILE-NAME
TEST***    DISPLAY 'TCSSPEC Output File in:' TEST-FILE-NAME
TEST***    ACCEPT CONTINUE-KEY
           OPEN OUTPUT TEST-REPORT.
TAB        MOVE SPACES TO TEST-TAB-NAME
TEST       STRING SPECS-LIB-NAME            DELIMITED BY SPACE
TAB              '\'                        DELIMITED BY SIZE
TAB               FILE-NAME                 DELIMITED BY SPACE
TEST             '.csv'                     DELIMITED BY SIZE
TAB           INTO TEST-TAB-NAME
TEST       DISPLAY 'TCSSPEC Output Table in:' TEST-TAB-NAME
TEST***    ACCEPT CONTINUE-KEY
TAB        OPEN OUTPUT TEST-TABLE.
TAB        MOVE TCS-CSV-TITLE-LINE TO TEST-TABLE-REC.
TAB        WRITE TEST-TABLE-REC.
      ******************************************************************
           MOVE ZERO  TO LINE-COUNT
           MOVE 1     TO PAGE-COUNT
           ACCEPT YYMMDD FROM DATE
           MOVE YY TO YEAR-OUT
           MOVE MM TO MONTH-OUT
           MOVE DD TO DAY-OUT.
           MOVE PAGE-COUNT   TO SEITE-OUT
           MOVE SYSTEM-NAME  TO SYST-NAME-OUT
           MOVE TEST-PROC-NAME TO FILE-NAME-OUT
           MOVE NEWPAGE-LINE TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE SEPARATOR-LINE TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE BLANK-LINE TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE TCS-REC-HEADER-1 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE TCS-REC-HEADER-2 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE BLANK-LINE TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE REC-HEADER-3 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE REC-HEADER-4 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE SEPARATOR-LINE-1 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE TCS-TITLE-LINE TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE SEPARATOR-LINE-1 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC.
           ADD 11 TO LINE-COUNT.
      *
      ******************************************************************
      *
       OPEN-TESTCASE-FILE.
           MOVE CASE-TAB-NAME TO CASE-FILE-NAME
           MOVE ZEROES TO CASE-STATUS
           OPEN INPUT CASE-FILE
           IF CASE-STATUS NOT = ZEROES
              DISPLAY '| TestCase File could not be opened !!'
              DISPLAY '| TestCase File Name = ' CASE-FILE-NAME
              DISPLAY '| Press Enter to continue !'
              ACCEPT CONTINUE-KEY
              MOVE 22 TO RET-CODE
              GOBACK
           END-IF.
           PERFORM CALL-REPOREF-PROC.
      *
      ******************************************************************
      *
       TESTCASE-PROCESSING.
      *
TEST***    DISPLAY 'Read in TestCase File !!'
TEST***    ACCEPT CONTINUE-KEY
           MOVE ZERO TO CURRENT-RECNR
           MOVE ZEROS TO CASE-STATUS
           MOVE ZEROS TO TCS-METRICS.
           READ CASE-FILE
           AT END
              MOVE '09' TO CASE-STATUS
           NOT AT END
              MOVE CASE-REC-NR TO CURRENT-RECNR
           END-READ
           PERFORM UNTIL CASE-STATUS NOT = ZERO
              MOVE CASE-REC TO CHAR-STRING
              PERFORM TRANSLATE-CHARS
              MOVE CHAR-STRING TO CASE-REC
              MOVE ZERO TO CASE-NR
              PERFORM UNTIL CASE-REC-NR NOT = CURRENT-RECNR
                         OR CASE-EOF
                 IF CASE-NR < CASE-MAX
                    ADD 1 TO CASE-NR
                    MOVE CASE-REC-LABEL TO CASE-LABEL (CASE-NR)
                    MOVE CASE-REC-VALUE TO CASE-VALUE (CASE-NR)
TEST***             DISPLAY 'CASE-LABEL:' CASE-LABEL (CASE-NR)
TEST***             DISPLAY 'CASE-VALUE:' CASE-VALUE (CASE-NR)
TEST***             ACCEPT CONTINUE-KEY
                 END-IF
                 MOVE TCS-ID TO TCS-ID-LAST
                 READ CASE-FILE
                 AT END
                    MOVE '09' TO CASE-STATUS
                 NOT AT END
                    MOVE CASE-REC TO CHAR-STRING
                    PERFORM TRANSLATE-CHARS
                    MOVE CHAR-STRING TO CASE-REC
TEST***             DISPLAY 'CASE-REC:' CASE-REC
TEST***             DISPLAY 'This_RecNr: ' CASE-REC-NR
TEST***             DISPLAY 'Current_RecNr: ' CURRENT-RECNR
TEST***             ACCEPT CONTINUE-KEY
                 END-READ
              END-PERFORM
              ADD 1 TO TCS-NR
              PERFORM CHECK-ATTRIBUTES
              PERFORM WRITE-TESTCASE
              MOVE CASE-REC-NR TO CURRENT-RECNR
              DISPLAY '| TestCase Nr. ' CURRENT-RECNR
TEST***       ACCEPT CONTINUE-KEY
           END-PERFORM.
TEST***    DISPLAY '| ' NR-TEST-CASES ' Test Cases were read !'
TEST***    ACCEPT CONTINUE-KEY.
      *
      ******************************************************************
      *
       CLOSE-TESTCASE-REPORT.
           MOVE TCS-NR TO TESTS-TOTAL
           MOVE TCS-TXT-NR TO TEXTS-TOTAL
           MOVE TCS-REQ-NR TO REQUS-TOTAL
           MOVE TCS-UCS-NR TO CASES-TOTAL
           MOVE TCS-OBJ-NR TO OBJTS-TOTAL
           MOVE SEPARATOR-LINE-1 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE BLANK-LINE TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE TCS-FOOT-LINE-1 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE TCS-FOOT-LINE-2 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE TCS-FOOT-LINE-3 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE TCS-FOOT-LINE-4 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE TCS-FOOT-LINE-5 TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE BLANK-LINE TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           MOVE SEPARATOR-LINE TO TEST-REPORT-REC
           WRITE TEST-REPORT-REC
           ADD 8 TO LINE-COUNT.
           CLOSE TEST-REPORT.
TAB        CLOSE TEST-TABLE.
      *
      ******************************************************************
      *
       TESTCASE-CLOSING.
           CLOSE CASE-FILE
           DISPLAY '+-----------------------------------------+'
           DISPLAY '| TestCases were documented !!'
           DISPLAY '| Number of TestCases             = ' TCS-NR
           DISPLAY '| Number of TestCase/Text    Rels = ' TCS-TXT-NR
           DISPLAY '| Number of TestCase/Require Rels = ' TCS-REQ-NR
           DISPLAY '| Number of TestCase/UseCase Rels = ' TCS-UCS-NR
           DISPLAY '| Number of TestCase/Object  Rels = ' TCS-OBJ-NR
           DISPLAY '+-----------------------------------------+'
TEST       DISPLAY '| Press enter to continue'
TEST       ACCEPT CONTINUE-KEY.
           GOBACK.
      *
      *           S U B     P R O C E D U R E S
      ******************************************************************
       TCSSPEC-SUB-PROCESSING SECTION.
      ******************************************************************
      *
       CHECK-ATTRIBUTES.
      *
           MOVE SPACES TO TCS-TEST-OUT
           MOVE SPACES TO TCS-REQU-OUT
           MOVE SPACES TO TCS-OBJT-OUT
           MOVE 1 TO I
           PERFORM UNTIL I > ATTR-NR
                      OR I > ATTR-MAX
              MOVE 1 TO J
              MOVE ZERO TO FOUND-SW
              PERFORM UNTIL J > CASE-NR
                         OR J > CASE-MAX
                         OR FOUND-SW = 1
                 IF CASE-LABEL (J) = ATTR-LABEL (I)
                    MOVE 1 TO FOUND-SW
                 ELSE
                    ADD 1 TO J
                 END-IF
              END-PERFORM
              IF FOUND-SW = 1
                 PERFORM UNTIL CASE-LABEL (J) NOT = ATTR-LABEL (I)
                            OR J > CASE-NR OR J > CASE-MAX
                    EVALUATE ATTR-TYPE (I)
                    WHEN 'RTYP'
                       MOVE CASE-VALUE (J) TO CURRENT-REQ-TYPE
                    WHEN 'ID '
                       MOVE CASE-VALUE (J) TO TCS-ID
                       MOVE CASE-VALUE (J) TO TCS-TEST-OUT
                       MOVE SPACES TO TCS-REQU-OUT
                       PERFORM CALL-REPOREF-CASE
TAB                    MOVE SPACES TO TEST-TABLE-LINE
TAB                    STRING TCS-TEST-OUT         DELIMITED BY SPACE
TAB                          ';'                   DELIMITED BY SIZE
TAB                       INTO TEST-TABLE-LINE
TAB                    END-STRING
                    WHEN 'REQU'
                       MOVE CASE-VALUE (J) TO TCS-REQU-OUT
                       ADD 1 TO TCS-REQ-NR
                       PERFORM CALL-REPOREF-REQUIREMENT
TAB                    MOVE SPACES TO TEST-TABLE-REC
TAB                    STRING TEST-TABLE-LINE      DELIMITED BY SPACE
TAB                       TCS-REQU-OUT             DELIMITED BY SPACE
TAB                          ';'                   DELIMITED BY SIZE
TAB                       INTO TEST-TABLE-REC
TAB                    END-STRING
TAB                    MOVE TEST-TABLE-REC TO TEST-TABLE-LINE
                    WHEN 'OBJT'
                       IF CASE-LABEL (J) = CASE-LABEL (J - 1)
                          MOVE CASE-VALUE (J - 1) TO TCS-OBJT-OUT
                       END-IF
                       MOVE CASE-VALUE (J) TO TCS-OBJT-OUT
                       IF TCS-OBJT-OUT (3:4) NOT = SPACES
                          ADD 1 TO TCS-OBJ-NR
                          PERFORM CALL-REPOREF-OBJECT
                       END-IF
TAB                    PERFORM WRITE-TESTCASE
                    WHEN 'CASE'
                       MOVE CASE-VALUE (J) TO TCS-REQU-OUT
                       ADD 1 TO TCS-UCS-NR
                       PERFORM CALL-REPOREF-USECASE
TAB                    STRING TEST-TABLE-LINE      DELIMITED BY SPACE
TAB                       TCS-REQU-OUT             DELIMITED BY SPACE
TAB                          ';'                   DELIMITED BY SIZE
TAB                       INTO TEST-TABLE-REC
TAB                    END-STRING
TAB                    MOVE TEST-TABLE-REC TO TEST-TABLE-LINE
                    WHEN 'GOAL'
                       MOVE CASE-VALUE (J) TO TCS-GOAL
                       IF TCS-GOAL (3:4) NOT = SPACES
                          PERFORM CALL-REPOREF-GOAL
                       END-IF
                    WHEN OTHER
                       CONTINUE
                    END-EVALUATE
                    ADD 1 TO J
                 END-PERFORM
              END-IF
              ADD 1 TO I
           END-PERFORM.
           EXIT.
      *
       WRITE-TESTCASE.
      *
           IF TCS-DATA-LINE NOT = TEST-REPORT-REC
              MOVE TCS-DATA-LINE TO TEST-REPORT-REC
              WRITE TEST-REPORT-REC
              ADD 1 TO LINE-COUNT
TAB           MOVE SPACES TO TEST-TABLE-REC
TAB           STRING TEST-TABLE-LINE       DELIMITED BY SPACE
TAB                  TCS-OBJT-OUT          DELIMITED BY SPACE
TAB                 ';'                    DELIMITED BY SIZE
TAB                 '$TestObjectName'      DELIMITED BY SIZE
TAB                 ';'                    DELIMITED BY SIZE
TAB                 '$TestObjectType '     DELIMITED BY SIZE
TAB              INTO TEST-TABLE-REC
TAB           END-STRING
TAB           WRITE TEST-TABLE-REC
           END-IF.
           EXIT.
      *
      ***************************************************************
      * Procedure for converting lower case characters to upper case
      ***************************************************************
      *
       TRANSLATE-CHARS.
           MOVE 1 TO COL-I
           PERFORM  UNTIL COL-I > STRING-LNG
              EVALUATE TRUE
                WHEN PRG-CH (COL-I) < 'a' OR PRG-CH (COL-I) > 'z'
                   ADD 1 TO COL-I
                WHEN OTHER
                   SET LO-I TO 1
                   SEARCH LOWER-CHARS VARYING LO-I
                     WHEN LOWER-C (LO-I) = PRG-CH (COL-I)
                       SET UP-I TO LO-I
                   END-SEARCH
                   MOVE UPPER-C (UP-I) TO PRG-CH (COL-I)
                   ADD 1 TO COL-I
               END-EVALUATE
           END-PERFORM.
           EXIT.
      *
      ******************************************************************
       REPOSITORY-ACCESS SECTION.
      ******************************************************************
      *
       CALL-REPOREF-PROC.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE 'SYST'    TO REF-BASE-TYPE
REPO          MOVE SYSTEM-NAME TO REF-BASE-NAME
REPO          MOVE 'OWNS'    TO REF-RELATION
REPO          MOVE 'PROC'    TO REF-TARGET-TYPE
REPO          MOVE TEST-PROC-NAME TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
      *
REPO          MOVE 'PROC'   TO REF-BASE-TYPE
REPO          MOVE TEST-PROC-NAME TO REF-BASE-NAME
REPO          MOVE 'IS_A'   TO REF-RELATION
REPO          MOVE 'PROC'   TO REF-TARGET-TYPE
REPO          MOVE ORIG-TAB-NAME TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
REPO       END-IF.
      *
      *****************************************************************
       CALL-REPOREF-CASE.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE 'PROC'       TO REF-BASE-TYPE
REPO          MOVE TEST-PROC-NAME TO REF-BASE-NAME
REPO          MOVE 'OWNS'       TO REF-RELATION
REPO          MOVE 'TEST'       TO REF-TARGET-TYPE
REPO          MOVE TCS-TEST-OUT TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
REPO       END-IF.
      *
      *****************************************************************
       CALL-REPOREF-TEXT.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE 'TEST'    TO REF-BASE-TYPE
REPO          MOVE TCS-TEST-OUT TO REF-BASE-NAME
REPO          MOVE 'TEST'    TO REF-RELATION
REPO          MOVE 'TEXT'    TO REF-TARGET-TYPE
REPO          MOVE TCS-REQU-OUT TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
REPO       END-IF.
      *
      *****************************************************************
       CALL-REPOREF-REQUIREMENT.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE 'TEST'    TO REF-BASE-TYPE
REPO          MOVE TCS-TEST-OUT TO REF-BASE-NAME
REPO          MOVE 'TEST'    TO REF-RELATION
REPO          MOVE 'REQU'    TO REF-TARGET-TYPE
REPO          MOVE TCS-REQU-OUT TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
REPO       END-IF.
      *
      *****************************************************************
       CALL-REPOREF-USECASE.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE 'TEST'    TO REF-BASE-TYPE
REPO          MOVE TCS-TEST-OUT TO REF-BASE-NAME
REPO          MOVE 'TEST'    TO REF-RELATION
REPO          MOVE 'CASE'    TO REF-TARGET-TYPE
REPO          MOVE TCS-REQU-OUT TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
REPO       END-IF.
      *
      *****************************************************************
       CALL-REPOREF-OBJECT.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE 'TEST'    TO REF-BASE-TYPE
REPO          MOVE TCS-TEST-OUT TO REF-BASE-NAME
REPO          MOVE 'TEST'    TO REF-RELATION
REPO          MOVE 'DATA'    TO REF-TARGET-TYPE
REPO          MOVE TCS-OBJT-OUT TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
REPO       END-IF.
      *
      *****************************************************************
      *
       CALL-REPOREF-GOAL.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE 'TEST'   TO REF-BASE-TYPE
REPO          MOVE TCS-ID   TO REF-BASE-NAME
REPO          MOVE 'IS_A'   TO REF-RELATION
REPO          MOVE 'TEST'   TO REF-TARGET-TYPE
REPO          MOVE TCS-GOAL TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
REPO       END-IF.
      *
      ********************** end of source *****************************
