      *****************************************************************
      *
       SOFTMESS-CALCULATION SECTION.
      *
      *****************************************************************
      *
       COMPUTE-SIZE-METRICS.
      *
      ******************************************************************
      *
      *  Compute Values of Export Metrics
      *
      ******************************************************************
      *
      *
      *  compute Data-Points
      *
           COMPUTE DATA-POINTS =  (NR-TARGET-OBJECTS * 4)
                               +  (NR-INPUTS * 1)
                               +  (NR-OUTPUTS * 1).
      *
      *  compute Function-Points
      *
           COMPUTE FUNC-POINTS =  (NR-TARGET-FUNCS * 15)
                               +  (NR-TARGET-OBJECTS * 3)
      *
      *
      *  compute Test-Points
      *
           COMPUTE TEST-POINTS = (NR-TEST-PROCS * 2)
                               + (NR-TEST-CASES * 1).
      ***************************************************************
      *
       COMPUTE-COMPLEXITY-METRICS.
      *
      * 1) Calculate Data Complexity according to Chapin Metric
      *
           COMPUTE THIS-VALUE = (NR-INPUTS + NR-INPUTS)
                              / ((NR-INPUTS * 1) + (NR-OUTPUTS * 2))
           MOVE THIS-VALUE TO DATA-COMPLEXITY
           IF DATA-COMPLEXITY >  0.920
              MOVE 0.920 TO DATA-COMPLEXITY
           END-IF
           IF DATA-COMPLEXITY <  0.080
              MOVE 0.080 TO DATA-COMPLEXITY
           END-IF.
      *
      * 2) Calculate Test Density according to Sneed Metric
      *
           COMPUTE THIS-VALUE =  NR-TARGET-OBJECTS
                              /  NR-TEST-CASES
           COMPUTE DENSITY = 1 - THIS-VALUE
           MOVE THIS-VALUE TO DENSITY.
           IF DENSITY > 0.920
              MOVE 0.920 TO DENSITY
           END-IF
           IF DENSITY < 0.080
              MOVE 0.080 TO DENSITY
           END-IF.
      *
      * 3) Calculate Test Intensity  according to Sneed Metric
      *
           COMPUTE THIS-VALUE = (NR-TARGET-FUNCS
                              +  NR-TARGET-REQUS)
                              /  NR-TEST-CASES
           COMPUTE INTENSITY = 1 - THIS-VALUE
           IF INTENSITY > 0.920
              MOVE 0.920 TO INTENSITY
           END-IF
           IF INTENSITY < 0.080
              MOVE 0.080 TO INTENSITY
           END-IF.
      *
      * 4) Calculate Test Volumne    according to Sneed Metric
      *
           COMPUTE THIS-VALUE = (NR-TEST-CASES * 4)
                              /  NR-TEST-ATTRS
           COMPUTE VOLUMNE  = 1 - THIS-VALUE
           IF VOLUMNE > 0.920
              MOVE 0.920 TO VOLUMNE
           END-IF
           IF VOLUMNE < 0.080
              MOVE 0.080 TO VOLUMNE
           END-IF.
      *
      * 5) Calculate average Complexity
      *
      *    Compute K = Number of Complexity Metrics
      *    Compute COMP-SUM as sum of selected Complexity Metrics
      *
           SET K TO 4.
           COMPUTE COMP-SUM = INTENSITY
                            + DENSITY
                            + VOLUMNE
                            + DATA-COMPLEXITY.
      *
           COMPUTE TEST-COMPLEXITY  =  COMP-SUM / K.
      *
      ****************************************************************
      *
       COMPUTE-QUALITY-METRICS.
      *
      * 1) Calculate Maintainability according to Sneed
      *
           COMPUTE THIS-VALUE  =  NR-TARGET-OBJECTS
                               /  NR-TEST-CASES
           MOVE THIS-VALUE TO MAINTAINABILITY
           IF MAINTAINABILITY > 0.960
              MOVE 0.960 TO MAINTAINABILITY
           END-IF
           IF MAINTAINABILITY < 0.080
              MOVE 0.080 TO MAINTAINABILITY
           END-IF.
      *
      * 2) Calculate Reusability according to Sneed
      *
           COMPUTE THIS-VALUE  = NR-TEST-CASES-AUTO
                               / NR-TEST-CASES
           MOVE THIS-VALUE TO REUSABILITY
           IF REUSABILITY > 0.960
              MOVE 0.960 TO REUSABILITY
           END-IF
           IF REUSABILITY < 0.080
              MOVE 0.080 TO REUSABILITY
           END-IF.
      *
      * 3) Calculate Coverage according to TestCases per Function
      *
           COMPUTE THIS-VALUE = (NR-TESTED-FUNCS + NR-TESTED-REQUS)
                              / (NR-TARGET-FUNCS + NR-TARGET-REQUS)
           MOVE THIS-VALUE TO COVERAGE
           IF COVERAGE  >  0.999
              MOVE 0.999 TO COVERAGE
           END-IF
           IF COVERAGE  <  0.010
              MOVE 0.010 TO COVERAGE
           END-IF.
      *
      * 4) Calculate Conformity according to Deficiency Count
      *
SN0203     COMPUTE WEIGHTED-DEFI-NR = (NR-MAJOR-DEFIS * 1.5)
                                    + (NR-MEDIA-DEFIS * 1)
                                    + (NR-MINOR-DEFIS * 0.5)
           IF WEIGHTED-DEFI-NR > NR-TEST-ATTRS
              MOVE NR-TEST-ATTRS TO WEIGHTED-DEFI-NR
           END-IF
SN0200     COMPUTE THIS-VALUE =  WEIGHTED-DEFI-NR / NR-TEST-ATTRS
SN0200     IF THIS-VALUE > 1
SN0200        MOVE 0.999 TO CONFORMITY
SN0200     ELSE
SN0200        IF THIS-VALUE < 0
SN0200           MOVE 0.001 TO CONFORMITY
SN0200        ELSE
SN0200           COMPUTE CONFORMITY  =  1 - THIS-VALUE
SN0200        END-IF
SN0200     END-IF.
           IF CONFORMITY > 0.999
              MOVE 0.999 TO CONFORMITY
           END-IF
           IF CONFORMITY < 0.001
              MOVE 0.001 TO CONFORMITY
           END-IF.
      *
      * 5) Calculate average Quality Rating
      *
      *    Compute K = Number of Quality Metrics
      *    Compute QUAL-SUM as sum of selected Quality Metrics
      *
           SET K TO 4.
           COMPUTE QUAL-SUM = MAINTAINABILITY
                            + REUSABILITY
                            + COVERAGE
                            + CONFORMITY.
      *
           COMPUTE TEST-QUALITY  =  QUAL-SUM / K.
      *
           EXIT.
      *
      ******************* End of Source ********************************
