       IDENTIFICATION DIVISION.
       PROGRAM-ID. TMETREPO.
      ******************************************************************
      *  PROGRAM NAME: TMETREPO (SOFT Auditor; METRics reporter)       *
      *  PURPOSE:      subprogram for producing the metric report      *
      *                within the tool TCSANAL                         *
      *                                                                *
      *  VERSION:      1.1 (02.06)                                     *
      *                                                                *
      *  LANGUAGE:     MICROSOFT (R) COBOL OPTIMIZIG COMPILER          *
      *                VERSION 3.00A ("MICROFOCUS COBOL")              *
      *                                                                *
      *  PROGRAM FUNCTIONS:                                            *
      *    - print header                                              *
      *    - print system metrics                                      *
      *    - print test case metrics                                   *
      *    - print complexity metrics                                  *
      *    - print quality metrics                                     *
      ******************************************************************
      *AUTHOR. SNEED.
      ******************************************************************
      *
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SOURCE-COMPUTER. PC.
       OBJECT-COMPUTER. PC.
      *
       INPUT-OUTPUT SECTION.
      ******************************************************************
       FILE-CONTROL.
      ******************************************************************
      *
      * MET audit  output file
      *
           SELECT  MET-FILE
                   ASSIGN TO DYNAMIC MET-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS MET-FSTS.
      *
       I-O-CONTROL.
      ******************************************************************
      *
       DATA DIVISION.
      *
      ******************************************************************
      *
       FILE SECTION.
      *
      ******************************************************************
      *
       FD  MET-FILE
           RECORD CONTAINS 72 CHARACTERS
           DATA RECORD IS MET-REC.
      *
       01  MET-REC              PIC X(72).
      *
      ******************************************************************
      *
       WORKING-STORAGE SECTION.
      *
      ******************************************************************
      *
       77  MET-FILE-NAME                PIC X(80).
      *
       77  MET-FSTS                     PIC XX.
      *
       01  INDEXES.
           05  I                        PIC 9999.
           05  J                        PIC 9999.
           05  K                        PIC 9999.
      *
       01  LOCAL-CONSTANTS.
           05  LINE-MAX                 PIC 99  VALUE 72.
      *
       01  LOCAL-VARIABLES.
           05  LINE-COUNT               PIC 99      VALUE 99.
           05  PAGE-COUNT               PIC 99      VALUE ZERO.
           05  THIS-VALUE               PIC S9V999  VALUE ZERO.
           05  NR-TESTCASES             PIC 999999  VALUE ZERO.
           05  FUNCT-AVERAGE            PIC 9999    VALUE ZERO.
           05  PARAM-AVERAGE            PIC 9999    VALUE ZERO.
           05  LAST-FUNCTION            PIC X(80)   VALUE SPACES.
           05  TEST-POINTS              PIC 999999  VALUE ZERO.
           05  DATA-POINTS              PIC 999999  VALUE ZERO.
           05  FUNC-POINTS              PIC 999999  VALUE ZERO.
           05  WEIGHTED-DEFI-NR         PIC 999999  VALUE ZERO.
           05  COMP-SUM                 PIC 9V999   VALUE ZERO.
           05  QUAL-SUM                 PIC 9V999   VALUE ZERO.
           05  DATA-COMPLEXITY          PIC 9V999   VALUE ZERO.
           05  DENSITY                  PIC 9V999   VALUE ZERO.
           05  INTENSITY                PIC 9V999   VALUE ZERO.
           05  VOLUMNE                  PIC 9V999   VALUE ZERO.
           05  COMPLEXITY               PIC 9V999   VALUE ZERO.
           05  TEST-COMPLEXITY          PIC 9V999   VALUE ZERO.
           05  COVERAGE                 PIC 9V999   VALUE ZERO.
           05  MAINTAINABILITY          PIC 9V999   VALUE ZERO.
           05  REUSABILITY              PIC 9V999   VALUE ZERO.
           05  CONFORMITY               PIC 9V999   VALUE ZERO.
           05  TEST-QUALITY             PIC 9V999   VALUE ZERO.
      *
       01  YYMMDD.
           05  YY                       PIC 99.
           05  MM                       PIC 99.
           05  DD                       PIC 99.
      *
      ****************************************************************
      *
       01  MET-SEPARATOR-LINE.
           05  FILLER           PIC X(01)  VALUE '+'.
           05  FILLER           PIC X(70)  VALUE ALL '-'.
           05  FILLER           PIC X(01)  VALUE '+'.
      *
       01  MET-BLANK-LINE.
           05  FILLER           PIC X(01)  VALUE '|'.
           05  FILLER           PIC X(70)  VALUE SPACES.
           05  FILLER           PIC X(01)  VALUE '|'.
      *
       01  MET-HEADER-1-LINE.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  MET-HEADER-1     PIC X(64)  VALUE
ENG     ' T C S A N A L   T E S T C A S E   M E T R I C   R E P O R T '.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  MET-HEADER-2-LINE.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  MET-HEADER-2-1   PIC X(10)  VALUE
ENG           'TXT-TYPE: '.
           05  PROC-TYPE-OUT    PIC X(10).
           05  FILLER           PIC X(30)  VALUE SPACES.
           05  MET-HEADER-2-2   PIC X(6)   VALUE
ENG           'DATE: '.
           05  DATUM-OUT.
               10  DAY-OUT      PIC 99.
               10  FILLER       PIC X VALUE '.'.
               10  MONTH-OUT    PIC 99.
               10  FILLER       PIC X VALUE '.'.
               10  YEAR-OUT     PIC 99.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  MET-HEADER-3-LINE.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  MET-HEADER-3-1   PIC X(10)  VALUE
ENG           'TEST-TAB: '.
           05  PROC-NAME-OUT    PIC X(40)  VALUE SPACES.
           05  MET-HEADER-3-2   PIC X(6)   VALUE
ENG           'PAGE: '.
           05  PAGE-OUT         PIC ZZZZZZZ9.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  MET-HEADER-4-LINE.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  FILLER           PIC X(6)   VALUE SPACES.
           05  MET-HEADER-4     PIC X(52)  VALUE
ENG           'T E S T C A S E   Q U A N T I T Y    M E T R I C S'.
           05  FILLER           PIC X(6)   VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  QUAN-MET-1-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-1-TITLE        PIC X(51)  VALUE
ENG           'Number of System Modules  analyzed       =======> '.
      **>> Number of Test Procedures processed
           05  QUAN-MET-1              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-2-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-2-TITLE        PIC X(51)  VALUE
ENG           'Number of System Documents analyzed      =======> '.
      **>> Number of Spec Documents  processed
           05  QUAN-MET-2              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-3-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-3-TITLE        PIC X(51)  VALUE
ENG           'Number of Test Procedures analyzed       =======> '.
      **>> Number of Test Procedures processed
           05  QUAN-MET-3              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-4-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-4-TITLE        PIC X(51)  VALUE
ENG           'Number of Text Cases analyzed            =======> '.
      **>> Number of Test Cases processed
           05  QUAN-MET-4              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-5-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-5-TITLE        PIC X(51)  VALUE
ENG           'Number of Test Case Attributes specified =======> '.
      **>> Number of Case Attributes
           05  QUAN-MET-5              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-6-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-6-TITLE        PIC X(51)  VALUE
ENG           'Number of Requirements to be tested      =======> '.
      **>> Number of Requirements not processed
           05  QUAN-MET-6              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-7-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-7-TITLE        PIC X(51)  VALUE
ENG           'Number of Use Cases to be tested         =======> '.
      **>> Total Number of Use Cases to be tested
           05  QUAN-MET-7              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-8-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-8-TITLE        PIC X(51)  VALUE
ENG           'Number of Data Objects to be tested      =======> '.
      **>> Number of Data Objects referred to
           05  QUAN-MET-8              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-9-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-9-TITLE        PIC X(51)  VALUE
ENG           'Number of Requirements tested            =======> '.
      **>> Number of tested Requirements
           05  QUAN-MET-9              PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-10-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-10-TITLE       PIC X(51)  VALUE
ENG           'Number of Use Cases tested               =======> '.
      **>> Number of tested Use Cases
           05  QUAN-MET-10             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-11-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-11-TITLE       PIC X(51)  VALUE
ENG           'Number of Data Objects tested            =======> '.
      **>> Number of tested data objects
           05  QUAN-MET-11             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-12-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-12-TITLE       PIC X(51)  VALUE
ENG           'Number of Actions to be tested           =======> '.
      **>> Number of Actions
           05  QUAN-MET-12             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-13-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-13-TITLE       PIC X(51)  VALUE
ENG           'Number of States to be tested            =======> '.
      **>> Number of Data states
           05  QUAN-MET-13             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-14-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-14-TITLE       PIC X(51)  VALUE
ENG           'Number of Rules to be tested             =======> '.
      **>> Number of Data Constants
           05  QUAN-MET-14             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-15-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-15-TITLE       PIC X(51)  VALUE
ENG           'Number of other Features to be tested    =======> '.
      **>> Number of Other Test cases
           05  QUAN-MET-15             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-16-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-16-TITLE       PIC X(51)  VALUE
ENG           'Number of different Test Case types      =======> '.
      **>> Number of Case Types
           05  QUAN-MET-16             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-17-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-17-TITLE       PIC X(51)  VALUE
ENG           'Number of Test Case sources              =======> '.
      **>> Number of Sources
           05  QUAN-MET-17             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-18-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-18-TITLE       PIC X(51)  VALUE
ENG           'Number of Test Case Goals                =======> '.
      **>> Number of Goals
           05  QUAN-MET-18             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-19-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-19-TITLE       PIC X(51)  VALUE
ENG           'Number of automated Test Cases           =======> '.
      **>> Number of automated cases
           05  QUAN-MET-19             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-20-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-20-TITLE       PIC X(51)  VALUE
ENG           'Number of Triggers specified             =======> '.
      **>> Number of Test Case Triggers
           05  QUAN-MET-20             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-21-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-21-TITLE       PIC X(51)  VALUE
ENG           'Number of Preconditions specified        =======> '.
      **>> Number of Preconditions
           05  QUAN-MET-21             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-22-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-22-TITLE       PIC X(51)  VALUE
ENG           'Number of Postconditions specified       =======> '.
      **>> Number of PostConditions
           05  QUAN-MET-22             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-23-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-23-TITLE       PIC X(51)  VALUE
ENG           'Number of Test Case Inputs               =======> '.
      **>> Number of Input arguments
           05  QUAN-MET-23             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-24-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-24-TITLE       PIC X(51)  VALUE
ENG           'Number of Test Case Outputs              =======> '.
      **>> Number of Outputs
           05  QUAN-MET-24             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
      *****************************************************************
      *
       01  MET-HEADER-5-LINE.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  FILLER           PIC X(8)   VALUE SPACES.
           05  MET-HEADER-5     PIC X(48)  VALUE
ENG           ' T E S T C A S E     S I Z E     M E T R I C S  '.
           05  FILLER           PIC X(8)   VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  QUAN-MET-26-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-26-TITLE       PIC X(51)  VALUE
ENG           'Number of Test Points                    =======> '.
      **>> Number of Test Points
           05  QUAN-MET-26             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-27-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-27-TITLE       PIC X(51)  VALUE
ENG           'Number of Function Points                =======> '.
      **>> Number of Function-Points
           05  QUAN-MET-27             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-28-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-28-TITLE       PIC X(51)  VALUE
ENG           'Number of Data Points                    =======> '.
      **>> Number of Data Points
           05  QUAN-MET-28             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
      *****************************************************************
      *
       01  MET-HEADER-6-LINE.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  FILLER           PIC X(8)   VALUE SPACES.
           05  MET-HEADER-6     PIC X(54)  VALUE
ENG           'T E S T C A S E   C O M P L E X I T Y   M E T R I C S '.
           05  FILLER           PIC X(2)   VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  COMP-MET-31-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-31-TITLE       PIC X(53)  VALUE
ENG           'Test Case Complexity Ratio               =======>   '.
           05  COMP-MET-31             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  COMP-MET-32-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-32-TITLE       PIC X(53)  VALUE
ENG           'Test Case Density Ratio                  =======>   '.
           05  COMP-MET-32             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  COMP-MET-33-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-33-TITLE       PIC X(53)  VALUE
ENG           'Test Case Intensity Ratio                =======>   '.
           05  COMP-MET-33             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  COMP-MET-34-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-34-TITLE       PIC X(53)  VALUE
ENG           'Test Case Volumne Ratio                  =======>   '.
           05  COMP-MET-34             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  COMP-MET-35-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-35-TITLE       PIC X(53)  VALUE
ENG           'Overall Test Case Complexity Rating      =======>   '.
           05  COMP-MET-35             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
      *****************************************************************
      *
       01  MET-HEADER-7-LINE.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  FILLER           PIC X(7)   VALUE SPACES.
           05  MET-HEADER-7     PIC X(50)  VALUE
ENG           '   T E S T C A S E   Q U A L I T Y   M E T R I C S'.
           05  FILLER           PIC X(7)   VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  QUAL-MET-41-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-41-TITLE       PIC X(53)  VALUE
ENG           'Test Case Maintainability Ratio          =======>   '.
           05  QUAL-MET-41             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAL-MET-42-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-42-TITLE       PIC X(53)  VALUE
ENG           'Test Case Coverage Ratio                 =======>   '.
           05  QUAL-MET-42             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAL-MET-43-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-43-TITLE       PIC X(53)  VALUE
ENG           'Test Case Completeness Ratio             =======>   '.
           05  QUAL-MET-43             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAL-MET-44-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-44-TITLE       PIC X(53)  VALUE
ENG           'Test Case Reusability Ratio              =======>   '.
           05  QUAL-MET-44             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAL-MET-45-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAL-MET-45-TITLE       PIC X(53)  VALUE
ENG           'Overall Test Case Quality Rating         =======>   '.
           05  QUAL-MET-45             PIC 9.999.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
      *****************************************************************
      *
       01  MET-HEADER-8-LINE.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  FILLER           PIC X(8)   VALUE SPACES.
           05  MET-HEADER-8     PIC X(54)  VALUE
ENG           'T E S T C A S E   D E F I C I E N C Y   M E T R I C S '.
           05  FILLER           PIC X(2)   VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  QUAN-MET-51-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-51-TITLE       PIC X(51)  VALUE
ENG           'Number of Major Rule Violations          =======> '.
      **>> (Number of Major Deficiencies
           05  QUAN-MET-51             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-52-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-52-TITLE       PIC X(51)  VALUE
ENG           'Number of Medium Rule Violations         =======> '.
      **>> (Number of Medium Deficiencies
           05  QUAN-MET-52             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-53-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-53-TITLE       PIC X(51)  VALUE
ENG           'Number of Minor Rule Violations          =======> '.
      **>> (Number of Minor Deficiencies
           05  QUAN-MET-53             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-54-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-54-TITLE       PIC X(51)  VALUE
ENG           'Number of Missing Attributes             =======> '.
      **>> (Number of Minor Deficiencies
           05  QUAN-MET-54             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
       01  QUAN-MET-55-LINE.
           05  FILLER                  PIC X(08)  VALUE '|       '.
           05  QUAN-MET-55-TITLE       PIC X(51)  VALUE
ENG           'Number of Missing References             =======> '.
      **>> (Number of Missing references
           05  QUAN-MET-55             PIC ZZZZZZ9.
           05  FILLER                  PIC X(06)  VALUE '     |'.
      *
      ****************************************************************
      *
       LINKAGE SECTION.
      *
      ******************************************************************
      * "Dynamic linkage file names" (and others)
      ******************************************************************
      *
       77  CONCEPT-NAME                    PIC X(40).
      *
       77  MODULE-NAME                     PIC X(8).
      *
       77  FILE-NAME                       PIC X(8).
      *
       77  FILE-TYPE                       PIC XXX.
      *
       77  OUTPUT-LIB                      PIC X(80).
      *
       COPY TMETPARS.
      *
       COPY TMETTAB.
      *
      ******************************************************************
      *
       PROCEDURE DIVISION USING CONCEPT-NAME,
                                MODULE-NAME,
                                FILE-NAME,
                                FILE-TYPE,
                                OUTPUT-LIB,
                                METR-TAB,
                                METRIC-TAB.
      *
      ******************************************************************
      *
       TXTMETR-MAIN SECTION.
      *
       SMETREPO-MAIN.
      *
           DISPLAY '+-------------------------------------------------+'
           DISPLAY '|SMETREPO Metric Report Version 3.1 of Feb. 2005  |'
           DISPLAY '+-------------------------------------------------+'
           PERFORM  OPEN-METRIC-REPORT.
           PERFORM WRITE-METRIC-REPORT.
           PERFORM CLOSE-METRIC-REPORT.
      *
           EXIT PROGRAM.
      *
      ******************************************************************
      *  Internal procedures used in producing the metric report
      ******************************************************************
      *
       OPEN-METRIC-REPORT.
      *
      ******************************************************************
      *  Open the Metric Report
      ******************************************************************
      *
           MOVE SPACES TO MET-FILE-NAME
           STRING OUTPUT-LIB     DELIMITED BY SPACE
                 '\'             DELIMITED BY SIZE
                  MODULE-NAME    DELIMITED BY SPACE
                 '.met'          DELIMITED BY SIZE
              INTO MET-FILE-NAME
           MOVE ZEROES TO MET-FSTS
           OPEN OUTPUT MET-FILE.
           DISPLAY 'Metric Report in File:' MET-FILE-NAME
           MOVE ZEROS TO LOCAL-VARIABLES.
      *
      ************************
      *
       WRITE-METRIC-REPORT.
      *
      ******************************************************************
      *  Generate Metric Report on 1 page with 72 lines
      ******************************************************************
      *
      *                            1) Set up Header Info
      *
           ACCEPT YYMMDD FROM DATE
           MOVE YY TO YEAR-OUT
           MOVE MM TO MONTH-OUT
           MOVE DD TO DAY-OUT
           ADD 1 TO PAGE-COUNT
           MOVE PAGE-COUNT TO PAGE-OUT
           MOVE CONCEPT-NAME TO PROC-NAME-OUT
           MOVE 'TestCase' TO PROC-TYPE-OUT
           EVALUATE FILE-TYPE
              WHEN 'tcs' MOVE 'MODULE: ' TO MET-HEADER-3-1
              WHEN 'sys' MOVE 'SYSTEM: ' TO MET-HEADER-3-1
              WHEN 'pro' MOVE 'PRODUCT:' TO MET-HEADER-3-1
              WHEN OTHER MOVE 'MODULE: ' TO MET-HEADER-3-1
           END-EVALUATE.
      *
      *                            2) Set up Metrics
      *
           PERFORM SET-QUANTITY-METRICS
      *
      *                            3) Write out Report
      *
      *      Write First Page of Metric Report
      *
      ***************************************Print Report header
           MOVE 1 TO PAGE-OUT.
           WRITE MET-REC FROM MET-SEPARATOR-LINE
           WRITE MET-REC FROM MET-HEADER-1-LINE
      *    WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM MET-HEADER-2-LINE
           WRITE MET-REC FROM MET-HEADER-3-LINE
           WRITE MET-REC FROM MET-SEPARATOR-LINE
      ***************************************Print Code metrics
           WRITE MET-REC FROM MET-HEADER-4-LINE
           WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM QUAN-MET-1-LINE
           WRITE MET-REC FROM QUAN-MET-2-LINE
           WRITE MET-REC FROM QUAN-MET-3-LINE
           WRITE MET-REC FROM QUAN-MET-4-LINE
           WRITE MET-REC FROM QUAN-MET-5-LINE
           WRITE MET-REC FROM QUAN-MET-6-LINE
           WRITE MET-REC FROM QUAN-MET-7-LINE
           WRITE MET-REC FROM QUAN-MET-8-LINE
           WRITE MET-REC FROM QUAN-MET-9-LINE
           WRITE MET-REC FROM QUAN-MET-10-LINE
           WRITE MET-REC FROM QUAN-MET-11-LINE
           WRITE MET-REC FROM QUAN-MET-12-LINE
           WRITE MET-REC FROM QUAN-MET-13-LINE
           WRITE MET-REC FROM QUAN-MET-14-LINE
           WRITE MET-REC FROM QUAN-MET-15-LINE
           WRITE MET-REC FROM QUAN-MET-16-LINE
           WRITE MET-REC FROM QUAN-MET-17-LINE
           WRITE MET-REC FROM QUAN-MET-18-LINE
           WRITE MET-REC FROM QUAN-MET-19-LINE
           WRITE MET-REC FROM QUAN-MET-20-LINE
           WRITE MET-REC FROM QUAN-MET-21-LINE
           WRITE MET-REC FROM QUAN-MET-22-LINE
           WRITE MET-REC FROM QUAN-MET-23-LINE
           WRITE MET-REC FROM QUAN-MET-24-LINE
      ***************************************Print Size Metrics
           WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM MET-HEADER-5-LINE
           WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM QUAN-MET-26-LINE
           WRITE MET-REC FROM QUAN-MET-27-LINE
           WRITE MET-REC FROM QUAN-MET-28-LINE
      ***************************************Print Report header
           WRITE MET-REC FROM MET-SEPARATOR-LINE
           MOVE '/ ' TO MET-REC.
           WRITE MET-REC.
           MOVE 2 TO PAGE-OUT.
           WRITE MET-REC FROM MET-SEPARATOR-LINE
           WRITE MET-REC FROM MET-HEADER-1-LINE
      *    WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM MET-HEADER-2-LINE
           WRITE MET-REC FROM MET-HEADER-3-LINE
           WRITE MET-REC FROM MET-SEPARATOR-LINE
      ***************************************Print Complexity metrics
           WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM MET-HEADER-6-LINE
           WRITE MET-REC FROM MET-BLANK-LINE
           IF COMP-METRIC (1) > ZERO
              WRITE MET-REC FROM COMP-MET-31-LINE
           END-IF.
           IF COMP-METRIC (2) > ZERO
              WRITE MET-REC FROM COMP-MET-32-LINE
           END-IF.
           IF COMP-METRIC (3) > ZERO
              WRITE MET-REC FROM COMP-MET-33-LINE
           END-IF.
           IF COMP-METRIC (4) > ZERO
              WRITE MET-REC FROM COMP-MET-34-LINE
           END-IF.
           WRITE MET-REC FROM COMP-MET-35-LINE
      ***************************************Print Quality Metrics
           WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM MET-HEADER-7-LINE
           WRITE MET-REC FROM MET-BLANK-LINE
           IF QUAL-METRIC (1) > ZERO
              WRITE MET-REC FROM QUAL-MET-41-LINE
           END-IF.
           IF QUAL-METRIC (2) > ZERO
              WRITE MET-REC FROM QUAL-MET-42-LINE
           END-IF.
           IF QUAL-METRIC (3) > ZERO
              WRITE MET-REC FROM QUAL-MET-43-LINE
           END-IF.
           IF QUAL-METRIC (4) > ZERO
              WRITE MET-REC FROM QUAL-MET-44-LINE
           END-IF.
           WRITE MET-REC FROM QUAL-MET-45-LINE
           WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM MET-SEPARATOR-LINE.
      ***************************************Print Deficiency Metrics
           WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM MET-HEADER-8-LINE
           WRITE MET-REC FROM MET-BLANK-LINE
           WRITE MET-REC FROM QUAN-MET-51-LINE
           WRITE MET-REC FROM QUAN-MET-52-LINE
           WRITE MET-REC FROM QUAN-MET-53-LINE
           WRITE MET-REC FROM QUAN-MET-54-LINE
           WRITE MET-REC FROM QUAN-MET-55-LINE
           WRITE MET-REC FROM MET-SEPARATOR-LINE.
      *
      ************************
      *
       CLOSE-METRIC-REPORT.
      *
      ************************************************************
      *  Close the Metric Report
      ************************************************************
      *
           CLOSE MET-FILE.
           DISPLAY '+--------------------------------------+'
           DISPLAY '| Text Metric Report was generated for  '
           DISPLAY '| Concept: ' CONCEPT-NAME
           DISPLAY '+--------------------------------------+'
           GOBACK.
      *
      ************************
      *
       TCS-METR-SUBROUTINES SECTION.
      *
      ******************************************************************
      *  Second level internal procedures
      *  used in producing the metric report
      ******************************************************************
      *
       SET-QUANTITY-METRICS.
      *
      *    Set Test Case Quantity Metrics
      *
           MOVE NR-MODULES        TO QUAN-MET-1.
           MOVE NR-CONCEPTS       TO QUAN-MET-2.
           MOVE NR-TEST-PROCS     TO QUAN-MET-3.
           MOVE NR-TEST-CASES     TO QUAN-MET-4.
           MOVE NR-TEST-ATTRS     TO QUAN-MET-5.
           MOVE NR-TARGET-REQUS   TO QUAN-MET-6.
           MOVE NR-TARGET-FUNCS   TO QUAN-MET-7.
           MOVE NR-TARGET-OBJECTS TO QUAN-MET-8.
           MOVE NR-TESTED-REQUS   TO QUAN-MET-9.
           MOVE NR-TESTED-FUNCS   TO QUAN-MET-10.
           MOVE NR-TESTED-OBJECTS TO QUAN-MET-11.
           MOVE NR-ACTIONS        TO QUAN-MET-12.
           MOVE NR-STATES         TO QUAN-MET-13.
           MOVE NR-RULES          TO QUAN-MET-14.
           MOVE NR-OTHERS         TO QUAN-MET-15.
           MOVE NR-TEST-CASE-TYPES TO QUAN-MET-16.
           MOVE NR-TEST-CASE-SRCS  TO QUAN-MET-17.
           MOVE NR-TEST-CASE-GOALS TO QUAN-MET-18.
           MOVE NR-TEST-CASES-AUTO TO QUAN-MET-19.
           MOVE NR-TEST-TRIGGERS   TO QUAN-MET-20.
           MOVE NR-PRE-CONDS       TO QUAN-MET-21.
           MOVE NR-POST-CONDS      TO QUAN-MET-22.
           MOVE NR-INPUTS          TO QUAN-MET-23.
           MOVE NR-OUTPUTS         TO QUAN-MET-24.
      *
      *    Set Text Size Metrics
      *
           PERFORM COMPUTE-SIZE-METRICS
           MOVE TEST-POINTS       TO QUAN-MET-26.
           MOVE FUNC-POINTS       TO QUAN-MET-27.
           MOVE DATA-POINTS       TO QUAN-MET-28.
      *
      *    Set Text Complexity Metrics
      *
           PERFORM COMPUTE-COMPLEXITY-METRICS
           MOVE DATA-COMPLEXITY   TO COMP-MET-31.
           MOVE DENSITY           TO COMP-MET-32.
           MOVE INTENSITY         TO COMP-MET-33.
           MOVE VOLUMNE           TO COMP-MET-34.
           MOVE TEST-COMPLEXITY   TO COMP-MET-35.
      *
      *    Set Text Quality Metrics
      *
           PERFORM COMPUTE-QUALITY-METRICS
           MOVE MAINTAINABILITY   TO QUAL-MET-41.
           MOVE COVERAGE          TO QUAL-MET-42.
           MOVE CONFORMITY        TO QUAL-MET-43.
           MOVE REUSABILITY       TO QUAL-MET-44.
           MOVE TEST-QUALITY      TO QUAL-MET-45.
      *
      ***************************************************************
      *    Set Conformance Metrics
      *
           MOVE NR-MAJOR-DEFIS    TO QUAN-MET-51.
           MOVE NR-MEDIA-DEFIS    TO QUAN-MET-52.
           MOVE NR-MINOR-DEFIS    TO QUAN-MET-53.
           MOVE NR-MISSING-ATTRS  TO QUAN-MET-54.
           MOVE NR-MISSING-REFS   TO QUAN-MET-55.
      *
           EXIT.
      *
      ***************************************************************
      *
           COPY TMETCALC.
      *
      *************** End of Source *********************************
