      ******************************************************************
      * Program for converting files                                   *
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. CONVERT.
      *AUTHOR.     H. SNEED.
      *DATE.       MARCH 2004
      *VERSION.    1.0 of March 2004
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *               Conversion Specification                         *
      * Accept Input & Output File names                               *
      * Accept Control Parameters                                      *
      * Open LogFile                                                   *
      * Open OutputFile                                                *
      * Select on Input File Type                                      *
      *    When 'Txt'                                                  *
      *      Open TxtFile                                              *
      *      Do for each Record                                        *
      *        Perform TxtParse                                        *
      *        Select on Output File Type                              *
      *           When 'Csv'                                           *
      *              Perform CsvWrite                                  *
      *           When 'Xml'                                           *
      *              If First Record                                   *
      *                 Copy Schema to Output File                     *
      *              Perform XmlWrite                                  *
      *      Enddo                                                     *
      *      Close TxtFile                                             *
      *    When 'Csv'                                                  *
      *      Open CsvFile                                              *
      *      Do for each Record                                        *
      *        Perform CsvParse                                        *
      *        Select on Output File Type                              *
      *           When 'Txt'                                           *
      *              Perform TxtWrite                                  *
      *           When 'Xml'                                           *
      *              If First Record                                   *
      *                 Copy Schema to Output File                     *
      *              Perform XmlWrite                                  *
      *      Enddo                                                     *
      *    When 'Xml'                                                  *
      *      Open XmlFile                                              *
      *      Do for each Record                                        *
      *        Perform XmlParse                                        *
      *        Select on Output File Type                              *
      *           When 'Txt'                                           *
      *              Perform TxtWrite                                  *
      *           When 'Csv'                                           *
      *              Perform CsvWrite                                  *
      *      Enddo                                                     *
      *      Close XmlFile                                             *
      * Close OutputFile                                               *
      * Close LogFile                                                  *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *
           SELECT INPUT-FILE
                  ASSIGN TO DYNAMIC INPUT-NAME
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS INPUT-STATUS.
      *
           SELECT OUTPUT-FILE
                  ASSIGN TO DYNAMIC OUTPUT-NAME
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS OUTPUT-STATUS.
      *
      ******************************************************************
      /
       DATA DIVISION.
      *
       FILE SECTION.
      *
       FD  INPUT-FILE
           RECORD CONTAINS 960 CHARACTERS
           DATA RECORD IS INPUT-REC.
       01  INPUT-REC.
           05  INPUT-LINE                    PIC X(1600).
           05  INPUT-CHARS                   REDEFINES INPUT-LINE.
               10  INPUT-CHAR                PIC X  OCCURS 1600 TIMES.
      *
       FD  OUTPUT-FILE
           RECORD CONTAINS 244 CHARACTERS
           DATA RECORD IS OUTPUT-REC.
       01  OUTPUT-REC.
           05  OUTPUT-RECNR                  PIC 9999.
           05  OUTPUT-LABEL                  PIC X(40).
           05  OUTPUT-VALUE                  PIC X(200).
      *
      ******************************************************************
      /
      *
       WORKING-STORAGE SECTION.
      *
      *    File Parameters
      *
       77  INPUT-NAME                        PIC X(80).
       77  OUTPUT-NAME                       PIC X(80).
       77  XSD-NAME                          PIC X(80).
       77  INPUT-STATUS                      PIC XX.
           88 INPUT-OK                       VALUE '00'.
           88 INPUT-EOF                      VALUE '09'.
       77  XSD-STATUS                        PIC XX.
           88 XSD-OK                         VALUE '00'.
           88 XSD-EOF                        VALUE '09'.
       77  OUTPUT-STATUS                     PIC XX.
           88 OUTPUT-OK                      VALUE '00'.
           88 OUTPUT-EOF                     VALUE '09'.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
      *
       01  INDEXES.
      *        indexes  for processing tables
           05  I                             PIC 999.
           05  J                             PIC 999.
           05  K                             PIC 999.
           05  L                             PIC 999.
           05  M                             PIC 999.
           05  COL-NR                        PIC 999  VALUE ZERO.
           05  OBJ-PTR                       PIC 999  VALUE ZERO.
      *
       01  SWITCHES.
      *        Switches for Recording States
           05  FOUND-SW                      PIC 9    VALUE 0.
      *
       01  CONSTANTS.
           05  MAX-FIELDS                    PIC 99    VALUE 40.
           05  MAX-COL-NR                    PIC 99    VALUE 40.
           05  FIELD-LNG                     PIC 99    VALUE 80.
           05  LINE-LNG                      PIC 9999  VALUE 800.
      *
       01  COUNTERS.
           05  FILE-COUNT                    PIC 9999  VALUE ZERO.
           05  LINE-COUNT                    PIC 9999  VALUE ZERO.
           05  REC-NR                        PIC 9999  VALUE ZERO.
      *
       01  VARIABLES.
      *                          Intermediate Work Fields
           05  OBJECT-NAME                   PIC X(40).
           05  LAST-OBJECT                   PIC X(40).
           05  OBJ-STRING                    PIC X(80).
           05  OBJ-CHARS                     REDEFINES OBJ-STRING.
               10  OBJ-CHAR                  PIC X OCCURS 80 TIMES.
           05  NAME-STRING                   PIC X(40).
           05  NAME-CHARS                    REDEFINES NAME-STRING.
               10  NAME-CHAR                 PIC X OCCURS 40 TIMES.
           05  THIS-COL-NR                   PIC 99    VALUE 40.
      *
       01  TABLES.
      *            Tables of tags and fields  indexed by COL-NR
           05  TAGS.
               10  TAG                       PIC X(40) OCCURS 40 TIMES.
           05  FIELDS.
               10  FIELD                     PIC X(80) OCCURS 40 TIMES.
           05  POSITIONS.
               10  POS                       PIC 999   OCCURS 40 TIMES.
      *
      ****************************************************************
      *                          Processing Data Copies
           COPY XMLDATA.cpy.
      *
           COPY CSVDATA.cpy.
      *
           COPY TXTDATA.cpy.
      *
      ****************************************************************
      *  Parameters from TestComp
       LINKAGE SECTION.
      *
           COPY CONVPARS.cpy.
      *
       77  ERROR-CODE                        PIC 99.
      *
      ***************************************************************
      /
       PROCEDURE DIVISION USING CONVERSION-PARAMS,
                                ERROR-CODE.
      ******************************************************************
      *
       INITIALIZATION SECTION.
      *
      ******************************************************************
      *
       ACCEPT-PARAMETERS.
      *
           MOVE ZEROES TO ERROR-CODE.
           MOVE ZEROES TO LINE-COUNT.
           MOVE OLD-FILE TO INPUT-NAME
           MOVE NEW-FILE TO OUTPUT-NAME
           DISPLAY '+---------------------------------------+'
           DISPLAY '| File Converter Vers 1.0 of March 2004 |'
           DISPLAY '+---------------------------------------+'
           DISPLAY '| File Name        = ' FILES-NAME
           DISPLAY '| Input File Type  = ' OLD-TYPE
           DISPLAY '| Output File Type = ' NEW-TYPE
           DISPLAY '| Input  File      = ' INPUT-NAME
           DISPLAY '| Output File      = ' OUTPUT-NAME
           DISPLAY '+---------------------------------------+'
TEST***    DISPLAY '| Press Enter to start Conversion !!'
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      ******************************************************************
      *
       MAIN-PROCESSING  SECTION.
      *
      ******************************************************************
      *
       SELECT-CONVERSION-TYPE.
      **   Branch to appropriate Conversion
           OPEN OUTPUT OUTPUT-FILE
           MOVE ZEROES TO REC-NR.
           EVALUATE OLD-TYPE
           WHEN 'TXT'
           WHEN 'txt'
               PERFORM PROCESS-TXT-FILE
           WHEN 'CSV'
           WHEN 'csv'
               PERFORM PROCESS-CSV-FILE
           WHEN 'XML'
           WHEN 'xml'
               PERFORM PROCESS-XML-FILE
           WHEN OTHER
               DISPLAY '|=> Input File Type not valid '
               DISPLAY '|=> File Name: ' OLD-TYPE
               DISPLAY '|=> Press Enter to Continue!'
               ACCEPT CONTINUE-KEY
               MOVE 72 TO ERROR-CODE
               CONTINUE
           END-EVALUATE.
      *************************************************************
      *   Close Conversion                                       *
      *************************************************************
       CLOSE-FILES.
           CLOSE INPUT-FILE.
           CLOSE OUTPUT-FILE.
           GOBACK.
      *
      ******************************************************************
      *
       PROCESS-TXT-FILE SECTION.
      *
      ******************************************************************
       OPEN-TXT-FILE.
      *****************************************************************
      *
           MOVE ZEROES TO INPUT-STATUS
           OPEN INPUT INPUT-FILE
           IF INPUT-STATUS NOT = ZEROES
              DISPLAY '|=> Input File not found '
              DISPLAY '|=> File Name: ' INPUT-NAME
              DISPLAY '|=> Press Enter to Continue!'
              ACCEPT CONTINUE-KEY
ERRO71        MOVE 71 TO ERROR-CODE
              GOBACK
           END-IF.
      *
      ******************************************************************
       READ-TXT-HEADER.
      *****************************************************************
      *                                 Read records of old xml file
TEST***    DISPLAY 'Read TXT Header of ' CONVERT-NAME
           MOVE ZEROES TO INPUT-STATUS.
           MOVE ZEROES TO LINE-COUNT.
HARRY      MOVE MAX-COL-NR TO THIS-COL-NR.
           MOVE SPACES TO TAGS.
           READ INPUT-FILE
           AT END
              MOVE '09' TO INPUT-STATUS
           NOT AT END
              MOVE INPUT-REC TO TXT-LINE
TEST***       DISPLAY 'TXT-LINE:' TXT-LINE
              MOVE 1 TO TXT-PTR
              MOVE 1 TO COL-NR
              PERFORM UNTIL TXT-PTR > TXT-LINE-LNG
                         OR COL-NR > MAX-COL-NR
HARRY                    OR COL-NR > THIS-COL-NR
                 PERFORM PARSE-TXT
TEST***          DISPLAY 'TXT-VAL:' TXT-VALUE
TEST***          ACCEPT CONTINUE-KEY
COPIED           MOVE TXT-VALUE TO TAG (COL-NR)
                 IF  TXT-PTR < TXT-LINE-LNG
                 AND COL-NR < MAX-COL-NR
                    ADD 1 TO COL-NR
                 END-IF
              END-PERFORM
           END-READ.
      *
      ******************************************************************
       READ-TXT-FILE.
      *****************************************************************
      *                                 Read records of old txt file
TEST***    DISPLAY 'Read Txt Records of ' CONVERT-NAME
           MOVE ZEROES TO INPUT-STATUS.
           MOVE ZEROES TO LINE-COUNT.
HARRY      MOVE MAX-COL-NR TO THIS-COL-NR.
           MOVE ZEROES TO POSITIONS
           PERFORM UNTIL INPUT-EOF
              READ INPUT-FILE
              AT END
                 MOVE '09' TO INPUT-STATUS
              NOT AT END
                 MOVE SPACES TO FIELDS
                 MOVE INPUT-REC TO TXT-LINE
TEST***          DISPLAY 'TXT-LINE:' TXT-LINE
                 MOVE 1 TO TXT-PTR
                 MOVE 1 TO COL-NR
                 PERFORM UNTIL TXT-PTR > TXT-LINE-LNG
                            OR COL-NR > MAX-COL-NR
HARRY                       OR COL-NR > THIS-COL-NR
                    PERFORM PARSE-TXT
COPIED              MOVE TXT-VALUE TO FIELD (COL-NR)
TEST***             DISPLAY 'Text Tag:  ' TAG   (COL-NR)
TEST***             DISPLAY 'Text Field:' FIELD (COL-NR)
TEST***             ACCEPT CONTINUE-KEY
                    IF  TXT-PTR < TXT-LINE-LNG
                    AND COL-NR < MAX-COL-NR
                       ADD 1 TO COL-NR
                    END-IF
                 END-PERFORM
                 PERFORM WRITE-TXT-RECORD
              END-READ
           END-PERFORM.
      *************************************************************
      *   Close Text Conversion                                       *
      *************************************************************
       CLOSE-TXT-FILE.
           DISPLAY '| TXT File Converted:' FILES-NAME
           DISPLAY '| Number of Records converted = ' REC-NR
TEST       DISPLAY '| Press Enter to continue Conversion ! '
TEST***    ACCEPT CONTINUE-KEY
           EXIT.
      *
      ******************************************************************
      *
       PROCESS-CSV-FILE SECTION.
      *
      ******************************************************************
       OPEN-CSV-FILE.
      *****************************************************************
      *
           MOVE ZEROES TO INPUT-STATUS
           OPEN INPUT INPUT-FILE
           IF INPUT-STATUS NOT = ZEROES
              DISPLAY '|=> Input File not found '
              DISPLAY '|=> File Name: ' INPUT-NAME
              DISPLAY '|=> Press Enter to Continue!'
              ACCEPT CONTINUE-KEY
ERRO71        MOVE 71 TO ERROR-CODE
              GOBACK
           END-IF.
      *
      ******************************************************************
       READ-CSV-HEADER.
      *****************************************************************
      *                                 Read records of old xml file
TEST***    DISPLAY 'Read CSV Header of ' CONVERT-NAME
           MOVE ZEROES TO INPUT-STATUS.
           MOVE ZEROES TO LINE-COUNT.
HARRY      MOVE MAX-COL-NR TO THIS-COL-NR.
           MOVE SPACES TO TAGS.
           READ INPUT-FILE
           AT END
              MOVE '09' TO INPUT-STATUS
           NOT AT END
              MOVE INPUT-REC TO CSV-LINE
TEST***       DISPLAY 'CSV-LINE:' CSV-LINE
              MOVE 1 TO CSV-PTR
              MOVE 1 TO COL-NR
              PERFORM UNTIL CSV-PTR > CSV-LINE-LNG
                         OR COL-NR > MAX-COL-NR
HARRY                    OR COL-NR > THIS-COL-NR
                 PERFORM PARSE-CSV
TEST***          DISPLAY 'CSV-VAL:' CSV-VALUE
TEST***          ACCEPT CONTINUE-KEY
COPIED           MOVE CSV-VALUE TO TAG (COL-NR)
                 IF  CSV-PTR < CSV-LINE-LNG
                 AND COL-NR < MAX-COL-NR
                    ADD 1 TO COL-NR
                 END-IF
              END-PERFORM
           END-READ.
      *
      ******************************************************************
       READ-CSV-FILE.
      *****************************************************************
      *                                 Read records of old xml file
TEST***    DISPLAY 'Read CSV Records of ' CONVERT-NAME
           MOVE ZEROES TO INPUT-STATUS.
           MOVE ZEROES TO LINE-COUNT.
           MOVE SPACES TO FIELDS.
HARRY      MOVE MAX-COL-NR TO THIS-COL-NR.
           PERFORM UNTIL INPUT-EOF OR ERROR-CODE > ZERO
              READ INPUT-FILE
              AT END
                 MOVE '09' TO INPUT-STATUS
              NOT AT END
                 MOVE SPACES TO FIELDS
                 MOVE INPUT-REC TO CSV-LINE
TEST***          DISPLAY 'CSV-LINE:' CSV-LINE
                 MOVE 1 TO CSV-PTR
                 MOVE 1 TO COL-NR
                 PERFORM UNTIL CSV-PTR > CSV-LINE-LNG
                            OR COL-NR > MAX-COL-NR
HARRY                       OR COL-NR > THIS-COL-NR
                    PERFORM PARSE-CSV
COPIED              MOVE CSV-VALUE TO FIELD (COL-NR)
TEST***             DISPLAY 'CSV-TAG:' TAG (COL-NR)
TEST***             DISPLAY 'CSV-VAL:' FIELD (COL-NR)
TEST***             ACCEPT CONTINUE-KEY
                    IF  CSV-PTR < CSV-LINE-LNG
                    AND COL-NR < MAX-COL-NR
                       ADD 1 TO COL-NR
                    END-IF
                 END-PERFORM
                 PERFORM WRITE-TXT-RECORD
              END-READ
           END-PERFORM.
      *************************************************************
      *   Close CSV  Conversion                                       *
      *************************************************************
       CLOSE-CSV-FILE.
           DISPLAY '| CSV File Converted:' FILES-NAME
           DISPLAY '| Number of Records converted = ' REC-NR
TEST***    DISPLAY '| Press Enter to continue Conversion ! '
TEST***    ACCEPT CONTINUE-KEY
           EXIT.
      *
      ******************************************************************
      *
       PROCESS-XML-FILE SECTION.
      *
      ******************************************************************
       OPEN-XML-FILE.
      *****************************************************************
      *
           MOVE ZEROES TO INPUT-STATUS
           OPEN INPUT INPUT-FILE
           IF INPUT-STATUS NOT = ZEROES
              DISPLAY '|=> Input File not found '
              DISPLAY '|=> File Name: ' INPUT-NAME
              DISPLAY '|=> Press Enter to Continue!'
              ACCEPT CONTINUE-KEY
ERRO72        MOVE 72 TO ERROR-CODE
              GOBACK
           END-IF.
      *
      ******************************************************************
       READ-XML-FILE.
      *****************************************************************
      *                                 Read records of old xml file
TEST***    DISPLAY 'Read XML Records of ' CONVERT-NAME
           MOVE ZEROES TO LINE-COUNT.
           MOVE ZEROES TO INPUT-STATUS.
           PERFORM UNTIL INPUT-EOF OR ERROR-CODE > ZERO
              MOVE ZERO TO COL-NR
              MOVE SPACES TO TAGS
              MOVE SPACES TO FIELDS
              MOVE SPACES TO XML-LABEL
              PERFORM UNTIL INPUT-EOF OR ERROR-CODE > ZERO
                        OR (XML-LABEL (1:1) = '/'
                        AND XML-LABEL (2:)  = FILES-NAME)
                 READ INPUT-FILE
                 AT END
                    MOVE '09' TO INPUT-STATUS
                 NOT AT END
                    MOVE 1 TO XML-PTR
                    MOVE INPUT-REC TO XML-LINE
                    PERFORM PARSE-XML
TEST***             DISPLAY 'Xml_Label=' XML-LABEL
TEST***             DISPLAY 'Xml_Attr =' XML-ATTRB
TEST***             DISPLAY 'Xml_Valu =' XML-VALUE
TEST***             ACCEPT CONTINUE-KEY
                    IF XML-LABEL (1:1) = '/'
                    OR XML-VALUE (1:1) = '_'
                       IF XML-LABEL (1:1) NOT = '/'
                          MOVE OBJECT-NAME TO LAST-OBJECT
                          MOVE XML-LABEL TO OBJECT-NAME
TEST***                   DISPLAY 'Last-Object:' LAST-OBJECT
TEST***                   DISPLAY 'This-Object:' OBJECT-NAME
TEST***                   ACCEPT CONTINUE-KEY
                       END-IF
                    ELSE
                       IF XML-LABEL NOT = SPACE
                          ADD 1 TO COL-NR
                          IF COL-NR < MAX-COL-NR
                             MOVE XML-LABEL TO TAG (COL-NR)
                             MOVE XML-VALUE TO FIELD (COL-NR)
TEST***                      DISPLAY 'This-Tag:' TAG (COL-NR)
TEST***                      DISPLAY 'This-Val:' FIELD (COL-NR)
TEST***                      ACCEPT CONTINUE-KEY
                          END-IF
                       END-IF
                    END-IF
                    IF XML-VALUE (1:1) = '_' AND
                       XML-ATTRB NOT = SPACES
                       ADD 1 TO COL-NR
                       IF COL-NR < MAX-COL-NR
                          MOVE XML-LABEL TO TAG (COL-NR)
                          MOVE XML-ATTRB TO FIELD (COL-NR)
TEST***                   DISPLAY 'This-Tag:' TAG (COL-NR)
TEST***                   DISPLAY 'This-Val:' FIELD (COL-NR)
TEST***                   ACCEPT CONTINUE-KEY
                       END-IF
                    END-IF
                 END-READ
              END-PERFORM
              IF INPUT-EOF OR ERROR-CODE > ZERO
                 CONTINUE
              ELSE
                 PERFORM WRITE-TXT-RECORD
              END-IF
           END-PERFORM.
      *************************************************************
      *   Close XML  Conversion                                       *
      *************************************************************
       CLOSE-XML-FILE.
           DISPLAY '| XML File Converted:' FILES-NAME
           DISPLAY '| Number of Records converted = ' REC-NR
TEST       DISPLAY '| Press Enter to continue Conversion ! '
TEST***    ACCEPT CONTINUE-KEY
           EXIT.
      *
      ******************************************************************
      *
       OUTPUT-ROUTINES SECTION.
      *
      **************************************************************
      *
       WRITE-TXT-RECORD.
           ADD 1 TO REC-NR
           MOVE 1 TO I.
           PERFORM UNTIL I > COL-NR
                      OR I > MAX-COL-NR
              MOVE REC-NR    TO OUTPUT-RECNR
              MOVE TAG   (I) TO OUTPUT-LABEL
NEW           IF TAG (I) = 'TestObjekte'
NEW           OR TAG (I) = 'Testobjekte'
NEW           OR TAG (I) = 'TestObjects'
NEW              MOVE FIELD (I) TO OBJ-STRING
NEW              INSPECT OBJ-STRING REPLACING ALL '_' BY SPACE
NEW              MOVE 1 TO OBJ-PTR
                 PERFORM UNTIL OBJ-PTR > FIELD-LNG
                    MOVE SPACES TO OUTPUT-VALUE
                    UNSTRING OBJ-STRING (OBJ-PTR:)  DELIMITED BY SPACE
                        INTO OUTPUT-VALUE
                    WRITE OUTPUT-REC
TEST***             DISPLAY 'Object = ' OUTPUT-VALUE
TEST***             ACCEPT CONTINUE-KEY
                    PERFORM UNTIL OBJ-CHAR (OBJ-PTR) = SPACE
                               OR OBJ-PTR > FIELD-LNG
                       ADD 1 TO OBJ-PTR
                    END-PERFORM
                    PERFORM UNTIL OBJ-CHAR (OBJ-PTR) NOT = SPACE
                               OR OBJ-PTR > FIELD-LNG
                       ADD 1 TO OBJ-PTR
                    END-PERFORM
                 END-PERFORM
              ELSE
                 MOVE FIELD (I) TO OUTPUT-VALUE
                 WRITE OUTPUT-REC
              END-IF
              ADD 1 TO I
           END-PERFORM.
      *                        Write out TXT Line
      **************************************************************
      **************************************************************
      *
       PARSING-ROUTINE SECTION.
      *
           COPY TXTPARSE.
      *
      *****************************************************************
      *
           COPY CSVPARSE.
      *
      *****************************************************************
      *
           COPY XMLPARSE.
      *
      ******************************************************************
      ********************** end of source *****************************
