      ******************************************************************
      *   PROGRAM  FOR  PROCESSING OBJECTS WITH TESTCASES              *
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. OBJSPEC.
      *AUTHOR.     H. SNEED.
      *DATE.       MARCH 2006.
      *VERS.       1.0 of Mar. 2006
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *                  OBJT DOCUMENTATION SPECIFICATION              *
      * Initialize Objt Report                                         *
      * Open Object File                                               *
      * Do for each Header                                             *
      *    Read Header                                                 *
      *    If Header = Object                                          *
      *       Call TCSIOMOD                                            *
      *       if Object has TestCase                                   *
      *          Print object to TestCase Link                         *
      *       else                                                     *
      *          Print object has no TestCase                          *
      *       end                                                      *
      *    end                                                         *
      * endDo                                                          *
      * Close Object File                                              *
      * Close DB-Testcases                                             *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
      ******************************************************************
       FILE-CONTROL.
      ******************************************************************
      *
      * Object Input File
      *
           SELECT  REPO-FILE
                   ASSIGN TO DYNAMIC REPO-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS REPO-STATUS.
      *
           SELECT  OBJT-REPORT
                   ASSIGN TO DYNAMIC OBJT-REPO-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS OBJT-STATUS.
      *
      * Object Table
      *
           SELECT  OBJT-TABLE
                   ASSIGN TO DYNAMIC OBJT-TAB-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS OBJT-STATUS.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
      *
       FD  REPO-FILE
           RECORD CONTAINS 112 CHARACTERS
           DATA RECORD IS REPO-REC.
       01  REPO-REC.
           05  REPO-BASE-TYPE               PIC X(4).
           05  FILLER                       PIC X.
           05  REPO-BASE-ENTITY             PIC X(36).
           05  FILLER                       PIC X.
           05  REPO-RELATION                PIC X(4).
           05  FILLER                       PIC X.
           05  REPO-TARGET-TYPE             PIC X(4).
           05  FILLER                       PIC X.
           05  REPO-TARGET-ENTITY           PIC X(60).
      *
       FD  OBJT-REPORT
           RECORD CONTAINS 96 CHARACTERS
           DATA RECORD IS OBJT-REPORT-REC.
       01  OBJT-REPORT-REC                  PIC X(96).
      *
       FD  OBJT-TABLE
           RECORD CONTAINS 160 CHARACTERS
           DATA RECORD IS OBJT-TABLE-REC.
      *
       01  OBJT-TABLE-REC                    PIC X(160).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
       77  REPO-FILE-NAME                    PIC X(80).
       77  OBJT-REPO-NAME                    PIC X(80).
       77  OBJT-STATUS                       PIC XX.
       77  REPO-STATUS                       PIC XX.
           88  REPO-OK                       VALUE '00'.
           88  REPO-EOF                      VALUE '09'.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 9(4) BINARY.
           05  J                             PIC 9(4) BINARY.
           05  K                             PIC 9(4) BINARY.
           05  OBJT-INDEX                    PIC 9(4) BINARY.
           05  CASE-INDEX                    PIC 9(4) BINARY.
KATI  *                                 sort indexes
KATI       05  S-I                           PIC 9(4) BINARY.
KATI       05  S-J                           PIC 9(4) BINARY.
KATI       05  MIN-I                         PIC 9(4) BINARY.
      *
      ****************************************************************
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  LINE-LNG                      PIC 99   VALUE 80.
           05  NAME-LNG                      PIC 99   VALUE 40.
           05  PAGE-MAX                      PIC 99   VALUE 60.
           05  MAX-TC-REFS                   PIC 99   VALUE 20.
KATI       05  OBJT-MAX                      PIC 9999 VALUE 500.
KATI       05  CASE-MAX                      PIC 9999 VALUE 499.
      *                          Array Lengths
      ****************************************************
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  FOUND-SW                      PIC 9    VALUE 0.
      *
      *
       01  COUNTS.
           05  LINE-COUNT               PIC 99     VALUE ZERO.
           05  PAGE-COUNT               PIC 9999   VALUE ZERO.
           05  NR-OBJECTS               PIC 9(7)   VALUE ZERO.
      *
       01  LOCAL-VARIABLES.
           05  LAST-FILE                PIC X(8).
           05  PERCENT                  PIC 999V99.
           05  CURRENT-OBJECT           PIC X(40).
           05  NAME-STRING              PIC X(40).
           05  NAME-CHARS               REDEFINES NAME-STRING.
               10  NAME-CHAR            PIC X OCCURS 40 TIMES.
      *
      **************************************************************
KATI  *                                 sort work field
KATI       05  WORK-OBJECT-NAME         PIC X(30).
KATI  *
KATI   01  TABLES.
KATI       05  OBJECT-TAB.
KATI           10  OBJECT-ENTRY         OCCURS 500 TIMES.
KATI               15  OBJECT-NAME      PIC X(30).
KATI  *
KATI       05  TESTCASE-TAB.
KATI           10  TESTCASE-ENTRY       OCCURS 500 TIMES.
KATI               15  TESTCASE-NAME    PIC X(12).
KATI               15  TESTCASE-BASE    PIC X(30).
      *
KATI***    COPY DBPARMS.
      *
      ****************************************************************
      *   Data for printing out Requirements
      *
       01  YYMMDD.
           05  YY                       PIC 99.
           05  MM                       PIC 99.
           05  DD                       PIC 99.

      *
       01  NEWPAGE-LINE.
           05  FILLER                   PIC X(96)  VALUE '/'.
      *
       01  SEPARATOR-LINE.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(94)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
      *
       01  SEPARATOR-LINE-1.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(34)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(34)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(24)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
      *
       01  BLANK-LINE.
           05  FILLER                   PIC X(01)  VALUE '|'.
           05  FILLER                   PIC X(94)  VALUE SPACES.
           05  FILLER                   PIC X(01)  VALUE '|'.
      *
       01  OBJ-REC-HEADER-1.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  FILLER                   PIC X(07)  VALUE SPACES.
           05  OBJ-HEADER-1             PIC X(50)  VALUE
ENG           ' T E S T A N A L  T E S T C A S E   A U D I T O R '.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  FILLER                   PIC X(07)  VALUE SPACES.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  OBJ-REC-HEADER-2.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  FILLER                   PIC X(14)  VALUE SPACES.
           05  OBJ-HEADER-2             PIC X(40)  VALUE
ENG           'D A T A   O B J E C T    R E P O R T   '.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  FILLER                   PIC X(10)  VALUE SPACES.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  REC-HEADER-3.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  REC-HEADER-3-1           PIC X(10)  VALUE
ENG           'TESTPROC: '.
           05  FILE-NAME-OUT            PIC X(40)  VALUE SPACES.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  REC-HEADER-3-2           PIC X(6)   VALUE
ENG           'DATE: '.
           05  DATUM-OUT.
               10  DAY-OUT              PIC 99.
               10  FILLER               PIC X VALUE '.'.
               10  MONTH-OUT            PIC 99.
               10  FILLER               PIC X VALUE '.'.
               10  YEAR-OUT             PIC 99.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  REC-HEADER-4.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  REC-HEADER-4-1           PIC X(10)  VALUE
ENG           'SYSTEM:   '.
           05  SYST-NAME-OUT            PIC X(40)  VALUE SPACES.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  REC-HEADER-4-2           PIC X(6)   VALUE
ENG           'PAGE: '.
           05  SEITE-OUT                PIC ZZZZZZZ9.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  OBJ-TITLE-LINE.
           05  FILLER                   PIC X(2)   VALUE '| '.
           05  OBJ-TITLE                PIC X(32)  VALUE
              'Data Object Name '.
           05  FILLER                   PIC X(3)   VALUE ' | '.
           05  OBJ-REL-TITLE            PIC X(32)  VALUE
              'in Document Section '.
           05  FILLER                   PIC X(3)   VALUE ' | '.
           05  OBJ-TEST-TITLE           PIC X(22)  VALUE
              ' TestCase Name '.
           05  FILLER                   PIC X(2)   VALUE ' |'.
      *
       01  OBJ-DATA-LINE.
           05  FILLER                   PIC X(2)   VALUE '| '.
           05  OBJ-NAME-OUT             PIC X(32).
           05  FILLER                   PIC X(3)   VALUE ' | '.
           05  OBJ-BASE-OUT             PIC X(32).
           05  FILLER                   PIC X(3)   VALUE ' | '.
           05  OBJ-TEST-OUT             PIC X(22).
           05  FILLER                   PIC X(2)   VALUE ' |'.
      *
       01  OBJ-FOOT-LINE-1.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  OBJ-FOOT-1       PIC X(48)  VALUE
ENG           'Number of Data Objects to be tested         =   '.
           05  OBJTS-TOTAL      PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(36)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  OBJ-FOOT-LINE-2.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  OBJ-FOOT-2       PIC X(48)  VALUE
ENG           'Number of Data Objects with a Test Case     =   '.
           05  OBJTS-WITH       PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(36)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  OBJ-FOOT-LINE-3.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  OBJ-FOOT-3       PIC X(48)  VALUE
ENG           'Number of Data Objects without a Test Case  =   '.
           05  OBJTS-WITHOUT    PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(36)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  OBJ-FOOT-LINE-4.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  OBJ-FOOT-4       PIC X(48)  VALUE
ENG           'Degree of Data Objects Coverage             =   '.
           05  OBJT-COVERAGE    PIC Z(2)9.99 VALUE ZERO.
           05  FILLER           PIC X(36)  VALUE SPACES.
           05  FILLER           PIC X(02)  VALUE ' |'.
      *
       01  OBJ-CSV-TITLE-LINE.
           05  OBJ-TITLE-1      PIC X(14)  VALUE 'LogicalObject;'.
           05  OBJ-TITLE-2      PIC X(20)  VALUE 'Requirement/UseCase;'.
           05  OBJ-TITLE-3      PIC X(08)  VALUE 'TestCase'.
      *
       01  OBJT-TABLE-LINE      PIC X(160) VALUE SPACES.
      *
      ****************************************************************
       LINKAGE SECTION.
      ****************************************************************
       77  SYSTEM-NAME                       PIC X(8).
       77  FILE-NAME                         PIC X(8).
       77  TEST-PROC-NAME                    PIC X(40).
       77  INPUT-FILE-NAME                   PIC X(80).
       77  SPECS-LIB-NAME                    PIC X(80).
       77  TABLE-LIB-NAME                    PIC X(80).
       77  WORK-LIB-NAME                     PIC X(80).
       77  RETCODE                           PIC 99.
      *
      * Interface to Object checking
      *
       01  OBJT-METRICS.
           05  NR-OBJTS                      PIC 9999 VALUE ZERO.
           05  NR-OBJTS-WITH                 PIC 9999 VALUE ZERO.
           05  NR-OBJTS-WITHOUT              PIC 9999 VALUE ZERO.
      *
KATI       COPY DBPARMS.
      *
      ****************************************************************
      /
       PROCEDURE DIVISION USING SYSTEM-NAME,
                                FILE-NAME,
                                TEST-PROC-NAME,
                                INPUT-FILE-NAME,
                                SPECS-LIB-NAME,
                                TABLE-LIB-NAME,
                                WORK-LIB-NAME,
                                RETCODE,
                                OBJT-METRICS,
KATI                            DB-PARAMS.
      ******************************************************************
      *
       OBJECT-PROCESSING SECTION.
      *
      ******************************************************************
      *
       START-OBJECTS.
      *
           DISPLAY '+----------------------------------------------+'
           DISPLAY '| OBJSPEC Report Program Vers 1.2 of Oct. 2008 |'
           DISPLAY '+----------------------------------------------+'
           DISPLAY '| System Name           = ' SYSTEM-NAME
           DISPLAY '| File   Name           = ' FILE-NAME
           DISPLAY '| Test Procedure Name   = ' TEST-PROC-NAME
           DISPLAY '| Import File Name      = ' INPUT-FILE-NAME
           DISPLAY '| Object Report Lib     = ' SPECS-LIB-NAME
           DISPLAY '| Object Table  Lib     = ' TABLE-LIB-NAME
           DISPLAY '+----------------------------------------------+'
TEST       DISPLAY '| Press Enter to continue !!'
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      ******************************************************************
      *
       OPEN-OBJECT-REPORT.
      *
           MOVE 'OBJT' TO DB-TYPE-CODE
           MOVE ZEROES TO COUNTS
           MOVE ZEROES TO SWITCHES
           MOVE ZEROES TO OBJT-METRICS
           MOVE ZEROES TO OBJT-STATUS
           MOVE SPACES TO OBJT-REPO-NAME
           STRING SPECS-LIB-NAME            DELIMITED BY SPACE
                 '\'                        DELIMITED BY SIZE
                  FILE-NAME                 DELIMITED BY SPACE
                 '.obj'                     DELIMITED BY SIZE
              INTO OBJT-REPO-NAME
           OPEN OUTPUT OBJT-REPORT.
TAB        MOVE SPACES TO OBJT-TAB-NAME
TEST       STRING SPECS-LIB-NAME            DELIMITED BY SPACE
TAB              '\'                        DELIMITED BY SIZE
TAB              FILE-NAME                  DELIMITED BY SPACE
TEST             '.cso'                     DELIMITED BY SIZE
TAB           INTO OBJT-TAB-NAME
TEST       DISPLAY 'OBJSPEC Output Table in:' OBJT-TAB-NAME
TEST***    ACCEPT CONTINUE-KEY
TAB        OPEN OUTPUT OBJT-TABLE.
           MOVE OBJ-CSV-TITLE-LINE TO OBJT-TABLE-REC
           WRITE OBJT-TABLE-REC.
     *******************************************************************
           MOVE ZERO  TO LINE-COUNT
           MOVE 1     TO PAGE-COUNT
           ACCEPT YYMMDD FROM DATE
           MOVE YY TO YEAR-OUT
           MOVE MM TO MONTH-OUT
           MOVE DD TO DAY-OUT.
           MOVE PAGE-COUNT   TO SEITE-OUT
           MOVE SYSTEM-NAME  TO SYST-NAME-OUT
           MOVE TEST-PROC-NAME TO FILE-NAME-OUT
           MOVE NEWPAGE-LINE TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE SEPARATOR-LINE TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE BLANK-LINE TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE OBJ-REC-HEADER-1 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE OBJ-REC-HEADER-2 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE BLANK-LINE TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE REC-HEADER-3 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE REC-HEADER-4 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE SEPARATOR-LINE-1 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE OBJ-TITLE-LINE TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE SEPARATOR-LINE-1 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC.
      *
      ******************************************************************
      *
       OPEN-OBJECT-FILE.
           MOVE INPUT-FILE-NAME TO REPO-FILE-NAME
           MOVE ZEROES TO REPO-STATUS
           OPEN INPUT REPO-FILE
           IF REPO-STATUS NOT = ZEROES
              DISPLAY '| Object File could not be opened !!'
              DISPLAY '| Object File Name = ' REPO-FILE-NAME
              DISPLAY '| Press Enter to continue !'
              ACCEPT CONTINUE-KEY
              MOVE 25 TO RETCODE
              GOBACK
           END-IF.
      *
      ******************************************************************
      *
       SORT-OBJECT-FILE.
TEST***    DISPLAY '| Sort the Objects !'.
           MOVE ZERO TO OBJT-INDEX
           MOVE ZERO TO CASE-INDEX
           MOVE SPACES TO OBJECT-TAB.
           MOVE ZEROES TO REPO-STATUS.
           PERFORM UNTIL REPO-EOF
              READ REPO-FILE
              AT END
                 MOVE '09' TO REPO-STATUS
              NOT AT END
                 IF REPO-TARGET-TYPE = 'DATA'
                 OR REPO-TARGET-TYPE = 'OBJT'
                    MOVE ZERO TO FOUND-SW
                    MOVE 1 TO J
                    PERFORM UNTIL J > OBJT-INDEX
                               OR J > OBJT-MAX
                               OR FOUND-SW = 1
                       IF REPO-TARGET-ENTITY (1:40) = OBJECT-NAME (J)
                          MOVE 1 TO FOUND-SW
                       END-IF
                       ADD 1 TO J
                    END-PERFORM
                    IF FOUND-SW = ZERO AND
                       OBJT-INDEX < OBJT-MAX
                       ADD 1 TO OBJT-INDEX
                    MOVE REPO-TARGET-ENTITY TO OBJECT-NAME (OBJT-INDEX)
                    END-IF
                 END-IF
                 IF REPO-TARGET-TYPE = 'TEST'
                    MOVE ZERO TO FOUND-SW
                    MOVE 1 TO J
                    PERFORM UNTIL J > CASE-INDEX
                               OR J > CASE-MAX
                               OR FOUND-SW = 1
                       IF REPO-TARGET-ENTITY (1:40) = TESTCASE-NAME (J)
                          MOVE 1 TO FOUND-SW
                       END-IF
                       ADD 1 TO J
                    END-PERFORM
                    IF FOUND-SW = ZERO AND
                       CASE-INDEX < CASE-MAX
                       ADD 1 TO CASE-INDEX
                   MOVE REPO-TARGET-ENTITY TO TESTCASE-NAME (CASE-INDEX)
                   MOVE REPO-BASE-ENTITY TO TESTCASE-BASE (CASE-INDEX)
                    END-IF
                 END-IF
              END-READ
           END-PERFORM.
TEST       DISPLAY '| ' OBJT-INDEX ' Objects were read in!'
TEST       DISPLAY '| ' CASE-INDEX ' Test Cases were read in!'
TEST***    DISPLAY '| Press Enter to continue !'
TEST***    ACCEPT CONTINUE-KEY
           PERFORM SORT-OBJECTS.
TEST       DISPLAY '| Objects were sorted !'.
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      ******************************************************************
      *
       PROCESS-OBJECTS.
      *
TEST***    DISPLAY 'Read in Objects  !!'
TEST***    ACCEPT CONTINUE-KEY
           MOVE 1 TO J
           PERFORM UNTIL J > OBJT-INDEX
                      OR J > OBJT-MAX
              ADD 1 TO NR-OBJTS
              MOVE OBJECT-NAME (J) TO OBJ-NAME-OUT
SN1008        MOVE OBJECT-NAME (J) TO CURRENT-OBJECT
              MOVE 'OBJT' TO DB-TYPE-CODE
              MOVE 'READ' TO DB-FUNC-CODE
              MOVE OBJECT-NAME (J) TO DB-OBJECT-NAME
              MOVE ZEROES TO DB-RETCODE
TEST***    DISPLAY 'CALL TCSIOMOD USING ' DB-TYPE-CODE ' ' DB-FUNC-CODE
TEST***            ' Key= '  DB-OBJECT-NAME
TEST***    ACCEPT CONTINUE-KEY
              CALL 'TCSIOMOD'  USING DB-PARAMS,
                               REPO-FILE-NAME,
                               WORK-LIB-NAME
CHEAT         ADD 1 TO NR-TC-REFS
CHEAT         MOVE ZERO TO DB-RETCODE
              MOVE DB-RETCODE TO RETCODE
              IF RETCODE > ZERO
TEST             DISPLAY '| Object: ' DB-OBJECT-NAME
TEST             DISPLAY '| is missing in the Test Cases!'
TEST             DISPLAY '| Press Enter to continue !'
TEST             ACCEPT CONTINUE-KEY
              END-IF
              IF NR-TC-REFS > ZERO
                 ADD 1 TO NR-OBJTS-WITH
                 MOVE 1 TO I
                 PERFORM UNTIL I > NR-TC-REFS
                            OR I > MAX-TC-REFS
                    MOVE TC-ID-REF (I) TO OBJ-TEST-OUT
                    MOVE SPACES TO OBJ-BASE-OUT
                    PERFORM FIND-BASE
                    MOVE OBJ-DATA-LINE TO OBJT-REPORT-REC
                    WRITE OBJT-REPORT-REC
TAB                 PERFORM WRITE-OBJT-TABLE
                    MOVE SPACES TO OBJ-NAME-OUT
                    ADD 1 TO LINE-COUNT
                    ADD 1 TO I
                 END-PERFORM
              ELSE
                 ADD 1 TO NR-OBJTS-WITHOUT
                 MOVE 'Testcase is missing!' TO OBJ-TEST-OUT
                 MOVE OBJ-DATA-LINE TO OBJT-REPORT-REC
                 WRITE OBJT-REPORT-REC
                 ADD 1 TO LINE-COUNT
              END-IF
              MOVE SEPARATOR-LINE-1 TO OBJT-REPORT-REC
              WRITE OBJT-REPORT-REC
              ADD 1 TO LINE-COUNT
              ADD 1 TO J
           END-PERFORM.
      *
      ******************************************************************
      *
       CLOSE-OBJECT-REPORT.
           MOVE NR-OBJTS TO OBJTS-TOTAL
           MOVE NR-OBJTS-WITH TO OBJTS-WITH
           MOVE NR-OBJTS-WITHOUT TO OBJTS-WITHOUT
           COMPUTE PERCENT = NR-OBJTS-WITH / NR-OBJTS
           MOVE PERCENT TO OBJT-COVERAGE
           MOVE SEPARATOR-LINE-1 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE BLANK-LINE TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE OBJ-FOOT-LINE-1 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE OBJ-FOOT-LINE-2 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE OBJ-FOOT-LINE-3 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE OBJ-FOOT-LINE-4 TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE BLANK-LINE TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           MOVE SEPARATOR-LINE TO OBJT-REPORT-REC
           WRITE OBJT-REPORT-REC
           CLOSE OBJT-REPORT.
TAB        CLOSE OBJT-TABLE.
      *
      ******************************************************************
      *
       CLOSE-OBJECT-FILES.
           CLOSE REPO-FILE
           MOVE 'OBJT' TO DB-TYPE-CODE
????***    MOVE 'CLOS' TO DB-FUNC-CODE
TEST***    DISPLAY 'CALL TCSIOMOD USING ' DB-TYPE-CODE ' ' DB-FUNC-CODE
TEST***    ACCEPT CONTINUE-KEY
????***    CALL 'TCSIOMOD'  USING DB-PARAMS,
????***                           REPO-FILE-NAME,
????***                           WORK-LIB-NAME
           DISPLAY '+-----------------------------------------+'
           DISPLAY '| Objects  were checked !!'
           DISPLAY '| Number of Objects          = ' NR-OBJTS
           DISPLAY '| Number of Objects tested   = ' NR-OBJTS-WITH
           DISPLAY '| Number of Objects untested = ' NR-OBJTS-WITHOUT
           DISPLAY '+-----------------------------------------+'
           DISPLAY '| Press Enter to terminate !'
TEST       ACCEPT CONTINUE-KEY.
           GOBACK.
      *
      ******************************************************************
      *
       WRITE-OBJT-TABLE.
TAB        MOVE SPACES TO OBJT-TABLE-REC
TAB        STRING CURRENT-OBJECT          DELIMITED BY SPACE
TAB                 ';'                   DELIMITED BY SIZE
TAB               OBJ-BASE-OUT            DELIMITED BY SPACE
TAB                 ';'                   DELIMITED BY SIZE
TAB               OBJ-TEST-OUT            DELIMITED BY SPACE
TAB           INTO OBJT-TABLE-REC
TAB        END-STRING
TAB        WRITE OBJT-TABLE-REC.
      *
KATI  ******************************************************************
       SORT-OBJECTS SECTION.
      ******************************************************************
      *          Sort the object tabel into ascending order
      *
       SORT-PROC.
           PERFORM VARYING S-I FROM 1 BY 1 UNTIL S-I > OBJT-INDEX
                                              OR S-I > OBJT-MAX
             MOVE S-I TO MIN-I
             MOVE OBJECT-NAME (S-I) TO WORK-OBJECT-NAME
             COMPUTE S-J = S-I + 1
             PERFORM VARYING S-J FROM S-J BY 1 UNTIL S-J > OBJT-INDEX
                                                  OR S-I > OBJT-MAX
               IF WORK-OBJECT-NAME > OBJECT-NAME (S-J)
                 MOVE OBJECT-NAME (S-J) TO WORK-OBJECT-NAME
                 MOVE S-J TO MIN-I
               END-IF
             END-PERFORM
             MOVE OBJECT-NAME (S-I) TO OBJECT-NAME (MIN-I)
             MOVE WORK-OBJECT-NAME TO OBJECT-NAME (S-I)
           END-PERFORM.
      *
KATI  ******************************************************************
       FIND-TESTCASE SECTION.
      ******************************************************************
      *          Find the Base Name of the TestCase
      *
       FIND-BASE.
           MOVE ZERO TO FOUND-SW
           PERFORM VARYING S-I FROM 1 BY 1 UNTIL S-I > CASE-INDEX
                                              OR S-I > CASE-MAX
                                              OR FOUND-SW = 1
             IF TC-ID-REF (I) = TESTCASE-NAME (S-I)
                MOVE 1 TO FOUND-SW
                MOVE TESTCASE-BASE (S-I) TO OBJ-BASE-OUT
                PERFORM ADJUST-BASE-ENTITY
             END-IF
           END-PERFORM.
           EXIT.
      *
       ADJUST-BASE-ENTITY.
           MOVE OBJ-BASE-OUT TO NAME-STRING
           MOVE NAME-LNG TO K
           PERFORM UNTIL NAME-CHAR (K) NOT = SPACE
                      OR K = 1
              SUBTRACT 1 FROM K
           END-PERFORM
           INSPECT NAME-STRING (1:K) REPLACING ALL ' ' BY '_'
           MOVE NAME-STRING TO OBJ-BASE-OUT.
           EXIT.
      *
      ********************** end of source *****************************
