      ******************************************************************
      * Analysis Tool for measuring & documenting Text Cases
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. TCSAUDIT.
      *AUTHOR.     H. SNEED.
      *DATE.       NOV. 2005.
      *VERSION.    1.1 of Nov. 2005
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *            Test Case Analysis Specification                    *
      * Accept XML Parameter File from the WebClient                   *
      * Convert XML Parameters to Internal Parameters                  *
      * Init Discrepancy Report                                        *
      * Init Metric Reports                                            *
      * Init Documentation Reports                                     *
      * Do for each TestCase Table                                     *
      *    Call TCSParse to parse test case file                       *
      * Enddo                                                          *
      * Print Metric Report                                            *
      * Print Specification Documents                                  *
      * Close Reports                                                  *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
      *
           SELECT TEST-FILE ASSIGN TO DYNAMIC INPUT-FILE
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS INPUT-STAT.
      *
           SELECT LOG-FILE ASSIGN TO DYNAMIC LOG-NAME
                  ORGANIZATION IS LINE SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS LOG-STAT.
      *
           SELECT TOTAL-FILE ASSIGN TO DYNAMIC SYST-NAME
                  ORGANIZATION IS RECORD SEQUENTIAL
                  ACCESS IS SEQUENTIAL
                  FILE STATUS IS SYST-STAT.
XML   *
XML        SELECT PARAM-FILE
XML               ASSIGN TO DYNAMIC XML-FILE
XML               ORGANIZATION IS LINE SEQUENTIAL
XML               ACCESS IS SEQUENTIAL
XML               FILE STATUS IS XML-STAT.
XML   *
XML        SELECT RESPONSE-FILE
XML               ASSIGN TO DYNAMIC OUTPUT-FILE
XML               ORGANIZATION IS LINE SEQUENTIAL
XML               ACCESS IS SEQUENTIAL
XML               FILE STATUS IS OUTPUT-STAT.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
XML   *
XML    FD  PARAM-FILE
XML        RECORD CONTAINS 80 CHARACTERS
XML        DATA RECORD IS PARAM-REC.
XML    01  PARAM-REC.
XML        05  PARAM-LINE                    PIC X(80).
XML        05  PARAM-CHARS                   REDEFINES PARAM-LINE.
XML            10  PARAM-CHAR                PIC X  OCCURS 80 TIMES.
XML   *
XML    FD  RESPONSE-FILE
XML        RECORD CONTAINS 80 CHARACTERS
XML        DATA RECORD IS RESPONSE-REC.
XML    01  RESPONSE-REC.
XML        05  RESPONSE-LINE                 PIC X(80).
XML        05  RESPONSE-CHARS                REDEFINES RESPONSE-LINE.
XML            10  RESPONSE-CHAR             PIC X  OCCURS 80 TIMES.
      *
       FD  TEST-FILE
           RECORD CONTAINS 160 CHARACTERS
           DATA RECORD IS TEST-REC.
       01  TEST-REC                         PIC X(160).
      *
       FD  LOG-FILE
           RECORD CONTAINS 72 CHARACTERS
           DATA RECORD IS LOG-REC.
       01  LOG-REC.
           05  FILLER                       PIC X(4).
           05  LOG-TEXT                     PIC X(64).
           05  FILLER                       PIC X(4).
      *
       FD  TOTAL-FILE
           RECORD CONTAINS 400 CHARACTERS
           DATA RECORD IS TOTAL-REC.
       01  TOTAL-REC                        PIC X(400).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
       77  SYST-NAME                         PIC X(80).
XML    77  XML-FILE                          PIC X(120).
       77  INPUT-FILE                        PIC X(80).
       77  OUTPUT-FILE                       PIC X(80).
       77  LOG-NAME                          PIC X(80).
       77  REPO-FILE-NAME                    PIC X(80).
       77  FUNCTION-FILE-NAME                PIC X(80).
XML    77  OUTPUT-STAT                       PIC XX.
       77  LOG-STAT                          PIC XX.
XML    77  XML-STAT                          PIC XX.
XML        88  XML-OK                        VALUE '00'.
XML        88  XML-EOF                       VALUE '09'.
       77  INPUT-STAT                        PIC XX.
           88  INPUT-OK                      VALUE '00'.
           88  INPUT-EOF                     VALUE '09'.
       77  PROD-STAT                         PIC XX.
           88  PROD-OK                       VALUE '00'.
           88  PROD-EOF                      VALUE '09'.
       77  SYST-STAT                         PIC XX.
           88  SYST-OK                       VALUE '00'.
           88  SYST-EOF                      VALUE '09'.
XML    77  ERROR-CODE                        PIC 99   VALUE ZERO.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
      *                          Parameter Keywords
           COPY XMLPARS.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 9999.
           05  J                             PIC 9999.
           05  K                             PIC 9999.
      ****************************************************************
      *
      *                          Pointer Fields
       01  POINTERS.
           05  TEXT-PTR                      PIC 9999.
           05  END-PTR                       PIC 9999.
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  LINE-LNG                      PIC 9999 VALUE 400.
           05  STRING-LNG                    PIC 9999 VALUE 120.
           05  LINE-MAX                      PIC 9999 VALUE 20.
           05  RULE-MAX                      PIC 9999 VALUE 80.
           05  FILE-MAX                      PIC 9999 VALUE 16.
           05  NAME-MAX                      PIC 9999 VALUE 8.
           05  LINE-LNG-MAX                  PIC 9999 VALUE 160.
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  FOUND-SW                      PIC 9    VALUE 0.
           05  END-SW                        PIC 9    VALUE 0.
           05  LIT-SW                        PIC 9    VALUE 0.
      *
      ****************************************************************
       01  VARIABLES.
           05  CHAR-COUNT                        PIC 9999.
           05  CODE-LINE                         PIC X(400).
           05  CODE-STRING           REDEFINES   CODE-LINE.
               10  CODE-CHAR                     PIC X OCCURS 400 TIMES.
           05  NAME-STRING.
               10  NAME-CHAR                     PIC X OCCURS 120 TIMES.
           05  SOURCE-STMT                       PIC X(80).
           05  TEST-TYPE                         PIC XXX.
           05  LAST-MODULE                       PIC X(8).
           05  LAST-COMPONENT                    PIC X(40).
           05  NR-CASE-FILES                     PIC 9999 VALUE ZERO.
           05  MIN-TESTCASES                     PIC 9999 VALUE ZERO.
      *
      ****************************************************************
      *        Table of key names
       01  KEY-TAB.
           05  TC-NAME                           PIC X(40).
           05  REQU-NAME                         PIC X(40).
           05  UC-NAME                           PIC X(40).
           05  OBJT-NAME                         PIC X(40).
      *
      ****************************************************************
           COPY TMETTAB.
      *
      ****************************************************************
           COPY TTOTTAB.
      *
      ****************************************************************
           COPY TFILPAR.
      *
      ****************************************************************
           COPY TRULPAR.
      *
      ****************************************************************
           COPY TMETPARS.
      *
      ****************************************************************
           COPY TMAXPARS.
      *
      ****************************************************************
           COPY DBPARMS.
      *
      ****************************************************************
REPO       COPY REPOPARM.
      *
      ****************************************************************
REPO       COPY REPODATA.
      *
      ****************************************************************
XML        COPY XMLMSGS.
      *
      ****************************************************************
      *  Data for processing XML Files                               *
      ****************************************************************
      *
XML        COPY XMLDATA.
      *
XML        COPY CONVPARS.
      *
      ****************************************************************
      ****************************************************************
      *LINKAGE SECTION.
      *                          Parameters
           COPY SYSPARS.
       77  TEST-NAME                         PIC X(8).
*******77  METRIC-FC                         PIC XXXX.
SN0202 77  METRIC-RC                         PIC 99.
SN0300 77  CHECK-OPTION                      PIC X.
SN0300 77  TEST-OPTION                       PIC X.
SN0300 77  SPEC-OPTION                       PIC X.
      *
      * Interface to Test Case Processing
      *
           COPY CASEPARS.
      *
      * Interface to Test Case checking
      *
       01  TCS-METRICS.
           05  TCS-NR                        PIC 9999.
           05  TCS-REQ-NR                    PIC 9999.
           05  TCS-UCS-NR                    PIC 9999.
           05  TCS-OBJ-NR                    PIC 9999.
      *
      * Interface to Requirements checking
      *
       01  REQU-METRICS.
           05  NR-REQUS                      PIC 9999.
           05  NR-REQUS-WITH                 PIC 9999.
           05  NR-REQUS-WITHOUT              PIC 9999.
      *
      * Interface to Use Case checking
      *
       01  USES-METRICS.
           05  NR-USES                       PIC 9999.
           05  NR-USES-WITH                  PIC 9999.
           05  NR-USES-WITHOUT               PIC 9999.
      *
      * Interface to Object checking
      *
       01  OBJT-METRICS.
           05  NR-OBJTS                      PIC 9999.
           05  NR-OBJTS-WITH                 PIC 9999.
           05  NR-OBJTS-WITHOUT              PIC 9999.
      *
      ****************************************************************
      *
      *  Parameters to be passed
      *
       77  FUNCT-CODE                        PIC XX.
       77  RET-CODE                          PIC XX.
       77  MESS-FILE-NAME                    PIC X(80).
      *
XML        COPY CALCPARS.
      *
           COPY CALCSUMS.
      *
      ****************************************************************
      *
      *  Parameters to Operating System
      *
       01  CMD-LINE-STRING                   PIC X(80).
       01  RESULT                            PIC X  COMP-X.
       01  FUNCT                             PIC X  COMP-X VALUE 35.
       01  CMD-LINED.
           05  CMD-LINE-LNG                  PIC X  COMP-X VALUE ZERO.
      ******************************************************************
      *
      /
       PROCEDURE DIVISION.
      ******************************************************************
      *     Start routine     ******************************************
      ******************************************************************
       CSSANAL-START SECTION.
      ******************************************************************
      *
XML    SETUP-PARAMETERS.
      *
WRAP       ACCEPT CMD-LINE-STRING FROM COMMAND-LINE
WRAP       MOVE CMD-LINE-STRING TO XML-FILE.
TEST***    DISPLAY 'Enter ParamFile Name !'
TEST***    ACCEPT XML-FILE.
      *
           MOVE ZERO   TO ERROR-CODE.
           MOVE ZEROES TO XML-STAT.
           OPEN INPUT PARAM-FILE
           IF XML-STAT NOT = ZEROS
              DISPLAY 'Parameter File not found'
              DISPLAY 'Parameter File Name:' XML-FILE
              DISPLAY 'TestComp  could not execute !!!'
              DISPLAY 'Press enter'
              ACCEPT CONTINUE-KEY
              MOVE 1 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           END-IF.
      *
XML        PERFORM READ-PARAM-FILE.
      *
      * for Link to Redoc Export File
      *
           IF ERROR-CODE = ZERO
TEST          MOVE ALL 'Y' TO FILE-TAB
              DISPLAY '+----------------------------------------------+'
              DISPLAY '| TCSAnal Test Case Analyzer Vers 1.4 Oct.2008 |'
              DISPLAY '| Copyright: Harry M. Sneed Budapest, Hungary  |'
              DISPLAY '+----------------------------------------------+'
              DISPLAY '| Product Name     = ' PRODUCT-NAME
              DISPLAY '| System Name      = ' SYSTEM-NAME
              DISPLAY '| Test Case Version= ' AUDIT-VERSION
              DISPLAY '| Test Case Type   = ' AUDIT-LANGUAGE
              DISPLAY '| Repository Code  = ' REPO-CODE
              DISPLAY '| Audit Code       = ' AUDIT-CODE
              DISPLAY '| Redoc Export File= ' REPO-FILE-NAME
              DISPLAY '| Input  Directory = ' INPUT-LIB-NAME
              DISPLAY '| Output Directory = ' OUTPUT-LIB-NAME
              DISPLAY '| Work   Directory = ' WORK-LIB-NAME
              DISPLAY '| Function Table   = ' FUNCTION-FILE-NAME
              DISPLAY '| Cross Ref Table  = ' FILE-TAB
              DISPLAY '+-------------------------------------------+'
TEST          DISPLAY '| Press Enter to start Analysis !!'
TEST          ACCEPT CONTINUE-KEY
           ELSE
              MOVE 2 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           END-IF.
      *
      *****************************************************
       SET-SUBDIRECTORIES.
      ***********************
      ***********************
      * for TestCase metrics
DOS        MOVE SPACES TO METRIC-LIB-NAME
DOS        STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
DOS               '\METRICS'       DELIMITED BY SIZE
DOS           INTO METRIC-LIB-NAME
DOS        END-STRING.
      ***********************
      * for TestCase  Deficiencies
DOS        MOVE SPACES TO DEFI-LIB-NAME
DOS        STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
DOS               '\DEFIS'         DELIMITED BY SIZE
DOS           INTO DEFI-LIB-NAME
DOS        END-STRING.
      ***********************
      * for TestCase reports
DOS        MOVE SPACES TO SPEC-LIB-NAME
DOS        STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
DOS               '\SPECS'         DELIMITED BY SIZE
DOS           INTO SPEC-LIB-NAME
DOS        END-STRING.
      ***********************
      * for TestCase tables
DOS        MOVE SPACES TO TABL-LIB-NAME
DOS        STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
DOS               '\TABLES'        DELIMITED BY SIZE
DOS           INTO TABL-LIB-NAME
DOS        END-STRING.
      ***********************
      * for Cross Reference Files
DOS        MOVE SPACES TO EXPO-LIB-NAME
DOS        STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
DOS               '\EXPORTS'       DELIMITED BY SIZE
DOS           INTO EXPO-LIB-NAME
DOS        END-STRING.
      *
      *****************************************************
      *
       INIT-COUNTERS.
           PERFORM OPEN-LOG-FILE
           MOVE ZEROES TO INDEXES.
           MOVE ZEROES TO METRIC-TAB.
           MOVE ZEROES TO TOTAL-TAB.
      *
      ***************************************************
      *
       OPEN-SYSTEM-TOTAL.
           MOVE SPACES TO SYST-NAME
           STRING OUTPUT-LIB-NAME     DELIMITED BY SPACE
                 '\'                  DELIMITED BY SIZE
                 PRODUCT-NAME         DELIMITED BY SPACE
                 '.sum'               DELIMITED BY SIZE
             INTO SYST-NAME
           END-STRING.
           MOVE ZEROES TO SYST-STAT.
           OPEN INPUT TOTAL-FILE.
           IF SYST-OK
              READ TOTAL-FILE
              AT END
                 MOVE '09' TO SYST-STAT
              NOT AT END
                 MOVE TOTAL-REC TO TOTAL-TAB
              END-READ
              CLOSE TOTAL-FILE
           ELSE
              MOVE ZEROES TO TOTAL-TAB
           END-IF.
      *
      ***************************************************************
      ***************************************************************
       PROCESS-ALL-FILES.
XML   *
XML   ** Read all Source File Names from XML File
XML   *
           MOVE SPACES TO LAST-MODULE.
           MOVE 1 TO NR-SYSTEMS.
           MOVE 1 TO NR-SYSTEMS-TOTAL.
           MOVE 1 TO NR-MODULES.
           MOVE 1 TO NR-MODULES-TOTAL.
XML        PERFORM UNTIL XML-LABEL = FILES-END
XML                   OR XML-EOF
XML           READ PARAM-FILE
XML           AT END
XML              MOVE '09' TO XML-STAT
XML           NOT AT END
XML              MOVE PARAM-REC TO XML-LINE
XML              PERFORM PARSE-XML
XML              IF ERROR-CODE > ZERO
XML                 MOVE 2 TO ERROR-CODE
XML                 PERFORM RETURN-RESPONSE
XML              END-IF
XML           END-READ
XML           IF XML-STAT = ZEROES AND
XML              XML-LABEL (1:9) = ORIG-FILE
XML              MOVE XML-VALUE TO PATH-NAME
TEST             DISPLAY 'Orig Text File = ' PATH-NAME
TEST***          ACCEPT CONTINUE-KEY
XML           END-IF
XML           IF XML-STAT = ZEROES AND
XML              XML-LABEL (1:5) = FILENR
XML              MOVE SPACES TO INPUT-FILE
XML              STRING
TEST                    INPUT-LIB-NAME   DELIMITED BY SPACE
TEST                   '\'               DELIMITED BY SIZE
XML                     XML-VALUE        DELIMITED BY SPACE
XML                 INTO INPUT-FILE
XML              END-STRING
TEST             DISPLAY 'Next Text File = ' PATH-NAME
TEST***          ACCEPT CONTINUE-KEY
XML   *************************************************************
                 PERFORM GET-ORIGINAL-NAMES
                 PERFORM GET-SHORT-NAMES
PATCH            MOVE SPACES TO TEST-PROC-NAME
PATCH            STRING CASE-TAB-NAME    DELIMITED BY SPACE
PATCH                  '-TEST'           DELIMITED BY SIZE
PATCH               INTO TEST-PROC-NAME
      *************************************************************
      **      Group Change for Module
      *************************************************************
                 IF MODULE-NAME NOT = LAST-MODULE
REPO                PERFORM OPEN-REPOREF
REPO                PERFORM CALL-REPOREF-SYSTEM
                 END-IF
                 IF MODULE-NAME NOT = LAST-MODULE
                    MOVE ZEROES TO METRIC-TAB
                    MOVE 1 TO NR-MODULES
                    ADD 1 TO NR-MODULES-TOTAL
                 END-IF
                 MOVE MODULE-NAME TO LAST-MODULE
      *************************************************************
      **      Read and Process next Text Member
      *************************************************************
                 ADD 1 TO NR-CASE-FILES
                 MOVE SPACES TO CASE-FILE-NAME
                 STRING WORK-LIB-NAME          DELIMITED BY SPACE
                       '\'                     DELIMITED BY SIZE
                        FILE-NAME              DELIMITED BY SPACE
                        '.tcs'                 DELIMITED BY SIZE
                    INTO CASE-FILE-NAME
                 END-STRING
      *       Invoke CSVPARS, XMLPARS oder TXTPARS
      *       depending on the File Type
                 MOVE ZERO TO RETCODE
                 PERFORM OPEN-INPUT-FILE
                 IF RETCODE = ZERO
                    EVALUATE FILE-TYPE
                       WHEN 'XML'
                       WHEN 'xml'
                          PERFORM CALL-XMLPARSE
                       WHEN 'CSV'
                       WHEN 'csv'
                          PERFORM CALL-CSVPARSE
                       WHEN 'TXT'
                       WHEN 'txt'
                          PERFORM CALL-TXTPARSE
                       WHEN OTHER
                          MOVE 52 TO ERROR-CODE
                    END-EVALUATE
                 END-IF
      ***************************************************************
      *  Process the Test Cases                                     *
      ***************************************************************
                 IF RETCODE = ZERO
                    CALL 'TCSPRUF' USING SYSTEM-NAME,
                                         FILE-NAME,
                                         TEST-PROC-NAME,
                                         DEFI-LIB-NAME,
                                         FUNCTION-FILE-NAME,
                                         CASE-FILE-NAME,
                                         AUDIT-CODE,
                                         RET-CODE,
                                         MAX-LIMITS-TAB,
                                         METRIC-TAB,
                                         KEY-TAB,
                 END-IF
      ***************************************************************
      *  Setup the Database of TestCases                            *
      ***************************************************************
                 IF RETCODE = ZERO
                    MOVE 'INIT' TO DB-FUNC-CODE
                    MOVE 'CASE' TO DB-TYPE-CODE
                    MOVE TC-NAME TO DB-TC-ID
                    MOVE REQU-NAME TO DB-REQ-NAME
                    MOVE UC-NAME TO DB-USECASE-NAME
                    MOVE OBJT-NAME TO DB-OBJECT-NAME
                    MOVE ZEROES TO DB-RETCODE
                    CALL 'TCSIOMOD'  USING DB-PARAMS,
                                           CASE-FILE-NAME,
                                           WORK-LIB-NAME,
                                           DB-RETCODE
                    MOVE DB-RETCODE TO RETCODE
                 END-IF
      ***************************************************************
      *  Process the Test Cases                                     *
      ***************************************************************
                 IF RETCODE = ZERO
                    CALL 'TCSSPEC' USING SYSTEM-NAME,
                                         FILE-NAME,
                                         TEST-PROC-NAME,
                                         CASE-FILE-NAME,
                                         SPEC-LIB-NAME,
                                         TABL-LIB-NAME,
                                         EXPO-LIB-NAME,
                                         FUNCTION-FILE-NAME,
                                         PATH-NAME,
                                         REPO-CODE,
                                         RETCODE,
                                         TCS-METRICS
      ***************************************************************
      *  Process the Requirements                                   *
      ***************************************************************
                    IF FILE-ENTRY (1) = 'Y'
                       CALL 'REQSPEC' USING SYSTEM-NAME,
                                            FILE-NAME,
                                            TEST-PROC-NAME,
                                            REPO-FILE-NAME,
                                            SPEC-LIB-NAME,
TEST                                        TABL-LIB-NAME,
                                            WORK-LIB-NAME,
                                            RETCODE,
                                            REQU-METRICS,
KATI                                        DB-PARAMS
METRIC                 ADD NR-REQUS TO NR-TARGET-REQUS
METRIC                 ADD NR-REQUS-WITH TO NR-TESTED-REQUS
METRIC                 ADD NR-REQUS-WITHOUT TO NR-UNTESTED-REQUS
                    END-IF
      ***************************************************************
      *  Process the Object  File                                   *
      ***************************************************************
                    IF FILE-ENTRY (2) = 'Y'
                       CALL 'OBJSPEC' USING SYSTEM-NAME,
                                            FILE-NAME,
                                            TEST-PROC-NAME,
                                            REPO-FILE-NAME,
                                            SPEC-LIB-NAME,
TEST                                        TABL-LIB-NAME,
                                            WORK-LIB-NAME,
                                            RETCODE,
                                            OBJT-METRICS,
KATI                                        DB-PARAMS
METRIC                 ADD NR-OBJTS TO NR-TARGET-OBJECTS
METRIC                 ADD NR-OBJTS-WITH TO NR-TESTED-OBJECTS
METRIC                 ADD NR-OBJTS-WITHOUT TO NR-UNTESTED-OBJECTS
                    END-IF
      ***************************************************************
      *  Process the UseCase File                                   *
      ***************************************************************
                    IF FILE-ENTRY (3) = 'Y'
                       CALL 'USESPEC' USING SYSTEM-NAME,
                                            FILE-NAME,
                                            TEST-PROC-NAME,
                                            REPO-FILE-NAME,
                                            SPEC-LIB-NAME,
TEST                                        TABL-LIB-NAME,
                                            WORK-LIB-NAME,
                                            RETCODE,
                                            USES-METRICS,
KATI                                        DB-PARAMS
METRIC                 ADD NR-USES TO NR-TARGET-FUNCS
METRIC                 ADD NR-USES-WITH TO NR-TESTED-FUNCS
METRIC                 ADD NR-USES-WITHOUT TO NR-UNTESTED-FUNCS
                    END-IF
                 END-IF
      **********************************************************
      **        Print Metric Report for Module
      **********************************************************
                 PERFORM ADJUST-METRICS
                 PERFORM TOTAL-METRICS
                 CALL 'TMETREPO' USING COMPONENT-NAME,
                                       FILE-NAME,
                                       FILE-NAME,
                                       FILE-TYPE,
                                       METRIC-LIB-NAME,
                                       METR-TAB,
                                       METRIC-TAB
              END-IF
           END-PERFORM.
           CLOSE PARAM-FILE.
      ***************************************************************
       PRINT-METRIC-SUMMARY.
      *  Print Summary for System
           MOVE SYSTEM-NAME TO MODULE-NAME
           MOVE SYSTEM-NAME TO COMPONENT-NAME
           MOVE 'sys' TO TEST-TYPE
           CALL 'TMETREPO' USING COMPONENT-NAME,
                                 MODULE-NAME,
                                 TEST-NAME,
                                 TEST-TYPE,
                                 METRIC-LIB-NAME,
                                 METR-TAB,
                                 METRIC-TAB.
           MOVE PRODUCT-NAME TO MODULE-NAME
           MOVE PRODUCT-NAME TO COMPONENT-NAME
           MOVE 'prod' TO TEST-TYPE
           CALL 'TMETREPO' USING COMPONENT-NAME,
                                 MODULE-NAME,
                                 TEST-NAME,
                                 TEST-TYPE,
                                 METRIC-LIB-NAME,
                                 METR-TAB,
                                 TOTAL-TAB.
           EXIT.
      **********************************************************
      **        Write Metric Export for System
      ***************************************************
       CREATE-MESS-FILE-SYSTEM.
           MOVE 'WR'          TO FUNCT-CODE.
           MOVE ZERO          TO RET-CODE.
           MOVE PRODUCT-NAME  TO USER-ID.
           MOVE SYSTEM-NAME   TO EXPORT-NAME.
           MOVE 'TCSANAL'     TO EXPORTEUR.
           MOVE 'PROCESS'     TO EXPORT-TYPE.
           MOVE 'SYSTEM '     TO ENTITY-TYPE.
           MOVE SYSTEM-NAME   TO  ENTITY-NAME
           MOVE FILE-TYPE     TO ENTITY-LANG.
           MOVE SPACES        TO MESS-FILE-NAME
           STRING OUTPUT-LIB-NAME  DELIMITED BY SPACE
SN1213            '\SoftEval\'     DELIMITED BY SIZE
                  SYSTEM-NAME      DELIMITED BY SPACE
                  '.xml'           DELIMITED BY SIZE
              INTO MESS-FILE-NAME
           END-STRING.
           CALL 'TCSMESS'  USING FUNCT-CODE,
                                 RET-CODE,
                                 MESS-FILE-NAME,
                                 EXPORT-DATA,
                                 METR-TAB,
                                 METRIC-TAB.
           IF RET-CODE NOT = ZEROES
              DISPLAY '| System Export File could not be opened'
              MOVE    '| System Export File not opened ' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| SoftMess File Name = ' DELIMITED BY SIZE
                       MESS-FILE-NAME          DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| Execution was terminated!' TO LOG-TEXT
              WRITE LOG-REC
              CLOSE LOG-FILE
TEST***       DISPLAY '| Press Enter to Terminate '
TEST***       ACCEPT CONTINUE-KEY
              MOVE 4 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           ELSE
              DISPLAY '| Metric Export File written!!'
              DISPLAY '| File: ' MESS-FILE-NAME
              MOVE    '| Metric Export File written' TO LOG-TEXT
              MOVE SPACES TO LOG-TEXT
              STRING  '| TestCase File Name = ' DELIMITED BY SIZE
                       MESS-FILE-NAME           DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
           END-IF.
      ***************************************************
       SAVE-SYSTEM-SUMMARY.
           MOVE SPACES TO SYST-NAME
           STRING OUTPUT-LIB-NAME     DELIMITED BY SPACE
                 '\'                  DELIMITED BY SIZE
                 PRODUCT-NAME         DELIMITED BY SPACE
                 '.sum'               DELIMITED BY SIZE
             INTO SYST-NAME
           END-STRING.
           MOVE ZEROES TO SYST-STAT.
           OPEN OUTPUT TOTAL-FILE.
           IF SYST-OK
              MOVE TOTAL-TAB TO TOTAL-REC
              WRITE TOTAL-REC
              CLOSE TOTAL-FILE
           END-IF.
      **********************************************************
       END-TCSANAL.
REPO       PERFORM CLOSE-REPOREF.
           DISPLAY '+---------------------------------------+'
           DISPLAY '| TCSANAL Test Case Analysis completed  |'
           DISPLAY '+---------------------------------------+'
           DISPLAY '| Tables processed = ' NR-TEST-PROCS
           PERFORM CLOSE-LOG-FILE.
TEST  *    DISPLAY '| Press Enter to Terminate!'
TEST  *    ACCEPT  CONTINUE-KEY
      *       return Reponse & terminate analysis
           PERFORM RETURN-RESPONSE.
           GOBACK.
      ******************************************************************
      ******************************************************************
      *      Main Loop for each Test Case Table       ******************
      ******************************************************************
       PROCESS-TESTCASE  SECTION.
      ***************************************************************
       OPEN-INPUT-FILE.
           MOVE ZEROES TO INPUT-STAT.
           OPEN INPUT TEST-FILE.
           IF INPUT-STAT NOT = ZEROES
              DISPLAY '| Test Case File cannot be opened'
              DISPLAY '| Test Case File Path = ' INPUT-FILE
              DISPLAY '| Analysis cannot be continued !!!'
              MOVE    '| Test Case File not opened' TO LOG-TEXT
              WRITE LOG-REC
              MOVE SPACES TO LOG-TEXT
              STRING  '| Test Case File Name = ' DELIMITED BY SIZE
                       INPUT-FILE               DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
              MOVE    '| File can not be processed!' TO LOG-TEXT
              WRITE LOG-REC
TEST          DISPLAY '| Press Enter to Continue!'
TEST          ACCEPT  CONTINUE-KEY
              MOVE 5 TO ERROR-CODE
              PERFORM RETURN-RESPONSE
              GOBACK
           ELSE
SN0406        READ TEST-FILE
              AT END
                 MOVE '09' TO INPUT-STAT
              NOT AT END
TEST             DISPLAY 'First-Rec=' TEST-REC (1:40)
TEST***          ACCEPT CONTINUE-KEY
                 IF TEST-REC (1:4) = 'Type'
                    MOVE '09' TO INPUT-STAT
                    MOVE  09  TO RETCODE
                 END-IF
              END-READ
              DISPLAY '+--------------------------------------------+'
              DISPLAY '| Test Case File to be analyzed = ' FILE-NAME
              DISPLAY '+--------------------------------------------+'
              MOVE SPACES TO LOG-TEXT
              STRING  '| Test Case File analyzed = ' DELIMITED BY SIZE
                       PATH-NAME                     DELIMITED BY SIZE
                 INTO LOG-TEXT
              WRITE LOG-REC
              CLOSE TEST-FILE
           END-IF.
      **************************************************************
       CALL-XMLPARSE.
      * Call CONVERT to convert XML Test Cases
TEST       DISPLAY 'Convert XML Test Cases'
TEST***    ACCEPT CONTINUE-KEY
METRIC     ADD 1 TO NR-TEST-PROCS
           MOVE 'TestFall' TO FILES-NAME
           MOVE INPUT-FILE TO OLD-FILE
           MOVE CASE-FILE-NAME TO NEW-FILE
           MOVE 'XML' TO OLD-TYPE
           MOVE 'TXT' TO NEW-TYPE
           MOVE FILE-NAME TO CONVERT-NAME
           CALL 'CONVERT' USING CONVERSION-PARAMS,
                                RETCODE
TEST***    DISPLAY 'Return from CONVERT Retcode = ' RETCODE
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      **************************************************************
       CALL-CSVPARSE.
      * Call CONVERT to convert CSV Test Cases
TEST       DISPLAY 'Convert CSV Test Cases'
TEST***    ACCEPT CONTINUE-KEY
METRIC     ADD 1 TO NR-TEST-PROCS
           MOVE 'TestFall' TO FILES-NAME
           MOVE INPUT-FILE TO OLD-FILE
           MOVE CASE-FILE-NAME TO NEW-FILE
           MOVE 'CSV' TO OLD-TYPE
           MOVE 'TXT' TO NEW-TYPE
           MOVE FILE-NAME TO CONVERT-NAME
           CALL 'CONVERT' USING CONVERSION-PARAMS,
                                RETCODE
TEST***    DISPLAY 'Return from CONVERT Retcode = ' RETCODE
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      **************************************************************
       CALL-TXTPARSE.
      * Call CONVERT to convert TXT Test Cases
TEST       DISPLAY 'Convert TXT Test Cases'
TEST***    ACCEPT CONTINUE-KEY
METRIC     ADD 1 TO NR-TEST-PROCS
           MOVE 'TestFall' TO FILES-NAME
           MOVE INPUT-FILE TO OLD-FILE
           MOVE CASE-FILE-NAME TO NEW-FILE
           MOVE 'TXT' TO OLD-TYPE
           MOVE 'TXT' TO NEW-TYPE
           MOVE FILE-NAME TO CONVERT-NAME
           CALL 'CONVERT' USING CONVERSION-PARAMS,
                                RETCODE
TEST***    DISPLAY 'Return from CONVERT Retcode = ' RETCODE
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      ******************************************************************
       LOGGING SECTION.
      ******************************************************************
      *   Open  Log File for this Comparison Job                  *
      *************************************************************
      *
       OPEN-LOG-FILE.
           MOVE SPACES TO LOG-NAME
           STRING OUTPUT-LIB-NAME    DELIMITED BY SPACE
                 '\'                 DELIMITED BY SIZE
                 SYSTEM-NAME         DELIMITED BY SPACE
                 '.log'              DELIMITED BY SIZE
             INTO LOG-NAME
           END-STRING.
           MOVE ZEROES TO LOG-STAT.
           OPEN OUTPUT LOG-FILE.
           MOVE SPACES TO LOG-REC.
           MOVE ALL '*' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '| TCSANAL Version 1.4 of October 2008' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '| Copyright Harry M Sneed ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '| of Soring Kft, Budapest, Hungary ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE '|         Execution Protocol       ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
           WRITE LOG-REC.
           EXIT.
      *
      *************************************************************
      *   Write Log File for this Comparison Job                  *
      *************************************************************
      *
       WRITE-LOG-FILE.
           MOVE SPACES TO LOG-TEXT
           STRING '| Analysis of Test Case File: ' DELIMITED BY SIZE
                  FILE-NAME                        DELIMITED BY SPACE
                  ' terminated !!       '          DELIMITED BY SIZE
              INTO LOG-TEXT
           WRITE LOG-REC.
TEST       DISPLAY '| LogFile written '.
      *
      *************************************************************
      *   Close Log File for this Comparison Job                  *
      *************************************************************
      *
       CLOSE-LOG-FILE.
PATCH      SUBTRACT 1 FROM NR-TEST-PROCS
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
           WRITE LOG-REC.
           MOVE    '| TCSANAL is finished ' TO LOG-TEXT
           WRITE LOG-REC.
           MOVE SPACES TO LOG-TEXT.
           STRING  '| Test Case Files processed = '   DELIMITED BY SIZE
                   NR-TEST-PROCS                      DELIMITED BY SIZE
              INTO LOG-TEXT
           WRITE LOG-REC.
           MOVE ALL '-' TO LOG-TEXT
           MOVE '+' TO LOG-TEXT (1:1)
           WRITE LOG-REC.
           CLOSE LOG-FILE.
TEST       DISPLAY '| LogFile closed '.
           EXIT.
      *
      *****************************************************************
REPO   REPOSITORY  SECTION.
      *****************************************************************
      *
REPO   OPEN-REPOREF.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE SYSTEM-NAME TO REPO-NAME
REPO          MOVE 'PROD' TO REF-BASE-TYPE
REPO          MOVE OUTPUT-LIB-NAME TO OUTPUT-DIR
REPO          MOVE PRODUCT-NAME TO REF-BASE-NAME
REPO          MOVE 'OWNS' TO REF-RELATION
REPO          MOVE 'SYS ' TO REF-TARGET-TYPE
REPO          MOVE SYSTEM-NAME TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE,
REPO          IF RETCODE NOT = ZERO
REPO             DISPLAY 'Reference File could not be opened !!'
REPO             DISPLAY 'Press Enter Key to Continue without it !!'
REPO             ACCEPT  CONTINUE-KEY
REPO          END-IF
REPO       END-IF.
      *
      *****************************************************************
       CALL-REPOREF-SYSTEM.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE 'PROD' TO REF-BASE-TYPE
REPO          MOVE PRODUCT-NAME TO REF-BASE-NAME
REPO          MOVE 'OWNS' TO REF-RELATION
REPO          MOVE 'SYST' TO REF-TARGET-TYPE
REPO          MOVE SYSTEM-NAME TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF,
REPO                               OUTPUT-DIR,
REPO                               RETCODE
REPO       END-IF.
      *
      *****************************************************************
REPO   CLOSE-REPOREF.
REPO       IF REPO-CODE = 'Y'
REPO          MOVE SYSTEM-NAME TO REPO-NAME
REPO          MOVE EXPO-LIB-NAME TO OUTPUT-DIR
REPO          MOVE 'CLOS' TO REF-BASE-TYPE
REPO          MOVE PRODUCT-NAME TO REF-BASE-NAME
REPO          MOVE 'CLOS' TO REF-RELATION
REPO          MOVE 'SYS ' TO REF-TARGET-TYPE
REPO          MOVE SYSTEM-NAME TO REF-TARGET-NAME
REPO          CALL 'REPOREF' USING REPO-NAME,
REPO                               EXTERNAL-REF
REPO                               OUTPUT-DIR
REPO                               RETCODE
REPO       END-IF.
      *
      ******************************************************************
       SUBROUTINES SECTION.
      ******************************************************************
      *    Extract Names from Paths
           COPY NAMEXTR.
           EXIT.
      ******************************************************************
      *    METRIC Summary
           COPY TMETSUM.
           EXIT.
      ******************************************************************
      *    XML Initialisation
           COPY XMLINIT.
           EXIT.
      ********************** end of source *****************************
