      ******************************************************************
      *   PROGRAM  FOR  PRODUCING DEFICIENCY REPORT FOR TESTCASES      *
      ******************************************************************
       IDENTIFICATION DIVISION.
VER-10 PROGRAM-ID. TCSPRUF.
      *AUTHOR.     H. SNEED.
      *DATE.       MARCH 2006.
      *VERS.       1.0 of Mar. 2006
      *REMARKS.    PROGRAM SPECIFICATION:
      ******************************************************************
      *                  TCS    CHECKING    SPECIFICATION              *
      * Read in Attributes                                             *
      * Initialize Defi Report                                         *
      * Open TestCase File                                             *
      * Open Database                                                  *
      * Read TestCases                                                 *
      *    Read a TestCase                                             *
      *       check attributes                                         *
      *       check requirement link                                   *
      *       check use case link                                      *
      *       check object link                                        *
      *    Print report if test case has a deficiency                  *
      * Eof Testcases                                                  *
      * Print Report Summary                                           *
      * Close File                                                     *
      ******************************************************************
      /
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SPECIAL-NAMES.
       INPUT-OUTPUT SECTION.
      ******************************************************************
       FILE-CONTROL.
      ******************************************************************
      *
      * Testcase Input File
      *
           SELECT  ATTR-FILE
                   ASSIGN TO DYNAMIC ATTR-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS ATTR-STATUS.
      *
           SELECT  CASE-FILE
                   ASSIGN TO DYNAMIC CASE-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS CASE-STATUS.
      *
      * DEF audit  output file
      *
           SELECT  DEFI-FILE
                   ASSIGN TO DYNAMIC DEFI-FILE-NAME
                   ORGANISATION LINE SEQUENTIAL
                   ACCESS SEQUENTIAL
                   FILE STATUS IS DEFI-STATUS.
      *
      ******************************************************************
      /
       DATA DIVISION.
       FILE SECTION.
      ******************************************
      *
       FD  ATTR-FILE
           RECORD CONTAINS 80 CHARACTERS
           DATA RECORD IS ATTR-REC.
       01  ATTR-REC.
           05 ATTR-NUM               PIC 9(4).
           05 FILLER                 PIC X.
           05 ATTR-LNG               PIC 99.
           05 FILLER                 PIC X.
           05 ATTR-TYP               PIC X(4).
           05 FILLER                 PIC X.
           05 ATTR-NAME              PIC X(36).
           05 FILLER                 PIC X(31).
      *
       FD  CASE-FILE
           RECORD CONTAINS 244 CHARACTERS
           DATA RECORD IS CASE-REC.
       01  CASE-REC.
           05  CASE-REC-NR                   PIC 9999.
           05  CASE-REC-LABEL                PIC X(40).
           05  CASE-REC-VALUE                PIC X(200).
      *
       FD  DEFI-FILE
           RECORD CONTAINS 92 CHARACTERS
           DATA RECORD IS DEFI-REC.
      *
       01  DEFI-REC                          PIC X(92).
      *
      ******************************************************************
      /
       WORKING-STORAGE SECTION.
      *                              private data
      *                              restricted to local use
      ****************************************************************
      *
       77  DEFI-FILE-NAME                    PIC X(80).
       77  ATTR-FILE-NAME                    PIC X(80).
       77  CASE-FILE-NAME                    PIC X(80).
       77  DEFI-STATUS                       PIC XX.
       77  CASE-STATUS                       PIC XX.
           88  CASE-OK                       VALUE '00'.
           88  CASE-EOF                      VALUE '09'.
       77  ATTR-STATUS                       PIC XX.
           88  ATTR-OK                       VALUE '00'.
           88  ATTR-EOF                      VALUE '09'.
       01  ATTR-STAT                         REDEFINES ATTR-STATUS.
           05  FILLER                        PIC X.
           05  ATTR-STAT-2                   PIC X COMP-X.
       77  CONTINUE-KEY                      PIC X.
      *
      ****************************************************************
       01  INDEXES.
      *                          Index Fields
           05  I                             PIC 99.
           05  J                             PIC 99.
           05  K                             PIC 99.
      *
      ****************************************************************
      *
       01  CONSTANTS.
      *                          Array Lengths
           05  LINE-LNG                      PIC 9999 VALUE 92.
           05  NAME-LNG                      PIC 9999 VALUE 40.
           05  PAGE-MAX                      PIC 99   VALUE 60.
      *                          Array Lengths
      ****************************************************
      *
       01  SWITCHES.
      *                          Switches for Recording States
           05  FOUND-SW                      PIC 9    VALUE 0.
      *
      *
       01  COUNTS.
           05  LINE-COUNT               PIC 99     VALUE ZERO.
           05  DEF-COUNT                PIC 99     VALUE ZERO.
           05  PAGE-COUNT               PIC 9999   VALUE ZERO.
           05  NR-DEFICIENCIES          PIC 9(7)   VALUE ZERO.
           05  WEIGHTED-DEFIS           PIC 9(7)   VALUE ZERO.
      *
       01  LOCAL-VARIABLES.
           05  LAST-TESTCASE-ID         PIC X(20).
           05  CURRENT-RECNR            PIC 9999  VALUE ZEROES.
      *
       01  TESTCASE-ATTRIBUTES.
           05  TESTCASE-ID              PIC X(20).
           05  TESTCASE-TYPE            PIC X(8).
           05  TESTCASE-GOAL            PIC X(60).
           05  TESTCASE-REQUIRE         PIC X(40).
           05  TESTCASE-USECASE         PIC X(40).
           05  TESTCASE-OBJECT          PIC X(40).
           05  TESTCASE-TESTER          PIC X(40).
           05  TESTCASE-PRECON          PIC X(40).
           05  TESTCASE-POSTCON         PIC X(40).
           05  TESTCASE-TRIGGER         PIC X(40).
           05  TESTCASE-STATUS          PIC X(8).
           05  TESTCASE-DATE            PIC X(10).
           05  TESTCASE-AUTO            PIC X(4).
           05  TESTCASE-PRIORITY        PIC X(4).
           05  TESTCASE-SOURCE          PIC X(40).
           05  TESTCASE-PREDECESSOR     PIC X(20).
           05  TESTCASE-ATTRIBUTE       PIC X(20).
      *
      ****************************************************************
      *
       77  ATTR-NR                      PIC 99 VALUE ZERO.
       77  ATTR-MAX                     PIC 99 VALUE 40.
       01  ATTR-TAB.
           05  ATTR-ENTRY               OCCURS 40 TIMES.
               10  ATTR-TYPE            PIC X(4).
               10  ATTR-LABEL           PIC X(40).
      *
       77  CASE-NR                      PIC 99 VALUE ZERO.
       77  CASE-MAX                     PIC 99 VALUE 40.
       01  CASE-TAB.
           05  CASE-ENTRY               OCCURS 40 TIMES.
               10  CASE-LABEL           PIC X(40).
               10  CASE-VALUE           PIC X(60).
      *
       77  TYPE-NR                      PIC 99 VALUE ZERO.
       77  TYPE-MAX                     PIC 99 VALUE 40.
       01  TYPE-TAB.
           05  TYPE-ENTRY               OCCURS 40 TIMES.
               10  TYPE-NAME            PIC X(8).
      *
       77  SRC-NR                       PIC 99 VALUE ZERO.
       77  SRC-MAX                      PIC 99 VALUE 40.
       01  SRC-TAB.
           05  SRC-ENTRY                OCCURS 40 TIMES.
               10  SRC-NAME             PIC X(40).
      *
      ***************************************************************
      * Data for converting lower case characters to upper case
      ***************************************************************
      *
       77  STRING-LNG                    PIC 9(4) BINARY VALUE 256.
       77  COL-I                         PIC 9(4) BINARY.
       01  CHAR-STRING.
           02 PRG-CH                     PIC X OCCURS 256 TIMES.
       01  UPPER-CASE-CHARS              PIC X(26) VALUE
                                        'ABCDEFGHIJKLMNOPQRSTUVWXYZ'.
       01  UPPER-CASE-TAB                REDEFINES UPPER-CASE-CHARS.
           05 UPPER-CHARS               OCCURS 26 TIMES INDEXED BY UP-I.
              10 UPPER-C                 PIC X.
       01  LOWER-CASE-CHARS              PIC X(26) VALUE
                                        'abcdefghijklmnopqrstuvwxyz'.
       01  LOWER-CASE-TAB                REDEFINES LOWER-CASE-CHARS.
           05 LOWER-CHARS               OCCURS 26 TIMES INDEXED BY LO-I.
              10 LOWER-C                 PIC X.
      *
      ****************************************************************
      *   Data for printing out deficiencies
      *
       77  DEFI-MAX                     PIC 99 VALUE 25.
       77  DEFI-INDEX                   PIC 99 VALUE ZERO.
       01  DEFI-TAB.
           05 DEFI-NR   OCCURS 25 TIMES PIC 99.
      *
       01  YYMMDD.
           05  YY                       PIC 99.
           05  MM                       PIC 99.
           05  DD                       PIC 99.

      *
       01  DEF-NEWPAGE-LINE.
           05  FILLER                   PIC X(92)  VALUE '/'.
      *
       01  DEF-SEPARATOR-LINE.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(90)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
      *
       01  DEF-SEPARATOR-LINE-1.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(19)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(06)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
           05  FILLER                   PIC X(63)  VALUE ALL '-'.
           05  FILLER                   PIC X(01)  VALUE '+'.
      *
       01  DEF-BLANK-LINE.
           05  FILLER                   PIC X(01)  VALUE '|'.
           05  FILLER                   PIC X(90)  VALUE SPACES.
           05  FILLER                   PIC X(01)  VALUE '|'.
      *
       01  DEF-SKIP-LINE.
           05  FILLER                   PIC X(12)  VALUE '|         | '.
           05  FILLER                   PIC X(78)  VALUE SPACES.
           05  FILLER                   PIC X(02)  VALUE ' |'.
      *
       01  DEF-REC-HEADER-1.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  FILLER                   PIC X(17)  VALUE SPACES.
           05  DEF-HEADER-1             PIC X(50)  VALUE
ENG           ' T E S T A N A L  A U D I T O R   T E S T C A S E '.
           05  FILLER                   PIC X(17)  VALUE SPACES.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-HEADER-2.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  FILLER                   PIC X(24)  VALUE SPACES.
           05  DEF-HEADER-2             PIC X(35)  VALUE
ENG           'C O M P L I A N C E    R E P O R T '.
           05  FILLER                   PIC X(25)  VALUE SPACES.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-HEADER-3.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  DEF-HEADER-3-1           PIC X(10)  VALUE
ENG           'TESTPROC: '.
           05  FILE-NAME-OUT            PIC X(60)  VALUE SPACES.
           05  DEF-HEADER-3-2           PIC X(6)   VALUE
ENG           'DATE: '.
           05  DATUM-OUT.
               10  DAY-OUT              PIC 99.
               10  FILLER               PIC X VALUE '.'.
               10  MONTH-OUT            PIC 99.
               10  FILLER               PIC X VALUE '.'.
               10  YEAR-OUT             PIC 99.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-HEADER-4.
           05  FILLER                   PIC X(04)  VALUE '|   '.
           05  DEF-HEADER-4-1           PIC X(10)  VALUE
ENG           'SYSTEM:   '.
           05  SYST-NAME-OUT            PIC X(60)  VALUE SPACES.
           05  DEF-HEADER-4-2           PIC X(6)   VALUE
ENG           'PAGE: '.
           05  SEITE-OUT                PIC ZZZZZZZ9.
           05  FILLER                   PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-HEADER-5.
           05  DEF-HEADER-5-1           PIC X(12)  VALUE '|  MODULE  |'.
           05  DEF-HEADER-5-2           PIC X(11)  VALUE ' TESTCASE |'.
           05  DEF-HEADER-5-3           PIC X(67)  VALUE
GER****       '     TESTFALL MANGEL        '.
ENG           '     TESTCASE DEFICIENCY    '.
           05  FILLER                   PIC X(02)  VALUE ' |'.
      *
       01  DEF-REC-LINE.
           05  FILLER                   PIC X(2)   VALUE '| '.
           05  TC-ID-OUT                PIC X(18).
           05  FILLER                   PIC X      VALUE '|'.
           05  TC-TYPE-OUT              PIC X(6).
           05  FILLER                   PIC X      VALUE '|'.
           05  TC-DEF-OUT               PIC X(63).
           05  FILLER                   PIC X      VALUE '|'.
      *
       01  DEF-REC-MSG.
           05  FILLER                   PIC X(2)   VALUE '| '.
           05  DEF-CATG-OUT             PIC X(18).
           05  FILLER                   PIC X      VALUE '|'.
           05  FILLER.
               10  FILLER               PIC X(4)   VALUE 'Msg:'.
               10  DEF-NR-OUT           PIC 99.
           05  FILLER                   PIC X      VALUE '|'.
           05  DEF-TEXT-OUT             PIC X(63).
           05  FILLER                   PIC X      VALUE '|'.
      *
       01  DEF-REC-FOOT-1.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  DEF-FOOT-1       PIC X(34)  VALUE
ENG           'Number of major Rule Violations = '.
           05  MAJOR-DEFS-OUT   PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-FOOT-2.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  DEF-FOOT-2       PIC X(34)  VALUE
ENG           'Number of media Rule Violations = '.
           05  MEDIA-DEFS-OUT   PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-FOOT-3.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  DEF-FOOT-3       PIC X(34)  VALUE
ENG           'Number of minor Rule Violations = '.
           05  MINOR-DEFS-OUT   PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-FOOT-4.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  DEF-FOOT-4       PIC X(34)  VALUE
ENG           'Total Number of Rule Violations = '.
           05  NR-DEFS-OUT      PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-FOOT-5.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  DEF-FOOT-5       PIC X(34)  VALUE
ENG           'Number of Functional Test Cases = '.
           05  NR-CASES-OUT     PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-FOOT-6.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  DEF-FOOT-6       PIC X(34)  VALUE
ENG           'Number of Test Case Attributes  = '.
           05  NR-ATTRS-OUT     PIC Z(5)9  VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
       01  DEF-REC-FOOT-7.
           05  FILLER           PIC X(04)  VALUE '|   '.
           05  DEF-FOOT-7       PIC X(34)  VALUE
ENG           'Rate of Attribute Conformity    = '.
           05  CONFORM-RATE     PIC Z9.999 VALUE ZERO.
           05  FILLER           PIC X(44)  VALUE SPACES.
           05  FILLER           PIC X(04)  VALUE '   |'.
      *
      ****************************************************************
      *    Deficiency Messages (16) for TCS
      *
           COPY TCSMSG.
      *
      ****************************************************************
       LINKAGE SECTION.
      ****************************************************************
       77  SYSTEM-NAME                       PIC X(8).
       77  MODULE-NAME                       PIC X(8).
       77  FILE-NAME                         PIC X(8).
       77  CONCEPT-NAME                      PIC X(40).
       77  OUTPUT-LIB-NAME                   PIC X(80).
       77  FUNC-TAB-NAME                     PIC X(80).
       77  CASE-TAB-NAME                     PIC X(80).
       77  AUDIT-CODE                        PIC XX.
       77  RET-CODE                          PIC 99.
      *
           COPY TMAXPARS.
      *
           COPY TMETTAB.
      *                         Table of key names
       01  KEY-TAB.
           05  TC-NAME                           PIC X(40).
           05  REQU-NAME                         PIC X(40).
           05  UC-NAME                           PIC X(40).
           05  OBJT-NAME                         PIC X(40).
      *
      ****************************************************************
      /
       PROCEDURE DIVISION USING SYSTEM-NAME,
                                FILE-NAME,
                                CONCEPT-NAME,
                                OUTPUT-LIB-NAME,
                                FUNC-TAB-NAME,
                                CASE-TAB-NAME,
                                AUDIT-CODE,
                                RET-CODE,
                                MAX-LIMITS-TAB,
                                METRIC-TAB,
                                KEY-TAB.
      ******************************************************************
      *           M A I N   P R O C E D U R E
      ******************************************************************
      *
       TCSPRUF-MAIN-PROCESSING SECTION.
      *
      ******************************************************************
      *
       START-TCSPRUF.
      *
           DISPLAY '+---------------------------------------------+'
           DISPLAY '| TCSPRUF Check Program Vers 1.0 of Feb. 2006 |'
           DISPLAY '+---------------------------------------------+'
           DISPLAY '| System Name           = ' SYSTEM-NAME
           DISPLAY '| File   Name           = ' FILE-NAME
           DISPLAY '| Test Table Name       = ' CONCEPT-NAME
           DISPLAY '| Audit Code            = ' AUDIT-CODE
           DISPLAY '+---------------------------------------------+'
TEST***    DISPLAY '| Press Enter to continue !!'
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
       READ-ATTRIBUTES.
      *
           MOVE FUNC-TAB-NAME TO ATTR-FILE-NAME
           MOVE ZEROES TO ATTR-STATUS
           OPEN INPUT ATTR-FILE
           IF ATTR-STATUS NOT = ZEROES
              DISPLAY '| Attribute File could not be opened !!'
              DISPLAY '| Attribute File Name = ' ATTR-FILE-NAME
              DISPLAY '| Press Enter to continue !'
              ACCEPT CONTINUE-KEY
              MOVE 21 TO RET-CODE
              GOBACK
           END-IF
TEST***    DISPLAY '| Read in Attribute File !!'
TEST***    ACCEPT CONTINUE-KEY
           MOVE SPACES TO ATTR-TAB
           MOVE ZERO TO ATTR-NR
           PERFORM UNTIL ATTR-STATUS NOT = ZERO
              READ ATTR-FILE
              AT END
                 MOVE '09' TO ATTR-STATUS
              NOT AT END
                 MOVE ATTR-REC TO CHAR-STRING
                 PERFORM TRANSLATE-CHARS
                 MOVE CHAR-STRING TO ATTR-REC
                 ADD 1 TO ATTR-NR
                 MOVE ATTR-TYP TO ATTR-TYPE (ATTR-NR)
                 MOVE ATTR-NAME TO ATTR-LABEL (ATTR-NR)
TEST***          DISPLAY 'CASE-LABEL:' ATTR-TYPE (ATTR-NR)
TEST***          DISPLAY 'CASE-VALUE:' ATTR-LABEL (ATTR-NR)
TEST***          ACCEPT CONTINUE-KEY
              END-READ
           END-PERFORM.
           CLOSE ATTR-FILE.
           DISPLAY '| ' ATTR-NR ' Attributes were read !'
TEST***    ACCEPT CONTINUE-KEY.
           EXIT.
      *
      ******************************************************************
      *
       INIT-DEFI-REPORT.
           MOVE ZEROES TO COUNTS
           MOVE ZEROES TO SWITCHES
           PERFORM OPEN-DEFI-FILE.
           PERFORM WRITE-PAGE-HEADER.
      *
      ******************************************************************
      *
       OPEN-TESTCASE-FILE.
           MOVE CASE-TAB-NAME TO CASE-FILE-NAME
           MOVE ZEROES TO CASE-STATUS
           OPEN INPUT CASE-FILE
           IF CASE-STATUS NOT = ZEROES
              DISPLAY '| TestCase File could not be opened !!'
              DISPLAY '| TestCase File Name = ' CASE-FILE-NAME
              DISPLAY '| Press Enter to continue !'
              ACCEPT CONTINUE-KEY
              MOVE 22 TO RET-CODE
              GOBACK
           END-IF.
      *
      ******************************************************************
      *
       TESTCASE-PROCESSING.
      *
TEST***    DISPLAY 'Read in TestCase File !!'
TEST***    ACCEPT CONTINUE-KEY
           MOVE ZERO TO CURRENT-RECNR
           MOVE ZEROS TO CASE-STATUS
           READ CASE-FILE
           AT END
              MOVE '09' TO CASE-STATUS
           NOT AT END
              MOVE CASE-REC-NR TO CURRENT-RECNR
           END-READ
           PERFORM UNTIL CASE-STATUS NOT = ZERO
              MOVE CASE-REC TO CHAR-STRING
              PERFORM TRANSLATE-CHARS
              MOVE CHAR-STRING TO CASE-REC
              MOVE ZERO TO CASE-NR
              PERFORM UNTIL CASE-REC-NR NOT = CURRENT-RECNR
                         OR CASE-EOF
                 IF CASE-NR < CASE-MAX
                    ADD 1 TO CASE-NR
                    MOVE CASE-REC-LABEL TO CASE-LABEL (CASE-NR)
                    MOVE CASE-REC-VALUE TO CASE-VALUE (CASE-NR)
TEST***             DISPLAY 'CASE-LABEL:' CASE-LABEL (CASE-NR)
TEST***             DISPLAY 'CASE-VALUE:' CASE-VALUE (CASE-NR)
TEST***             ACCEPT CONTINUE-KEY
                 END-IF
                 READ CASE-FILE
                 AT END
                    MOVE '09' TO CASE-STATUS
                 NOT AT END
                    MOVE CASE-REC TO CHAR-STRING
                    PERFORM TRANSLATE-CHARS
                    MOVE CHAR-STRING TO CASE-REC
TEST***             DISPLAY 'This_RecNr: ' CASE-REC-NR
TEST***             DISPLAY 'Current_RecNr: ' CURRENT-RECNR
TEST***             ACCEPT CONTINUE-KEY
                 END-READ
              END-PERFORM
METRIC        ADD 1 TO NR-TEST-CASES
              PERFORM CHECK-ATTRIBUTES
              IF DEFI-INDEX > ZERO AND AUDIT-CODE = 'Y'
                 PERFORM WRITE-DEFI
              END-IF
              MOVE CASE-REC-NR TO CURRENT-RECNR
              DISPLAY '| TestCase Nr. ' CURRENT-RECNR ' ' TESTCASE-ID
TEST***       ACCEPT CONTINUE-KEY
           END-PERFORM.
METRIC     ADD 1 TO NR-TEST-PROCS.
TEST***    DISPLAY '| ' NR-TEST-CASES ' Test Cases were read !'
TEST***    ACCEPT CONTINUE-KEY.
      *
      ******************************************************************
      *
       TESTCASE-LIMITS.
           MOVE ZEROES TO DEFI-TAB
           MOVE ZEROES TO DEFI-INDEX
           IF NR-TEST-CASES > MAX-CASES
              ADD 1 TO DEFI-INDEX
DEFI18        MOVE 18 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
           END-IF.
           IF NR-TEST-ATTRS > MAX-ATTRS
              ADD 1 TO DEFI-INDEX
DEFI19        MOVE 19 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
           END-IF.
           IF NR-ACTIONS > MAX-ACTIONS
              ADD 1 TO DEFI-INDEX
DEFI20        MOVE 20 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
           END-IF.
           IF NR-STATES  > MAX-STATES
              ADD 1 TO DEFI-INDEX
DEFI21        MOVE 21 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
           END-IF.
           IF NR-STATES  > MAX-RULES
              ADD 1 TO DEFI-INDEX
DEFI22        MOVE 22 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
           END-IF.
           IF NR-INPUTS  > MAX-INPUTS
              ADD 1 TO DEFI-INDEX
DEFI23        MOVE 23 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
           END-IF.
           IF NR-OUTPUTS > MAX-OUTPUTS
              ADD 1 TO DEFI-INDEX
DEFI24        MOVE 24 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
           END-IF.
           IF NR-PARAMS > MAX-PARAMS
              ADD 1 TO DEFI-INDEX
DEFI25        MOVE 25 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
           END-IF.
           MOVE CONCEPT-NAME TO TESTCASE-ID
           MOVE 'Proc'       TO TESTCASE-TYPE
           MOVE 'Limits exceeded ' TO TESTCASE-GOAL
           IF DEFI-INDEX > ZERO AND AUDIT-CODE = 'Y'
              PERFORM WRITE-DEFI
           END-IF.
      *
      ******************************************************************
      *
       TESTCASE-CLOSING.
           PERFORM WRITE-DEFI-SUMMARY
           CLOSE DEFI-FILE
           CLOSE CASE-FILE
           DISPLAY '+-----------------------------------------+'
           DISPLAY '| TestCases were checked !!'
           DISPLAY '| Number of TestCases       = ' NR-TEST-CASES
           DISPLAY '| Number of TestCase Types  = ' NR-TEST-CASE-TYPES
           DISPLAY '| Number of automated Cases = ' NR-TEST-CASES-AUTO
           DISPLAY '| Number of Pre Conditions  = ' NR-PRE-CONDS
           DISPLAY '| Number of Post Conditions = ' NR-POST-CONDS
           DISPLAY '| Number of Test Triggers   = ' NR-TEST-TRIGGERS
           DISPLAY '+-----------------------------------------+'
TEST       DISPLAY '| Press Enter to Continue !'
TEST       ACCEPT CONTINUE-KEY.
           GOBACK.
      *
      ******************************************************************
       TESTCASE-CHECKING SECTION.
      ******************************************************************
      *
       CHECK-ATTRIBUTES.
           MOVE SPACES TO TESTCASE-ATTRIBUTES.
           MOVE ZEROES TO DEFI-TAB
           MOVE ZEROES TO DEFI-INDEX
           MOVE 1 TO I
           PERFORM UNTIL I > ATTR-NR
                      OR I > ATTR-MAX
              MOVE 1 TO J
              MOVE ZERO TO FOUND-SW
              PERFORM UNTIL J > CASE-NR
                         OR J > CASE-MAX
                         OR FOUND-SW = 1
                 IF CASE-LABEL (J) = ATTR-LABEL (I)
                    MOVE 1 TO FOUND-SW
                 ELSE
                    ADD 1 TO J
                 END-IF
              END-PERFORM
              IF FOUND-SW = 1
                 EVALUATE ATTR-TYPE (I)
                    WHEN 'ID '
                       IF CASE-VALUE (J) = TESTCASE-ID
                          ADD  1 TO DEFI-INDEX
DEFI01                    MOVE 01 TO DEFI-NR (DEFI-INDEX)
                       END-IF
                       MOVE CASE-VALUE (J) TO TESTCASE-ID
                       MOVE CASE-LABEL (J) TO TC-NAME
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'TYPE'
                       MOVE CASE-VALUE (J) TO TESTCASE-TYPE
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'REQU'
                       MOVE CASE-VALUE (J) TO TESTCASE-REQUIRE
                       MOVE CASE-LABEL (J) TO REQU-NAME
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'CASE'
                       MOVE CASE-VALUE (J) TO TESTCASE-USECASE
                       MOVE CASE-LABEL (J) TO UC-NAME
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'OBJT'
                       MOVE CASE-VALUE (J) TO TESTCASE-OBJECT
                       MOVE CASE-LABEL (J) TO OBJT-NAME
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'TEST'
                       MOVE CASE-VALUE (J) TO TESTCASE-TESTER
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'PRE '
                       MOVE CASE-VALUE (J) TO TESTCASE-PRECON
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'POST'
                       MOVE CASE-VALUE (J) TO TESTCASE-POSTCON
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'TRIG'
                       MOVE CASE-VALUE (J) TO TESTCASE-TRIGGER
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'STAT'
                       MOVE CASE-VALUE (J) TO TESTCASE-STATUS
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'DATE'
                       MOVE CASE-VALUE (J) TO TESTCASE-DATE
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'AUTO'
                       MOVE CASE-VALUE (J) TO TESTCASE-AUTO
METRIC                 ADD 1 TO NR-TEST-ATTRS
METRIC                 ADD 1 TO NR-TEST-CASES-AUTO
                    WHEN 'GOAL'
                       MOVE CASE-VALUE (J) TO TESTCASE-GOAL
METRIC                 ADD 1 TO NR-TEST-ATTRS
METRIC                 ADD 1 TO NR-TEST-CASE-GOALS
070321              WHEN 'PRIO'
070321                 MOVE CASE-VALUE (J) TO TESTCASE-PRIORITY
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'SRC '
                       MOVE CASE-VALUE (J) TO TESTCASE-SOURCE
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'PRED'
                       MOVE CASE-VALUE (J) TO TESTCASE-PREDECESSOR
METRIC                 ADD 1 TO NR-TEST-ATTRS
                    WHEN 'INPT'
METRIC                 ADD 1 TO NR-INPUTS
                    WHEN 'OUTP'
METRIC                 ADD 1 TO NR-OUTPUTS
                    WHEN OTHER
                       MOVE CASE-VALUE (J) TO TESTCASE-ATTRIBUTE
METRIC                 ADD 1 TO NR-TEST-ATTRS
                 END-EVALUATE
              END-IF
              ADD 1 TO I
           END-PERFORM.
           IF TESTCASE-ID (1:2) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI01        MOVE 01 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MAJOR-DEFIS
           END-IF.
           IF TESTCASE-TYPE (1:2) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI02        MOVE 02 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MAJOR-DEFIS
           ELSE
              PERFORM CHECK-CASE-TYPE
              IF FOUND-SW = ZERO
METRIC           ADD 1 TO NR-TEST-CASE-TYPES
              END-IF
              EVALUATE TESTCASE-TYPE
                 WHEN 'ACTION'
                 WHEN 'action'
METRIC              ADD 1 TO NR-ACTIONS
                 WHEN 'RULE  '
                 WHEN 'rule  '
METRIC              ADD 1 TO NR-RULES
                 WHEN 'STATE '
                 WHEN 'state '
METRIC              ADD 1 TO NR-STATES
                 WHEN OTHER
METRIC              ADD 1 TO NR-OTHERS
              END-EVALUATE
           END-IF.
           IF TESTCASE-REQUIRE (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI03        MOVE 03 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MAJOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-REFS
           END-IF.
           IF TESTCASE-USECASE (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI04        MOVE 04 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MAJOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-REFS
           END-IF.
           IF TESTCASE-OBJECT (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI05        MOVE 05 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MAJOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-REFS
           END-IF.
           IF TESTCASE-TESTER (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI06        MOVE 06 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MAJOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-REFS
           END-IF.
           IF TESTCASE-PRECON (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI07        MOVE 07 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MEDIA-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           ELSE
METRIC        ADD 1 TO NR-PRE-CONDS
           END-IF.
           IF TESTCASE-POSTCON (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI08        MOVE 08 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MEDIA-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           ELSE
METRIC        ADD 1 TO NR-POST-CONDS
           END-IF.
           IF TESTCASE-TRIGGER (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI09        MOVE 09 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MEDIA-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           ELSE
METRIC        ADD 1 TO NR-TEST-TRIGGERS
           END-IF.
           IF TESTCASE-STATUS (1:2) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI10        MOVE 10 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MEDIA-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           END-IF.
           IF TESTCASE-DATE (1:2) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI11        MOVE 11 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MEDIA-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           END-IF.
           IF TESTCASE-AUTO (1:2) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI12        MOVE 12 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MEDIA-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           ELSE
METRIC        ADD 1 TO NR-TEST-CASES-AUTO
           END-IF.
           IF TESTCASE-GOAL (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI13        MOVE 13 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           END-IF.
           IF TESTCASE-SOURCE (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI14        MOVE 14 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           ELSE
              PERFORM CHECK-CASE-SRC
              IF FOUND-SW = ZERO
METRIC           ADD 1 TO NR-TEST-CASE-SRCS
              END-IF
           END-IF.
           IF TESTCASE-PREDECESSOR (3:4) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI15        MOVE 15 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           END-IF.
           IF TESTCASE-PRIORITY (1:2) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI13        MOVE 16 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           END-IF.
           IF TESTCASE-ATTRIBUTE (1:2) = SPACES
              ADD 1 TO DEFI-INDEX
DEFI16        MOVE 17 TO DEFI-NR (DEFI-INDEX)
METRIC        ADD 1 TO NR-MINOR-DEFIS
METRIC        ADD 1 TO NR-MISSING-ATTRS
           END-IF.
           EXIT.
      *
       CHECK-CASE-TYPE.
TEST***    DISPLAY 'Check TestCase Type:' TESTCASE-TYPE
TEST***    ACCEPT CONTINUE-KEY
           MOVE ZERO TO FOUND-SW
           MOVE 1 TO K
           PERFORM UNTIL K > TYPE-NR
                      OR K > TYPE-MAX
                      OR FOUND-SW = 1
              IF TESTCASE-TYPE = TYPE-NAME (K)
                 MOVE 1 TO FOUND-SW
              ELSE
                 ADD 1 TO K
              END-IF
           END-PERFORM.
           IF FOUND-SW = ZERO
              IF TYPE-NR < TYPE-MAX
                 ADD 1 TO TYPE-NR
                 MOVE TESTCASE-TYPE TO TYPE-NAME (TYPE-NR)
              END-IF
           END-IF.
           EXIT.
      *
       CHECK-CASE-SRC.
           MOVE ZERO TO FOUND-SW
           MOVE 1 TO K
           PERFORM UNTIL K > SRC-NR
                      OR K > SRC-MAX
                      OR FOUND-SW = 1
              IF TESTCASE-SOURCE = SRC-NAME (K)
                 MOVE 1 TO FOUND-SW
              ELSE
                 ADD 1 TO K
              END-IF
           END-PERFORM.
           IF FOUND-SW = ZERO
              IF SRC-NR < SRC-MAX
                 ADD 1 TO SRC-NR
                 MOVE TESTCASE-SOURCE TO SRC-NAME (SRC-NR)
              END-IF
           END-IF.
           EXIT.
      *
      ***************************************************************
      * Procedure for converting lower case characters to upper case
      ***************************************************************
      *
       TRANSLATE-CHARS.
           MOVE 1 TO COL-I
           PERFORM  UNTIL COL-I > STRING-LNG
              EVALUATE TRUE
                WHEN PRG-CH (COL-I) < 'a' OR PRG-CH (COL-I) > 'z'
                   ADD 1 TO COL-I
                WHEN OTHER
                   SET LO-I TO 1
                   SEARCH LOWER-CHARS VARYING LO-I
                     WHEN LOWER-C (LO-I) = PRG-CH (COL-I)
                       SET UP-I TO LO-I
                   END-SEARCH
                   MOVE UPPER-C (UP-I) TO PRG-CH (COL-I)
                   ADD 1 TO COL-I
               END-EVALUATE
           END-PERFORM.
           EXIT.
      *
      ******************************************************************
      ******************************************************************
       DEFICIENCY-REPORTING SECTION.
      ******************************************************************
      *
       OPEN-DEFI-FILE.
      *
      ******************************************************************
      * Open the file DEF-FILE as output
      ******************************************************************
      *
           MOVE SPACES TO DEFI-FILE-NAME
           STRING OUTPUT-LIB-NAME    DELIMITED BY SPACE
                 '\'                 DELIMITED BY SIZE
                  FILE-NAME          DELIMITED BY SPACE
                 '.def'              DELIMITED BY SIZE
             INTO DEFI-FILE-NAME.
           OPEN OUTPUT DEFI-FILE.
TEST***    DISPLAY '| Test Case Deficiency Report in : ' DEFI-FILE-NAME
TEST***    ACCEPT CONTINUE-KEY.
      *
      *  initialization of report.
      *
           MOVE ZERO  TO LINE-COUNT
           MOVE ZERO  TO PAGE-COUNT
           ACCEPT YYMMDD FROM DATE
           MOVE YY TO YEAR-OUT
           MOVE MM TO MONTH-OUT
           MOVE DD TO DAY-OUT.
           WRITE DEFI-REC FROM DEF-NEWPAGE-LINE.
      *
      ******************************************************************
      *
       WRITE-DEFI-SUMMARY.
      *
      ************************************************************
      *  Write the summary of the listed deficiencies
      ************************************************************
      *
           MOVE NR-MAJOR-DEFIS  TO MAJOR-DEFS-OUT
           MOVE NR-MEDIA-DEFIS  TO MEDIA-DEFS-OUT
           MOVE NR-MINOR-DEFIS  TO MINOR-DEFS-OUT
           COMPUTE NR-DEFICIENCIES = NR-MAJOR-DEFIS
                                   + NR-MEDIA-DEFIS
                                   + NR-MINOR-DEFIS
           MOVE NR-DEFICIENCIES TO NR-DEFS-OUT
           MOVE NR-TEST-CASES   TO NR-CASES-OUT
           MOVE NR-TEST-ATTRS   TO NR-ATTRS-OUT
      *
           COMPUTE WEIGHTED-DEFIS = (NR-MAJOR-DEFIS * 1.5)
                                  + (NR-MEDIA-DEFIS * 1)
                                  + (NR-MINOR-DEFIS * 0.5)
           IF WEIGHTED-DEFIS > NR-TEST-ATTRS
              COMPUTE CONFORM-RATE  = 0.001
           ELSE
             COMPUTE CONFORM-RATE = 1 - (WEIGHTED-DEFIS / NR-TEST-ATTRS)
           END-IF.
      *
           WRITE DEFI-REC FROM DEF-SEPARATOR-LINE-1
           WRITE DEFI-REC FROM DEF-BLANK-LINE
           WRITE DEFI-REC FROM DEF-REC-FOOT-1
           WRITE DEFI-REC FROM DEF-REC-FOOT-2
           WRITE DEFI-REC FROM DEF-REC-FOOT-3
           WRITE DEFI-REC FROM DEF-REC-FOOT-4
           WRITE DEFI-REC FROM DEF-REC-FOOT-5
           WRITE DEFI-REC FROM DEF-REC-FOOT-6
           WRITE DEFI-REC FROM DEF-BLANK-LINE.
           WRITE DEFI-REC FROM DEF-REC-FOOT-7
           WRITE DEFI-REC FROM DEF-BLANK-LINE.
           WRITE DEFI-REC FROM DEF-SEPARATOR-LINE.
      *
      *
      ******************************************************************
      *
       WRITE-DEFI.
      *
      ******************************************************************
      * Write the first/next record DEFI-REC to the file DEFI-FILE
      * - Fill up the record DEFI-REC with the message informations
      * - Write   the record DEFI-REC to the file DEFI-FILE
      ******************************************************************
      *  Write all deficiencies found in this statement
      *
TEST***    DISPLAY '| Write out Deficiencies for this Test Case'
TEST***    ACCEPT CONTINUE-KEY
      *
           IF DEFI-INDEX > ZERO
              MOVE TESTCASE-ID   TO TC-ID-OUT
              MOVE TESTCASE-TYPE TO TC-TYPE-OUT
              MOVE TESTCASE-GOAL TO TC-DEF-OUT
              WRITE DEFI-REC FROM DEF-REC-LINE
              ADD 1 TO LINE-COUNT
              MOVE 1 TO J
              PERFORM UNTIL J > DEFI-INDEX
                         OR J > DEFI-MAX
                 MOVE SPACES TO DEF-CATG-OUT
                 IF DEFI-NR (J) < 7
                    MOVE 'Problem:'  TO  DEF-CATG-OUT
                 ELSE
                    IF DEFI-NR (J) < 13
                       MOVE 'Missing:'  TO  DEF-CATG-OUT
                    ELSE
                       MOVE 'Warning:'  TO  DEF-CATG-OUT
                    END-IF
                 END-IF
      *
                 MOVE DEFI-NR (J) TO K
                 MOVE DEFI-NR (J) TO DEF-NR-OUT
                 MOVE DEFI-MSG (K) TO DEF-TEXT-OUT
                 WRITE DEFI-REC  FROM DEF-REC-MSG
                 ADD 1 TO LINE-COUNT
                 ADD 1 TO J
              END-PERFORM
           END-IF.
           EXIT.
      *
      ******************************************************************
      *
       WRITE-PAGE-HEADER.
      *
      *    Initialize Report Header
      *
           ADD  1             TO PAGE-COUNT
           MOVE PAGE-COUNT    TO SEITE-OUT
           MOVE SYSTEM-NAME   TO SYST-NAME-OUT.
           MOVE CONCEPT-NAME  TO FILE-NAME-OUT
      *
      ******************************************************************
      *  Execute Page Change and Write Page Header
      ******************************************************************
      *
           WRITE DEFI-REC FROM DEF-SEPARATOR-LINE
           WRITE DEFI-REC FROM DEF-REC-HEADER-1
           WRITE DEFI-REC FROM DEF-REC-HEADER-2
           WRITE DEFI-REC FROM DEF-REC-HEADER-3
           WRITE DEFI-REC FROM DEF-REC-HEADER-4.
           WRITE DEFI-REC FROM DEF-SEPARATOR-LINE-1.
      *
           MOVE 6 TO LINE-COUNT.
      *
      ********************** end of source *****************************
